#include "stdafx.h"
#include "emule.h"
#include "BarShader.h"
#include "math.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifndef M_PI_2
#define M_PI_2     1.57079632679489661923
#endif

#ifndef PI
#define PI 3.14159265358979323846264338328
#endif

#define HALF(X) (((X) + 1) / 2)

CBarShader::CBarShader(uint32 dwHeight, uint32 dwWidth, COLORREF crColor /*= 0*/, uint32 dwFileSize /*= 1*/)
{
	m_iWidth = dwWidth;
	m_iHeight = dwHeight;
	m_dwFileSize = 0;
	m_Spans.SetAt(0, crColor);
	m_Spans.SetAt(dwFileSize, 0);
	m_pdblModifiers = NULL;
	m_bIsPreview = false;
	m_used3dlevel = 0;
	SetFileSize(dwFileSize);
}

CBarShader::~CBarShader(void)
{
	delete[] m_pdblModifiers;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBarShader::BuildModifiers()
{
	if (m_pdblModifiers != NULL)
	{
		delete[] m_pdblModifiers;
		m_pdblModifiers = NULL; // 'new' may throw an exception
	}

	if (!m_bIsPreview) 
		m_used3dlevel = g_eMuleApp.m_pGlobPrefs->Get3DDepth();

	// New property page slider to control depth of gradient

	// Depth must be at least 2
	// 2 gives greatest depth, the higher the value, the flatter the appearance
	// m_pdblModifiers[count-1] will always be 1, m_pdblModifiers[0] depends on the value of depth
	static const double dDepths[5] = { 5.5, 4.0, 3.0, 2.50, 2.25 };		//aqua bar - smoother gradient jumps...
	double	depth = dDepths[((m_used3dlevel > 5) ? (256 - m_used3dlevel) : m_used3dlevel) - 1];
	int count = HALF(m_iHeight);
	double piOverDepth = PI / depth;
	double base = PI / 2 - piOverDepth;
	double increment = piOverDepth / (count - 1);

	m_pdblModifiers = new double[count];
	for (int i = 0; i < count; i++, base += increment)
		m_pdblModifiers[i] = sin(base);
}

void CBarShader::SetWidth(int width)
{
	if(m_iWidth != width) {
		m_iWidth = width;
		if (m_dwFileSize)
			m_dblPixelsPerByte = (double)m_iWidth / m_dwFileSize;
		else
			m_dblPixelsPerByte = 0.0;
		if (m_iWidth)
			m_dblBytesPerPixel = (double)m_dwFileSize / m_iWidth;
		else
			m_dblBytesPerPixel = 0.0;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBarShader::SetFileSize(uint32 dwFileSize)
{
	if (m_dwFileSize != dwFileSize)
	{
		m_dwFileSize = dwFileSize;
		if (m_dwFileSize)
			m_dblPixelsPerByte = static_cast<double>(m_iWidth) / m_dwFileSize;
		else
			m_dblPixelsPerByte = 0.0;
		if (m_iWidth)
			m_dblBytesPerPixel = static_cast<double>(m_dwFileSize) / m_iWidth;
		else
			m_dblBytesPerPixel = 0.0;
	}
}

void CBarShader::SetHeight(int height)
{
	if(m_iHeight != height)
 	{
		m_iHeight = height;

		BuildModifiers();
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBarShader::FillRange(uint32 start, uint32 end, COLORREF crColor)
{
	if(end > m_dwFileSize)
		end = m_dwFileSize;

	if(start >= end)
		return;
	POSITION endprev, endpos = m_Spans.FindFirstKeyAfter(end+1);

	if ((endprev = endpos) != NULL)
		m_Spans.GetPrev(endpos);
	else
		endpos = m_Spans.GetTailPosition();

	ASSERT(endpos != NULL);

	COLORREF endcolor = m_Spans.GetValueAt(endpos);

	if ((endcolor == crColor) && (m_Spans.GetKeyAt(endpos) <= end) && (endprev != NULL))
		endpos = endprev;
	else
		endpos = m_Spans.SetAt(end, endcolor);

	for (POSITION pos = m_Spans.FindFirstKeyAfter(start); pos != endpos; )
	{
		POSITION pos1 = pos;
		m_Spans.GetNext(pos);
		m_Spans.RemoveAt(pos1);
	}
	
	m_Spans.GetPrev(endpos);

	if ((endpos == NULL) || (m_Spans.GetValueAt(endpos) != crColor))
		m_Spans.SetAt(start, crColor);
}

void CBarShader::Fill(COLORREF crColor)
{
	m_Spans.RemoveAll();
	m_Spans.SetAt(0, crColor);
	m_Spans.SetAt(m_dwFileSize, 0);
}

void CBarShader::DrawPreview(CDC* dc, int iLeft, int iTop, byte previewLevel)		//Cax2 aqua bar
{
	m_bIsPreview = true;
	m_used3dlevel = previewLevel;
	BuildModifiers();
	Draw(dc, iLeft, iTop, (previewLevel == 0));
	m_bIsPreview = false;
}

void CBarShader::Draw(CDC* dc, int iLeft, int iTop, bool bFlat)
{
	COLORREF crLastColor = ~0, crPrevBkColor = dc->GetBkColor();
	POSITION pos = m_Spans.GetHeadPosition();
	RECT rectSpan;
	rectSpan.top = iTop;
	rectSpan.bottom = iTop + m_iHeight;
	rectSpan.right = iLeft;

	int iBytesInOnePixel = static_cast<int>(m_dblBytesPerPixel + 0.5);
	uint32 start = 0;
	COLORREF crColor = m_Spans.GetNextValue(pos);

	iLeft += m_iWidth;
	while ((pos != NULL) && (rectSpan.right < iLeft)) {
		uint32 uSpan = m_Spans.GetKeyAt(pos) - start;
		int iPixels = static_cast<int>(uSpan * m_dblPixelsPerByte + 0.5);

		if(iPixels > 0) {
			rectSpan.left = rectSpan.right;
			rectSpan.right += iPixels;
			FillRect(dc, &rectSpan, crLastColor = crColor, bFlat);

			start += static_cast<int>(iPixels * m_dblBytesPerPixel + 0.5);
		} else {
			double dRed = 0, dGreen = 0, dBlue = 0;
			uint32 dwRed, dwGreen, dwBlue, dwLast = start, dwEnd = start + iBytesInOnePixel;

			do {
				double	dblWeight = (min(m_Spans.GetKeyAt(pos), dwEnd) - dwLast) * m_dblPixelsPerByte;
				dRed   += GetRValue(crColor) * dblWeight;
				dGreen += GetGValue(crColor) * dblWeight;
				dBlue  += GetBValue(crColor) * dblWeight;
				if ((dwLast = m_Spans.GetKeyAt(pos)) >= dwEnd)
					break;
				crColor = m_Spans.GetValueAt(pos);
				m_Spans.GetNext(pos);
			} while(pos != NULL);
			rectSpan.left = rectSpan.right;
			rectSpan.right++;

		//	Saturation
			dwRed = static_cast<uint32>(dRed);
			if (dwRed > 255)
				dwRed = 255;
			dwGreen = static_cast<uint32>(dGreen);
			if (dwGreen > 255)
				dwGreen = 255;
			dwBlue = static_cast<uint32>(dBlue);
			if (dwBlue > 255)
				dwBlue = 255;

			FillRect(dc, &rectSpan, crLastColor = RGB(dwRed, dwGreen, dwBlue), bFlat);
			start += iBytesInOnePixel;
		}
		while((pos != NULL) && (m_Spans.GetKeyAt(pos) <= start))
			crColor = m_Spans.GetNextValue(pos);
	}
	if ((rectSpan.right < iLeft) && (crLastColor != ~0))
	{
		rectSpan.left = rectSpan.right;
		rectSpan.right = iLeft;
		FillRect(dc, &rectSpan, crLastColor, bFlat);
	}
	dc->SetBkColor(crPrevBkColor);	//restore previous background color
}

void CBarShader::FillRect(CDC *dc, LPCRECT rectSpan, COLORREF crColor, bool bFlat)
{
	if(!crColor || bFlat)
		dc->FillSolidRect(rectSpan, crColor);
	else
	{
		if ((m_used3dlevel != g_eMuleApp.m_pGlobPrefs->Get3DDepth() && !m_bIsPreview) || m_pdblModifiers == NULL)
			BuildModifiers();

		double	dblRed = GetRValue(crColor), dblGreen = GetGValue(crColor), dblBlue = GetBValue(crColor);
		double	dAdd, dMod;

		if (m_used3dlevel > 5)		//Cax2 aqua bar
		{
			dMod = 1.0 - .025 * (256 - m_used3dlevel);		//variable central darkness - from 97.5% to 87.5% of the original colour...
			dAdd = 255;

			dblRed = dMod * dblRed - dAdd;
			dblGreen = dMod * dblGreen - dAdd;
			dblBlue = dMod * dblBlue - dAdd;
		}
		else
			dAdd = 0;

		RECT		rect;
		int			iTop = rectSpan->top, iBot = rectSpan->bottom;
		double		*pdCurr = m_pdblModifiers, *pdLimit = pdCurr + HALF(m_iHeight);

		rect.right = rectSpan->right;
		rect.left = rectSpan->left;

		for (; pdCurr < pdLimit; pdCurr++)
		{
			crColor = RGB( static_cast<int>(dAdd + dblRed * *pdCurr),
				static_cast<int>(dAdd + dblGreen * *pdCurr),
				static_cast<int>(dAdd + dblBlue * *pdCurr) );
			rect.top = iTop++;
			rect.bottom = iTop;
			dc->FillSolidRect(&rect, crColor);

			rect.bottom = iBot--;
			rect.top = iBot;
		//	Fast way to fill, background color is already set inside previous FillSolidRect
			dc->ExtTextOut(0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
