//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


/******************************************************************
 *                      Based on eMule Morph                      *
 ******************************************************************/


#include "StdAfx.h"
#include "fakecheck.h"
#include "emule.h"
#include "otherfunctions.h"
#include "ED2KLink.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/////////////////////////////////////////////////////////////////////////////////////////////
CFakecheck::CFakecheck() : m_sLastHit(_T(""))
{
}

/////////////////////////////////////////////////////////////////////////////////////////////
CFakecheck::~CFakecheck()
{
	RemoveAllFakes();
}

/////////////////////////////////////////////////////////////////////////////////////////////
void CFakecheck::AddFake(CString Hash, uint32 Lenght, CString Realtitle)
{
	EMULE_TRY

	FakeMap::iterator it = m_mapFakes.find(Hash);

	if (it == m_mapFakes.end())
	{
		Fakes_Struct* newFilter = new Fakes_Struct();

		newFilter->Hash = Hash;
		newFilter->Lenght = Lenght;
		newFilter->RealTitle = Realtitle;
		m_mapFakes[Hash] = newFilter;
	}

	EMULE_CATCH
}

/////////////////////////////////////////////////////////////////////////////////////////////
INT CFakecheck::LoadFromDatFile()
{
	EMULE_TRY

	CString sbuffer, sbuffer2, sbuffer3, sbuffer4;
	int pos, fakecounter;
	CString Hash, Title;
	uint32 Lenght;
	char buffer[1024];
	int lenBuf = 1024;
	fakecounter = 0;
	RemoveAllFakes();
	FILE* readFile = fopen(CString(g_eMuleApp.m_pGlobPrefs->GetConfigDir()) + "fakes.dat", "r");
	if (readFile != NULL)
	{
		while (!feof(readFile))
		{
			if (fgets(buffer, lenBuf, readFile) == 0)
				break;
			sbuffer = buffer;
			if (sbuffer.GetAt(0) == '#' || sbuffer.GetAt(0) == '/' || sbuffer.GetLength() < 5)
				continue;
			pos = sbuffer.Find(',');
			if (pos == -1)
				continue;
			Hash = sbuffer.Left(pos).Trim();
			Hash.MakeUpper();
			int pos2 = sbuffer.Find(',', pos + 1);
			if (pos2 == -1)
				continue;
			sbuffer2 = sbuffer.Mid(pos + 1, pos2 - pos - 1).Trim();
			Lenght = _tstoi(sbuffer2);
			sbuffer2 = sbuffer.Mid(pos2 + 1, sbuffer.GetLength() - pos2 - 2);
			Title = sbuffer2;
			AddFake(Hash, Lenght, Title);
			fakecounter++;
		}
		fclose(readFile);
		g_eMuleApp.AddDebugLogLine(_T("Loaded %i fakes from fakes list"), fakecounter);
		return fakecounter;
	}

	EMULE_CATCH

	return 0;
}

/////////////////////////////////////////////////////////////////////////////////////////////
void CFakecheck::RemoveAllFakes()
{
	EMULE_TRY

	Fakes_Struct* search;

	map<CString, Fakes_Struct*>::const_iterator it;
	for (it = m_mapFakes.begin(); it != m_mapFakes.end(); ++it)
	{
		search = (*it).second;
		delete search;
	}

	m_mapFakes.clear();

	EMULE_CATCH
}

/////////////////////////////////////////////////////////////////////////////////////////////
CString CFakecheck::GetFakeCheckComment(const CString &Hash2test, uint32 length)
{
	EMULE_TRY

	if (m_mapFakes.size() != 0)
	{
		Fakes_Struct* search;
		FakeMap::const_iterator it = m_mapFakes.upper_bound(Hash2test);
		it--;

		do
		{
			search = (*it).second;

			if ((search->Hash == Hash2test) && (search->Lenght == length))
			{
				m_sLastHit = search->RealTitle;
				return m_sLastHit;
			}

			it--;
		}
		while (it != m_mapFakes.begin());
	}

	EMULE_CATCH

	return CString(_T(""));
}

/////////////////////////////////////////////////////////////////////////////////////////////
BOOL CFakecheck::UpdateFakeList()
{
	EMULE_TRY

	uint32	dwVersion = 0;
	CString	strED2KLink;

	if (!RetrieveFakesDotTxt(dwVersion, strED2KLink))
	{
		g_eMuleApp.AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_FAKE_LATEST_VERSION_ERROR));
		g_eMuleApp.AddDebugLogLine(_T("Failed to download fakes.txt!"));
		return FALSE;
	}

	g_eMuleApp.AddDebugLogLine( _T("Latest fake list version found: v%u, Your version: v%u"), 
								dwVersion, g_eMuleApp.m_pGlobPrefs->GetFakesDatVersion() );

	if (g_eMuleApp.m_pGlobPrefs->GetFakesDatVersion() >= dwVersion)
	{
		g_eMuleApp.AddLogLine(true, GetResString(IDS_FAKE_GOT_LATEST_VERSION), g_eMuleApp.m_pGlobPrefs->GetFakesDatVersion());
		return FALSE;
	}

	bool bUpdated = false;

	if (!strED2KLink.IsEmpty())
	{
		CED2KLink* pLink = CED2KLink::CreateLinkFromUrl(strED2KLink);

		if (pLink != NULL)
		{
			CED2KFileLink* pFileLink = pLink->GetFileLink();

			if (pFileLink != NULL)
			{
				if (!g_eMuleApp.m_pDownloadQueue->FileExists(pFileLink->GetHashKey()))
				{
					g_eMuleApp.m_pDownloadQueue->AddFileLinkToDownload(pFileLink);

					CPartFile* pFakesDotRarFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(pFileLink->GetHashKey());

					if (pFakesDotRarFile != NULL)
					{
						pFakesDotRarFile->SetFakesDotRar();
						pFakesDotRarFile->ResumeFile();
						g_eMuleApp.m_pGlobPrefs->SetDLingFakeListVersion(dwVersion);
						g_eMuleApp.m_pGlobPrefs->SetDLingFakeListLink(strED2KLink);
						bUpdated = true;
					}

					if (!bUpdated)
					{
						g_eMuleApp.AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_FAKE_CHECKUPERROR));
					}
				}
			}
		}

		delete pLink;
	}

	return bUpdated;

	EMULE_CATCH

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////////////////////
BOOL CFakecheck::ExtractRARArchive(CString strArchivePath, CString strDestFolder)
{
	EMULE_TRY

	HINSTANCE hUnRARDLL = LoadLibrary(_T("unrar.dll"));

	if (hUnRARDLL == NULL)
	{
		g_eMuleApp.AddDebugLogLine(_T("%s: Can't find or load Unrar.dll"), __FUNCTION__);
		return FALSE;
	}

	g_eMuleApp.AddDebugLogLine(_T("%s: Unrar.dll loaded"), __FUNCTION__);

	(FARPROC&)pfnRAROpenArchiveEx	= GetProcAddress(hUnRARDLL, _T("RAROpenArchiveEx"));
	(FARPROC&)pfnRARCloseArchive	= GetProcAddress(hUnRARDLL, _T("RARCloseArchive"));
	(FARPROC&)pfnRARReadHeader		= GetProcAddress(hUnRARDLL, _T("RARReadHeader"));
	(FARPROC&)pfnRARProcessFile		= GetProcAddress(hUnRARDLL, _T("RARProcessFile"));

	if ( (pfnRAROpenArchiveEx == NULL) || (pfnRARCloseArchive == NULL) ||
		 (pfnRARReadHeader    == NULL) || (pfnRARProcessFile  == NULL) )
	{
		FreeLibrary(hUnRARDLL);
		g_eMuleApp.AddDebugLogLine(_T("%s: Unrar.dll unloaded"), __FUNCTION__);
		return FALSE;
	}

	struct RAROpenArchiveDataEx OpenArchiveData;

	memzero(&OpenArchiveData, sizeof(OpenArchiveData));
	OpenArchiveData.ArcName		= strArchivePath.GetBuffer();
	OpenArchiveData.CmtBuf		= NULL;
	OpenArchiveData.OpenMode	= RAR_OM_EXTRACT;

	HANDLE hArcData = pfnRAROpenArchiveEx(&OpenArchiveData);

	if (OpenArchiveData.OpenResult != 0)
	{
		OutOpenArchiveError(OpenArchiveData.OpenResult, strArchivePath);
		FreeLibrary(hUnRARDLL);
		g_eMuleApp.AddDebugLogLine(_T("%s: Unrar.dll unloaded"), __FUNCTION__);
		return FALSE;
	}

	struct RARHeaderData HeaderData;
	HeaderData.CmtBuf = NULL;

	int iReadHeaderCode;
	while ((iReadHeaderCode = pfnRARReadHeader(hArcData, &HeaderData)) == 0)
	{
	//	The following line is important to support characters such as '', '', '',
	//	', '', '', ... to decompress fakes.dat in the correct directory.
		strDestFolder.AnsiToOem();

		int iProcessFileCode = pfnRARProcessFile(hArcData, RAR_EXTRACT, strDestFolder.GetBuffer(), NULL);

		if (iProcessFileCode != 0)
		{
			OutProcessFileError(iProcessFileCode);
			break;
		}
	}

	pfnRARCloseArchive(hArcData);

	FreeLibrary(hUnRARDLL);
	g_eMuleApp.AddDebugLogLine(_T("%s: Unrar.dll unloaded"), __FUNCTION__);

	if (iReadHeaderCode == ERAR_BAD_DATA)
	{
		return FALSE;
	}

	return TRUE;

	EMULE_CATCH

	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////////////////////
void CFakecheck::OutOpenArchiveError(int iError, const CString& strArchivePath)
{
	EMULE_TRY

	switch(iError)
	{
		case ERAR_NO_MEMORY:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: Not enough memory to extract %s"), __FUNCTION__, strArchivePath);
			break;
		}
		case ERAR_EOPEN:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: Cannot open %s"), __FUNCTION__, strArchivePath);
			break;
		}
		case ERAR_BAD_ARCHIVE:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: %s is not RAR archive"), __FUNCTION__, strArchivePath);
			break;
		}
		case ERAR_BAD_DATA:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: %s: archive header broken"), __FUNCTION__, strArchivePath);
			break;
		}
		case ERAR_UNKNOWN:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: Unknown error when extracting %s"), __FUNCTION__, strArchivePath);
			break;
		}
	}

	EMULE_CATCH
}

/////////////////////////////////////////////////////////////////////////////////////////////
void CFakecheck::OutProcessFileError(int iError)
{
	EMULE_TRY

	switch(iError)
	{
		case ERAR_UNKNOWN_FORMAT:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Unknown archive format"), __FUNCTION__);
			break;
		}
		case ERAR_BAD_ARCHIVE:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Bad volume"), __FUNCTION__);
			break;
		}
		case ERAR_ECREATE:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: File create error"), __FUNCTION__);
			break;
		}
		case ERAR_EOPEN:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Volume open error"), __FUNCTION__);
			break;
		}
		case ERAR_ECLOSE:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: File close error"), __FUNCTION__);
			break;
		}
		case ERAR_EREAD:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Read error"), __FUNCTION__);
			break;
		}
		case ERAR_EWRITE:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Write error"), __FUNCTION__);
			break;
		}
		case ERAR_BAD_DATA:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: CRC error"), __FUNCTION__);
			break;
		}
		case ERAR_UNKNOWN:
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: fakes.rar: Unknown error"), __FUNCTION__);
			break;
		}
	}

	EMULE_CATCH
}
/////////////////////////////////////////////////////////////////////////////////////////////
BOOL CFakecheck::RetrieveFakesDotTxt(OUT uint32& rdwVersion, OUT CString& rstrED2KLink)
{
	EMULE_TRY

	CString strURL = g_eMuleApp.m_pGlobPrefs->GetFakeListURL();
	bool bSuccess = false;

	HINTERNET hOpen = ::InternetOpen(_T("eMule"), INTERNET_OPEN_TYPE_PRECONFIG , NULL, NULL, 0);

	if(hOpen == NULL)
	{
		g_eMuleApp.AddDebugLogLine(_T("%s: Error opening connection (error code : %u"), __FUNCTION__, GetLastError());
		bSuccess = false;
	}
	else
	{
		HINTERNET hURL = ::InternetOpenUrl(hOpen, strURL, _T(""), NULL, INTERNET_FLAG_NO_CACHE_WRITE, NULL);

		if (hURL == NULL)
		{
			g_eMuleApp.AddDebugLogLine(_T("%s: Error opening URL %s (error code : %u"), __FUNCTION__, strURL, GetLastError());
			bSuccess = false;
		}
		else
		{
			CString strFile;
			DWORD dwSize;

			if(!::InternetReadFile(hURL, strFile.GetBuffer(256), 256, &dwSize))
			{
				g_eMuleApp.AddDebugLogLine(_T("%s: Error downloading fakes.txt (error code : %u"), __FUNCTION__, strURL, GetLastError());
				bSuccess = false;
			}
			else
			{
				strFile.ReleaseBuffer(dwSize);

				int iCurPos = 0;
				CString strVersion;
				CString strED2KLink;

				strVersion = strFile.Tokenize(_T("\n"), iCurPos);
				strED2KLink	= strFile.Tokenize(_T("\n"), iCurPos);
				strVersion.Trim();
				strED2KLink.Trim();

				rdwVersion = _tstoi(strVersion);
				rstrED2KLink = URLDecode(strED2KLink);

				if (rdwVersion > 0)
				{
					bSuccess = true;
				}
			}
			::InternetCloseHandle(hURL);
		}
		::InternetCloseHandle(hOpen);
	}

	return bSuccess;

	EMULE_CATCH

	return FALSE;
}
/////////////////////////////////////////////////////////////////////////////////////////////
void CFakecheck::Init()
{
	EMULE_TRY

	bool bNeedToLoad = true;

	if (g_eMuleApp.m_pGlobPrefs->IsUpdateFakeStartupEnabled())
	{
		bNeedToLoad = !g_eMuleApp.m_pFakeCheck->UpdateFakeList();
	}
	if (bNeedToLoad)
	{
		LoadFromDatFile();
	}

	if (!g_eMuleApp.m_pGlobPrefs->GetDLingFakeListLink().IsEmpty())
	{
		CED2KLink* pLink = CED2KLink::CreateLinkFromUrl(g_eMuleApp.m_pGlobPrefs->GetDLingFakeListLink());

		if (pLink != NULL)
		{
			CED2KFileLink* pFileLink = pLink->GetFileLink();

			if (pFileLink != NULL)
			{
				CPartFile* pFakesDotRarFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(pFileLink->GetHashKey());

				if (pFakesDotRarFile != NULL)
				{
					pFakesDotRarFile->SetFakesDotRar();
				}
			}
		}

		delete pLink;
	}

	EMULE_CATCH
}
/////////////////////////////////////////////////////////////////////////////////////////////
