//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


//	SearchListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "SearchListCtrl.h"
#include "otherfunctions.h"
#include "FakeCheck.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

//	CSearchListCtrl

IMPLEMENT_DYNAMIC(CSearchListCtrl, CMuleListCtrl)

BEGIN_MESSAGE_MAP(CSearchListCtrl, CMuleListCtrl)
	ON_NOTIFY_REFLECT (NM_CUSTOMDRAW, OnCustomdraw)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchListCtrl::CSearchListCtrl()
{
	m_pSearchList = NULL;
	m_dwResultsID = 0;
	m_bSetImageList = false;
	SetGeneralPurposeFind(true);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::Init(CSearchList *pSearchList)
{
	static const uint16 s_auColHdr[][2] =
	{
		{ LVCFMT_LEFT,  250 },	//SL_COLUMN_FILENAME
		{ LVCFMT_RIGHT, 100 },	//SL_COLUMN_SIZE
		{ LVCFMT_RIGHT,  50 },	//SL_COLUMN_SOURCES
		{ LVCFMT_LEFT,   65 },	//SL_COLUMN_TYPE
		{ LVCFMT_LEFT,  220 },	//SL_COLUMN_FILEHASH
		{ LVCFMT_LEFT,  220 },	//SL_COLUMN_FAKECHECK
		{ LVCFMT_LEFT,  130 }	//SL_COLUMN_LASTSEENCOMPLETE
	};

	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	ModifyStyle(LVS_SINGLESEL, 0);

	m_pSearchList = pSearchList;

	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_auColHdr); ui++)
		InsertColumn(ui, _T(""), static_cast<int>(s_auColHdr[ui][0]), static_cast<int>(s_auColHdr[ui][1]), ui);

	memzero(&m_bSortAscending, sizeof(m_bSortAscending));
	LoadSettings(CPreferences::TABLE_SEARCH);
	if (g_eMuleApp.m_pGlobPrefs->DoUseSort())
	{
		m_dwLastSortCode = g_eMuleApp.m_pGlobPrefs->GetSearchSortCol();
		SortInit(g_eMuleApp.m_pGlobPrefs->GetSearchSortCol());
	}
	else
	{
		int		iSortCode = g_eMuleApp.m_pGlobPrefs->GetColumnSortItem(CPreferences::TABLE_SEARCH);

		iSortCode |= g_eMuleApp.m_pGlobPrefs->GetColumnSortAscending(CPreferences::TABLE_SEARCH) ? MLC_SORTASC : MLC_SORTDESC;
		m_dwLastSortCode = iSortCode;
		SortInit(iSortCode);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchListCtrl::~CSearchListCtrl()
{
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::SortInit(int iSortCode)
{
//	Get the sort column
	int	iSortColumn = (iSortCode & MLC_COLUMNMASK);
//	Get the sort order
	bool	bSortAscending = (iSortCode & MLC_SORTDESC) == 0;

	SetSortArrow(iSortColumn, bSortAscending);
	SortItems(SortProc, iSortCode);
	m_bSortAscending[iSortColumn] = bSortAscending;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::Localize()
{
	static const uint16 s_auResTbl[] =
	{
		IDS_DL_FILENAME,
		IDS_DL_SIZE,
		IDS_DL_SOURCES,
		IDS_TYPE,
		IDS_FILEHASH,
		IDS_FAKE_CHECK_HEADER,
		IDS_LASTSEENCOMPLETE
	};

	if (GetSafeHwnd())
	{
		CHeaderCtrl	*pHeaderCtrl = GetHeaderCtrl();
		CString		strRes;
		HDITEM		hdi;

		hdi.mask = HDI_TEXT;

		for (unsigned ui = 0; ui < ELEMENT_COUNT(s_auResTbl); ui++)
		{
			::GetResString(&strRes, static_cast<UINT>(s_auResTbl[ui]));
			hdi.pszText = (LPSTR)strRes.GetString();
			pHeaderCtrl->SetItem(static_cast<int>(ui), &hdi);
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::AddResult(CSearchFile *pSearchFile)
{
	EMULE_TRY

//	If there's at least one search tab...
	if (g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.GetItemCount() > 0)
	{
	//	Update tab-iWebServiceCounter for the given search pSearchFile
		CString		strName, strSource;
		int			j;
		TCHAR		tstrText[510];

		TCITEM		tabItem;

		tabItem.mask = TCIF_PARAM;

		int		iTabCounter;

		for (iTabCounter = 0; iTabCounter < g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.GetItemCount(); iTabCounter++)
		{
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.GetItem(iTabCounter, &tabItem);
			if (tabItem.lParam == pSearchFile->GetSearchID())
				break;
		}

		if (tabItem.lParam == pSearchFile->GetSearchID())
		{
			tabItem.pszText = tstrText;
			tabItem.cchTextMax = 510;
			tabItem.mask = TCIF_TEXT;
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.GetItem(iTabCounter, &tabItem);

			strName.Format(_T("%s"), tstrText);

			tabItem.mask = TCIF_PARAM;
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.GetItem(iTabCounter, &tabItem);

			strSource.Format(_T(" (%u)"), m_pSearchList->GetFoundFiles(pSearchFile->GetSearchID()));

			j = strName.ReverseFind(_T('('));
			if (j > 0)
				strName.Truncate(j - 1);
			strName.Append(strSource);

			tabItem.mask = TCIF_TEXT;
			tabItem.pszText = strName.LockBuffer();
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchTabs.SetItem(iTabCounter, &tabItem);
			strName.UnlockBuffer();
		}
	}

	if (pSearchFile->GetSearchID() != m_dwResultsID)
		return;

	uint32		dwIndex = GetItemCount();

	if (g_eMuleApp.m_pGlobPrefs->ShowFileTypeIcon())
	{
		int		iImage = g_eMuleApp.GetFileTypeSystemImageIdx(pSearchFile->GetFileName());

		if (!m_bSetImageList && g_eMuleApp.GetSystemImageList() != NULL)
		{
		//	ASSERT( GetStyle() & LVS_SHAREIMAGELISTS );
			ApplyImageList(g_eMuleApp.GetSystemImageList());
			m_bSetImageList = true;
		}
		dwIndex = InsertItem(LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE, dwIndex, pSearchFile->GetFileName(), 0, 0, iImage, (LPARAM)pSearchFile);
	}
	else
	{
		ApplyImageList(NULL);
		m_bSetImageList = false;
		dwIndex = InsertItem(LVIF_TEXT | LVIF_PARAM, dwIndex, pSearchFile->GetFileName(), 0, 0, 1, (LPARAM)pSearchFile);
	}

	CString		strTmp;

	UpdateLine(dwIndex, pSearchFile);

	SetItemText(dwIndex, SL_COLUMN_SIZE, CastItoXBytes(pSearchFile->GetFileSize()));
	SetItemText(dwIndex, SL_COLUMN_TYPE, pSearchFile->GetFileTypeString(true));
	strTmp = HashToString(pSearchFile->GetFileHash());
	SetItemText(dwIndex, SL_COLUMN_FILEHASH, strTmp);
 //	FakeCheck
	if (IsColumnHidden(SL_COLUMN_FAKECHECK))
		SetItemText(dwIndex, SL_COLUMN_FAKECHECK, _T(""));
	else
		SetItemText(dwIndex, SL_COLUMN_FAKECHECK, g_eMuleApp.m_pFakeCheck->GetFakeCheckComment(strTmp, pSearchFile->GetFileSize()));

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::UpdateChangingColumns(CSearchFile *pSearchFile)
{
	EMULE_TRY

	LVFINDINFO		find;

	find.flags = LVFI_PARAM;
	find.lParam = reinterpret_cast<LPARAM>(pSearchFile);

	int		iIndex = FindItem(&find);

	if (iIndex != -1)
	{
		UpdateLine(iIndex, pSearchFile);
		Update(iIndex);
		RedrawItems(iIndex, iIndex);
		SortInit(m_dwLastSortCode);
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::UpdateLine(int iIndex, CSearchFile *pSearchFile)
{
	CString		strTmp;
	uint32		dwLastSeenVal;

	strTmp.Format(_T("%u (%u)"), pSearchFile->GetSourceCount(), pSearchFile->GetCompleteSourceCount());
	SetItemText(iIndex, SL_COLUMN_SOURCES, strTmp);

	if ((dwLastSeenVal = pSearchFile->GetLastSeenCompleteValue()) == 0)
		SetItemText(iIndex, SL_COLUMN_LASTSEENCOMPLETE, _T(""));	//currently available
	else
	{
		if (dwLastSeenVal == 0x7FFFFFFF)
			GetResString(&strTmp, IDS_NEVER);	//complete file wasn't seen
		else
		{
			COleDateTime lastSeenTime = COleDateTime::GetCurrentTime() - COleDateTimeSpan(0, 0, 0, dwLastSeenVal - 1);

			strTmp = lastSeenTime.Format();
		}
		SetItemText(iIndex, SL_COLUMN_LASTSEENCOMPLETE, strTmp);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::RemoveResult(CSearchFile *pSearchFile)
{
	EMULE_TRY

	LVFINDINFO		find;

	find.flags = LVFI_PARAM;
	find.lParam = reinterpret_cast<LPARAM>(pSearchFile);

	sint32		lIndex = FindItem(&find);

	if (lIndex != -1)
		DeleteItem(lIndex);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::ShowResults(uint32 dwResultsID)
{
	EMULE_TRY

	DeleteAllItems();
	m_dwResultsID = dwResultsID;
	if (m_pSearchList != NULL)
		m_pSearchList->ShowResults(m_dwResultsID);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::OnCustomdraw(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	NMLVCUSTOMDRAW		*pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>(pNMHDR);

	*pResult = CDRF_DODEFAULT;

	if (CDDS_PREPAINT == pLVCD->nmcd.dwDrawStage)
	{
		*pResult = CDRF_NOTIFYITEMDRAW;
	}
	else if (CDDS_ITEMPREPAINT == pLVCD->nmcd.dwDrawStage)
	{
		COLORREF	crText = ::GetSysColor(COLOR_WINDOWTEXT);
		uint32		dwRed = GetRValue(crText);
		uint32		dwGreen = GetGValue(crText);
		uint32		dwBlue = GetGValue(crText);

		CSearchFile		*pSearchFile = reinterpret_cast<CSearchFile*>(pLVCD->nmcd.lItemlParam);

		if (pSearchFile == NULL)
			return ;

		CKnownFile		*pKnownFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(pSearchFile->GetFileHash());

	//	If it's not a completed shared file, check to see if it's a part file
		if (pKnownFile == NULL)
		{
			pKnownFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(pSearchFile->GetFileHash());
		}
		if (pKnownFile != NULL)
		{
		//	Currently downloading (-> red)
			if (pKnownFile->IsPartFile())
			{
				dwRed = (pSearchFile->GetSourceCount() + 4) * 20;
				if (dwRed > 255)
					dwRed = 255;
			}
			else
			{
			//	Already downloaded and currently shared (-> green)
				dwGreen = 128;
			}
		}
		else
		{
			pKnownFile = g_eMuleApp.m_pKnownFilesList->FindKnownFileByID(pSearchFile->GetFileHash());

		//	Known but currently not shared (-> Orange)
			if  (pKnownFile != NULL)
			{
				dwRed = 255;
				dwGreen = 165;
				dwBlue = 0;
			}
		//	Not known at all (-> indicate availability in shades of blue)
			else
			{
				dwBlue = (pSearchFile->GetSourceCount() - 1) * 20;
				if (dwBlue > 255)
				{
					dwBlue = 255;
				}
			}
		}
		pLVCD->clrText = RGB(dwRed, dwGreen , dwBlue);

		*pResult = CDRF_DODEFAULT;
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::OnColumnClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	NM_LISTVIEW	*pNMListView = reinterpret_cast<NM_LISTVIEW*>(pNMHDR);
	int			iSubItem = pNMListView->iSubItem;
	bool		bSortOrder = m_bSortAscending[iSubItem];

// Reverse sorting direction for the same column and keep the same if column was changed
	if ((m_dwParamSort & MLC_COLUMNMASK) == iSubItem)
		m_bSortAscending[iSubItem] = bSortOrder = !bSortOrder;

	SetSortArrow(iSubItem, bSortOrder);
	m_dwLastSortCode = iSubItem | ((bSortOrder) ? MLC_SORTASC : MLC_SORTDESC);
	SortItems(SortProc, m_dwLastSortCode);
	g_eMuleApp.m_pGlobPrefs->SetColumnSortItem(CPreferences::TABLE_SEARCH, iSubItem);
	g_eMuleApp.m_pGlobPrefs->SetColumnSortAscending(CPreferences::TABLE_SEARCH, bSortOrder);

	*pResult = 0;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CSearchListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	EMULE_TRY
	CSearchFile		*pSearchFile1 = reinterpret_cast<CSearchFile*>(lParam1);
	CSearchFile		*pSearchFile2 = reinterpret_cast<CSearchFile*>(lParam2);

	//eklmn: don't sort
	if (pSearchFile1 == NULL || pSearchFile2 == NULL)
		return 0;

	int				iCompare = 0;
	int				iSortMod = ((lParamSort & MLC_SORTDESC) == 0) ? 1 : -1;
	int				iSortCol = lParamSort & MLC_COLUMNMASK;

	switch (iSortCol)
	{
		case SL_COLUMN_FILENAME:
		{
			iCompare = _tcsicmp(pSearchFile1->GetFileName(), pSearchFile2->GetFileName()) ;
			break;
		}
		case SL_COLUMN_SIZE:
		{
			iCompare = CompareUnsigned(pSearchFile1->GetIntTagValue(FT_FILESIZE), pSearchFile2->GetIntTagValue(FT_FILESIZE));
			break;
		}
		case SL_COLUMN_SOURCES:
		{
			iCompare = pSearchFile1->GetSourceCount() - pSearchFile2->GetSourceCount();
			break;
		}
		case SL_COLUMN_TYPE:
		{
			iCompare = _tcsicmp(pSearchFile1->GetFileTypeString(true), pSearchFile2->GetFileTypeString(true));
			if (iCompare == 0)
			{
				iCompare = SortProc(lParam1,lParam2, SL_COLUMN_FILENAME);
				iSortMod = 1;		//sort always in ascending order
			}
			break;
		}
		case SL_COLUMN_FILEHASH:
		{
			iCompare = memcmp(pSearchFile1->GetFileHash(), pSearchFile2->GetFileHash(), 16);
			break;
		}
		case SL_COLUMN_FAKECHECK:
		{
			CString	strFake1 = g_eMuleApp.m_pFakeCheck->GetFakeCheckComment(HashToString(pSearchFile1->GetFileHash()), pSearchFile1->GetFileSize());
			CString	strFake2 = g_eMuleApp.m_pFakeCheck->GetFakeCheckComment(HashToString(pSearchFile2->GetFileHash()), pSearchFile2->GetFileSize());

			iCompare = ((strFake1 > strFake2) ? 1 : ((strFake1 < strFake2) ? -1 : 0));
			break;
		}
		case SL_COLUMN_LASTSEENCOMPLETE:
			iCompare = pSearchFile1->GetLastSeenCompleteValue() - pSearchFile2->GetLastSeenCompleteValue();
			if (iCompare == 0)
			{
				iCompare = SortProc(lParam1, lParam2, SL_COLUMN_SOURCES);
				iSortMod = -1;		//in descending order
			}
			break;
		default:
		{
			return 0;
		}
	}
	return iCompare * iSortMod;

	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchListCtrl::OnContextMenu(CWnd *pWnd, CPoint point)
{
	EMULE_TRY

	CTitleMenu	menuSearchFile;
	CMenu		menuED2K, menuWebServices;

	menuSearchFile.CreatePopupMenu();
	menuSearchFile.AddMenuTitle(GetResString(IDS_FILE));

	int			iSelectionMark = GetSelectionMark();
	UINT		dwSelectedCount = GetSelectedCount();
	UINT		dwMenuFlags = MF_STRING | (((iSelectionMark != -1) && (dwSelectedCount > 0)) ? MF_ENABLED : MF_GRAYED);

	menuSearchFile.AppendMenu(dwMenuFlags, MP_RESUME, GetResString(IDS_DOWNLOAD_VERB));
	menuSearchFile.AppendMenu(dwMenuFlags, MP_RESUMEPAUSED, GetResString(IDS_DOWNLOAD_VERB) + _T(" (") + GetResString(IDS_PAUSED) + _T(")"));

	menuSearchFile.SetDefaultItem(
		(g_eMuleApp.m_pGlobPrefs->StartDownloadPaused()) ? MP_RESUMEPAUSED : MP_RESUME );

	menuSearchFile.AppendMenu(MF_SEPARATOR);

	menuSearchFile.AppendMenu(dwMenuFlags, MP_REMOVESELECTED, GetResString(IDS_REMOVESELECTED));
	menuSearchFile.AppendMenu(dwMenuFlags, MP_REMOVE, GetResString(IDS_REMOVESEARCHSTRING));
	menuSearchFile.AppendMenu(dwMenuFlags, MP_REMOVEALL, GetResString(IDS_REMOVEALLSEARCH));

	menuSearchFile.AppendMenu(MF_SEPARATOR);

	menuED2K.CreateMenu();
	menuED2K.AppendMenu(MF_STRING, MP_GETED2KLINK, GetResString(IDS_DL_LINK1));
	menuED2K.AppendMenu(MF_STRING, MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2));
	menuSearchFile.AppendMenu(dwMenuFlags | MF_POPUP, (UINT_PTR)menuED2K.m_hMenu, GetResString(IDS_ED2KLINKFIX));

	int		iWebServiceCounter;

	menuWebServices.CreateMenu();
	UpdateURLMenu(menuWebServices, iWebServiceCounter);

	dwMenuFlags |= ( ( (iWebServiceCounter == 0) ||
		((iSelectionMark != -1) && (dwSelectedCount > 1)) ) ? MF_GRAYED : 0 );

	menuSearchFile.AppendMenu(dwMenuFlags | MF_POPUP, (UINT_PTR)menuWebServices.m_hMenu, GetResString(IDS_WEBSERVICES));

	menuSearchFile.TrackPopupMenuEx(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this, NULL);

//	Menu objects are destroyed in their destructor

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CSearchListCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	EMULE_TRY

	if (GetSelectionMark() != ( -1))
	{
		CSearchFile		*pSearchFile = reinterpret_cast<CSearchFile*>(GetItemData(GetSelectionMark()));

		if (wParam >= MP_WEBURL && wParam <= MP_WEBURL + 64)
		{
			RunURL(pSearchFile, g_eMuleApp.m_strWebServiceURLArray.GetAt(wParam - MP_WEBURL));
		}
	//	For multiple selections
		UINT		dwNumSelectedFiles = GetSelectedCount();

		CTypedPtrList<CPtrList, CSearchFile*> selectedList;

		int			iIndex = -1;
		POSITION	pos = GetFirstSelectedItemPosition();

		while (pos != NULL)
		{
			iIndex = GetNextSelectedItem(pos);
			if (iIndex > -1)
			{
				selectedList.AddTail(reinterpret_cast<CSearchFile*>(GetItemData(iIndex)));
			}
		}

		switch (wParam)
		{
			case MP_GETED2KLINK:
			{
				CString		str;

				while (!selectedList.IsEmpty())
				{
					str += selectedList.GetHead()->CreateED2kLink();
					selectedList.RemoveHead();
					if (!selectedList.IsEmpty())
						str += "\r\n";
				}
				g_eMuleApp.CopyTextToClipboard(str);
				break;
			}
			case MP_GETHTMLED2KLINK:
			{
				CString		str;

				while (!selectedList.IsEmpty())
				{
					str += selectedList.GetHead()->CreateHTMLED2kLink();
					selectedList.RemoveHead();
					if (!selectedList.IsEmpty())
						str += "\r\n";
				}
				g_eMuleApp.CopyTextToClipboard(str);
				break;
			}
			case MP_RESUMEPAUSED:
			case MP_RESUME:
			{
				g_eMuleApp.m_pdlgEmule->m_dlgSearch.DownloadSelected(wParam == MP_RESUMEPAUSED);
				break;
			}
			case MP_REMOVEALL:
			{
				g_eMuleApp.m_pdlgEmule->m_dlgSearch.DeleteAllSearchs();
				break;
			}
			case MP_REMOVESELECTED:
			{
				SetRedraw(false);
				while (!selectedList.IsEmpty())
				{
					g_eMuleApp.m_pSearchList->RemoveResults(selectedList.GetHead());
					selectedList.RemoveHead();
				}
				SetRedraw(true);
				break;
			}
		}
		selectedList.RemoveAll();
	}
	switch (wParam)
	{
		case MP_REMOVE:
		{
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.DeleteSearch(m_dwResultsID);
			break;
		}
	}
	EMULE_CATCH
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
