//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "packets.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::Packet(EnumProtocols eProtocol)
{
	m_bSplit = false;
	m_bLastSplit = false;
	m_dwSize = 0;
	m_pcBuffer = NULL;
	m_pcCompleteBuffer = NULL;
	m_pcTempBuffer = NULL;
	m_eProtocol = eProtocol;
	m_bPacked = false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::Packet(PacketHeader_Struct *pHeaderStruct)
{
	m_bSplit = false;
	m_bPacked = false;
	m_bLastSplit = false;
	m_pcTempBuffer = NULL;
	m_pcBuffer = NULL;
	m_pcCompleteBuffer = NULL;
	m_dwSize = pHeaderStruct->m_lPacketLength-1;
	m_eOpcode = pHeaderStruct->m_eCommand;
	m_eProtocol = pHeaderStruct->m_eEDonkeyProtocol;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::Packet(char *pcPacketPart, uint32 dwSize, bool bLast, byte byteFilePriority, bool bFromPF)
{
// -khaos--+++> Slightly modified for our stats uses...
	m_byteFilePriority = byteFilePriority;
	m_bFromPF = bFromPF;
	m_bSplit = true;
	m_bPacked = false;
	m_bLastSplit = bLast;
	m_pcTempBuffer = NULL;
	m_pcBuffer = NULL;
	m_pcCompleteBuffer = pcPacketPart;
	m_dwSize = dwSize-sizeof(PacketHeader_Struct);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Packet(EnumOpCodes,uint32,eProtocol,bool) constructs a Packet with the spec'd opcode, data size, and protocol.
//		'bFromPF' is only used to set the "from PartFile" state of the Packet. If 'm_bFromPF' = false then this
//		packet was formed from a complete shared file rather than a part file.
//		It is used only in the UploadClient to track transferred data stats.
Packet::Packet(EnumOpcodes eOpcode,uint32 dwSize/*=0*/,EnumProtocols eProtocol/*=OP_EDONKEYPROT*/,bool bFromPF/*=true*/)
{
	m_bFromPF = bFromPF;
	m_bSplit = false;
	m_bPacked = false;
	m_bLastSplit = false;
	m_pcTempBuffer = NULL;
	if (dwSize)
	{
		m_pcCompleteBuffer = new char[dwSize + 10];
		m_pcBuffer = m_pcCompleteBuffer+6;
		memzero(m_pcCompleteBuffer, dwSize + 10);
	}
	else
	{
		m_pcBuffer = 0;
		m_pcCompleteBuffer = 0;
	}
	m_eOpcode = eOpcode;
	m_dwSize = dwSize;
	m_eProtocol = eProtocol;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::Packet(EnumOpcodes eOpcode, uint32 dwSize, EnumProtocols eProtocol, byte byteFilePriority, bool bFromPF)
{
	m_byteFilePriority = byteFilePriority;
	m_bFromPF = bFromPF;
	m_bSplit = false;
	m_bPacked = false;
	m_bLastSplit = false;
	m_pcTempBuffer = NULL;
	if (dwSize)
	{
		m_pcCompleteBuffer = new char[dwSize + 10];
		m_pcBuffer = m_pcCompleteBuffer+6;
		memzero(m_pcCompleteBuffer, dwSize + 10);
	}
	else
	{
		m_pcBuffer = 0;
		m_pcCompleteBuffer = 0;
	}
	m_eOpcode = eOpcode;
	m_dwSize = dwSize;
	m_eProtocol = eProtocol;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::Packet(CMemFile *pDataFile,EnumProtocols eProtocol)
{
	m_bSplit = false;
	m_bPacked = false;
	m_bLastSplit = false;
	m_dwSize = pDataFile->GetLength();
	m_pcCompleteBuffer = new char[pDataFile->GetLength()+10];	//why 10?
	m_pcBuffer = m_pcCompleteBuffer+sizeof(PacketHeader_Struct);

	BYTE	*pbyteBuffer = pDataFile->Detach();

	memcpy2(m_pcBuffer,pbyteBuffer,m_dwSize);
	free(pbyteBuffer);
	m_pcTempBuffer = NULL;
	m_eProtocol = eProtocol;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet::~Packet()
{
	if (m_pcCompleteBuffer != NULL)
		delete[] m_pcCompleteBuffer;
	else if (m_pcBuffer != NULL)
		delete [] m_pcBuffer;
	if (m_pcTempBuffer != NULL)
		delete[] m_pcTempBuffer;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
char *Packet::GetPacket()
{
	if (m_pcCompleteBuffer != NULL)
	{
		if (!m_bSplit)
			memcpy2(m_pcCompleteBuffer,GetHeader(),sizeof(PacketHeader_Struct));

		return m_pcCompleteBuffer;
	}
	else
	{
		if (m_pcTempBuffer != NULL)
			delete[] m_pcTempBuffer;
		m_pcTempBuffer = new char[m_dwSize+10];	// Why 10?
		memcpy2(m_pcTempBuffer,GetHeader(),sizeof(PacketHeader_Struct));
		memcpy2(m_pcTempBuffer+sizeof(PacketHeader_Struct),m_pcBuffer,m_dwSize);

		return m_pcTempBuffer;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
char *Packet::DetachPacket()
{
	if (m_pcCompleteBuffer != NULL)
	{
		if (!m_bSplit)
			memcpy2(m_pcCompleteBuffer,GetHeader(),sizeof(PacketHeader_Struct));

		char	*pcResult = m_pcCompleteBuffer;

		m_pcCompleteBuffer = NULL;
		m_pcBuffer = NULL;

		return pcResult;
	}
	else
	{
		if (m_pcTempBuffer != NULL)
			delete[] m_pcTempBuffer;

		m_pcTempBuffer = new char[m_dwSize+10];	// Why 10?
		memcpy2(m_pcTempBuffer,GetHeader(),sizeof(PacketHeader_Struct));
		memcpy2(m_pcTempBuffer+sizeof(PacketHeader_Struct),m_pcBuffer,m_dwSize);

		char	*pcResult = m_pcTempBuffer;

		m_pcTempBuffer = NULL;
		return pcResult;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
char *Packet::GetHeader()
{
	ASSERT(!m_bSplit);

	PacketHeader_Struct		*pHeaderStruct = reinterpret_cast<PacketHeader_Struct*>(m_arrcHead);

	pHeaderStruct->m_eCommand = m_eOpcode;
	pHeaderStruct->m_eEDonkeyProtocol =  m_eProtocol;
	pHeaderStruct->m_lPacketLength = m_dwSize+1;
	return m_arrcHead;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
char *Packet::GetUDPHeader()
{
	ASSERT(!m_bSplit);
	memzero(m_arrcHead, sizeof(PacketHeader_Struct));

	UDP_Header_Struct	*pUDPHeader = reinterpret_cast<UDP_Header_Struct*>(m_arrcHead);

	pUDPHeader->m_eCommand = m_eOpcode;
	pUDPHeader->m_eEDonkeyProtocol =  m_eProtocol;

	return m_arrcHead;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void Packet::PackPacket()
{
	ASSERT(!m_bSplit);

	BYTE		*pbyteOutput = new BYTE[m_dwSize+300];

	uLongf		dwNewSize = m_dwSize+300;
//	Zip 'm_pcBuffer' to 'pbyteOutput'.
	uint16		nResult = compress2(pbyteOutput,&dwNewSize,reinterpret_cast<BYTE*>(m_pcBuffer),m_dwSize,9);

//	If the compressed buffer is actually larger than the original...
	if (nResult != Z_OK || m_dwSize <= dwNewSize)
	{
		delete[] pbyteOutput;
		return;
	}
	m_eProtocol = OP_PACKEDPROT;
	memcpy2(m_pcBuffer,pbyteOutput,dwNewSize);
	delete[] pbyteOutput;
	m_bPacked = true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool Packet::UnpackPacket(uint32 dwMaxUnpackedSize /*= 0*/)
{
	ASSERT(m_eProtocol == OP_PACKEDPROT); 

	if (dwMaxUnpackedSize == 0)
		dwMaxUnpackedSize = m_dwSize*10+300;
	
	//uint32		nNewSize = m_dwSize*10+300;

	BYTE		*pbyteUnpackBuffer = new BYTE[dwMaxUnpackedSize];

	uLongf		dwUnpackedSize = dwMaxUnpackedSize;
	uint16		nResult = uncompress(pbyteUnpackBuffer,&dwUnpackedSize,reinterpret_cast<BYTE*>(m_pcBuffer),m_dwSize);

	if (nResult == Z_OK)
	{
		ASSERT(m_pcCompleteBuffer == NULL);
		ASSERT(m_pcBuffer != NULL);
		m_dwSize = dwUnpackedSize;
		delete[] m_pcBuffer;
		m_pcBuffer = reinterpret_cast<char*>(pbyteUnpackBuffer);
		m_eProtocol =  OP_EMULEPROT;
		return true;				
	}
	delete[] pbyteUnpackBuffer;

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CTag

CTag::CTag(LPCTSTR strName, uint32 dwValue)
{
	m_fltValue = 0.0f;
	m_strName = strName;
	m_byteTagType = TAGTYPE_UINT32;
	m_dwValue = dwValue;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CTag::CTag(byte byteTagID, uint32 dwValue)
{
	m_fltValue = 0.0f;
	m_byteTagType = TAGTYPE_UINT32;
	m_dwValue = dwValue;
	m_byteTagID = byteTagID;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CTag::CTag(LPCTSTR strName, LPCTSTR strValue)
{
	m_dwValue = 0;
	m_fltValue = 0.0f;
	m_strName = strName;
	m_byteTagType = TAGTYPE_STRING;
	m_strValue = strValue;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CTag::CTag(byte byteTagID, LPCTSTR strValue)
{
	m_dwValue = 0;
	m_fltValue = 0.0f;	
	m_byteTagType = TAGTYPE_STRING;
	m_strValue = strValue;
	m_byteTagID = byteTagID;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Construct a tag from a CFile stream.
CTag::CTag(CFile& file)
{
	uint32	dwLength;
	uint16	uTmp;

	m_dwValue = 0;
	m_fltValue = 0.0f;

	file.Read(&m_byteTagType, 1);
	if (m_byteTagType & 0x80)
	{
		m_byteTagType &= 0x7F;
		file.Read(&m_byteTagID, 1);
	}
	else
	{
		file.Read(&uTmp, 2);
	//	Read the tag identifier; Either a special tag code or a tag name
		if ((dwLength = uTmp) == 1)
			file.Read(&m_byteTagID, 1);
		else
		{
			m_byteTagID = 0;
			file.Read(m_strName.GetBuffer(dwLength), dwLength);
			m_strName.ReleaseBuffer(dwLength);
		}
	}

//	NOTE: It's very important that we read the *entire* packet data, even if we do not use each tag
	if (m_byteTagType == TAGTYPE_STRING)
	{
		file.Read(&uTmp, 2);
		dwLength = uTmp;
		file.Read(m_strValue.GetBuffer(dwLength), dwLength);
		m_strValue.ReleaseBuffer(dwLength);
	}
	else if (m_byteTagType == TAGTYPE_UINT32)
	{
		file.Read(&m_dwValue, 4);
	}
	else if (m_byteTagType == TAGTYPE_UINT16)
	{
		file.Read(&uTmp, 2);
		m_dwValue = uTmp;
		m_byteTagType = TAGTYPE_UINT32;
	}
	else if (m_byteTagType == TAGTYPE_UINT8)
	{
		byte	byteOctet;

		file.Read(&byteOctet, 1);
		m_dwValue = byteOctet;
		m_byteTagType = TAGTYPE_UINT32;
	}
	else if ((m_byteTagType >= TAGTYPE_STR1) && (m_byteTagType <= TAGTYPE_STR16))
	{
		dwLength = m_byteTagType - TAGTYPE_STR1 + 1;
		file.Read(m_strValue.GetBuffer(dwLength), dwLength);
		m_strValue.ReleaseBuffer(dwLength);
		m_byteTagType = TAGTYPE_STRING;
	}
	else if (m_byteTagType == TAGTYPE_FLOAT32) // (used by Hybrid 0.48)
	{
		file.Read(&m_fltValue, 4);
	}
	else if (m_byteTagType == TAGTYPE_HASH)
	{
		file.Seek(16, CFile::current);
	}
	else if (m_byteTagType == TAGTYPE_BOOL)
	{
		file.Seek(1, CFile::current);
	}
	else if (m_byteTagType == TAGTYPE_BOOLARRAY)
	{
		file.Read(&uTmp, 2);
		file.Seek((uTmp / 8) + 1, CFile::current);
	}
	else if (m_byteTagType == TAGTYPE_BLOB)
	{
		file.Read(&dwLength, 4);
		if (dwLength <= (file.GetLength() - file.GetPosition()))
			file.Seek(dwLength, CFile::current);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CTag::WriteToFile(CFile &file)
{
	file.Write(&m_byteTagType, 1);
//
//	Write the tag identifier; Either a special tag code or the tag name
	if (!m_strName.IsEmpty())
	{
		uint16		nTagLength = m_strName.GetLength();

		file.Write(&nTagLength,2);
		file.Write(m_strName,nTagLength);
	}
	else
	{
		uint16		nTagLength = 1;

		file.Write(&nTagLength, 2);
		file.Write(&m_byteTagID, nTagLength);
	}
//
//	Write the tag data value
	if (m_byteTagType == TAGTYPE_STRING)
	{
		uint16		nStrLength = m_strValue.GetLength();

		file.Write(&nStrLength,2);
		file.Write(m_strValue,nStrLength);
	}
	else if (m_byteTagType == TAGTYPE_UINT32)
	{
		file.Write(&m_dwValue, 4);
	}
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CTag::WriteToFile(FILE *pFile)
{
	fputc(m_byteTagType, pFile);
//
//	Write the tag identifier; Either a special tag code or the tag name
	if (!m_strName.IsEmpty() && (m_byteTagID == 0))
	{
		uint16		nTagLength = m_strName.GetLength();

		fwrite(&nTagLength,2,1,pFile);
		fwrite(m_strName,nTagLength,1,pFile);
	}
	else
	{
		uint16		nTagLength = 1;

		fwrite(&nTagLength,2,1,pFile);
		fwrite(&m_byteTagID, nTagLength ,1 , pFile);
	}
	if (m_byteTagType == TAGTYPE_STRING)
	{
		uint16		nStrLength = m_strValue.GetLength();

		fwrite(&nStrLength,2,1,pFile);
		fwrite(m_strValue,nStrLength,1,pFile);
	}
	else if (m_byteTagType == TAGTYPE_UINT32)
	{
		fwrite(&m_dwValue, 4, 1, pFile);
	}

	return ferror(pFile);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
