//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "PartFile.h"
#include "emule.h"
#include "updownclient.h"
#include <math.h>
#include "ED2KLink.h"
#include "Preview.h"
#include "ini2.h"
#include "ArchiveRecovery.h"
#include <sys/stat.h>
#include <io.h>
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#define PROGRESS_HEIGHT 4

CPartFile::CPartFile()
{
	Init();
}

CPartFile::CPartFile(CSearchFile* searchresult, EnumCategories eCatID)
{
	EMULE_TRY

	Init();

	md4cpy(m_fileHash, searchresult->GetFileHash());
	for (int i = 0; i < searchresult->m_tagArray.GetCount();i++)
	{
		switch (searchresult->m_tagArray[i]->GetSpecialTag())
		{
			case FT_FILENAME:
			{
				SetFileName(searchresult->m_tagArray[i]->GetStringValue());
				break;
			}
			case FT_FILESIZE:
			{
				SetFileSize(searchresult->m_tagArray[i]->GetIntValue());
				break;
			}
			case FT_SOURCES:
			case FT_COMPLETE_SOURCES:
			case FT_LASTSEENCOMPLETE:
				break;
			default:
				CFileTag* newtag = new CFileTag(*searchresult->m_tagArray[i]);
				m_tagArray.Add(newtag);
		}
	}
//	Don't ask for a hashset, if there is none.
	if (GetFileSize() < PARTSIZE)
		m_bHashSetNeeded = false;

//	Set Category, don't save data yet
	if (eCatID == CAT_NONE)
		g_eMuleApp.m_pDownloadQueue->SetAutoCat(this);
	else
		SetCatID(eCatID);

	CreatePartFile();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPartFile::CPartFile(CString edonkeylink, EnumCategories eCatID)
{
	EMULE_TRY

	CED2KLink* pLink = NULL;
	try
	{
		pLink = CED2KLink::CreateLinkFromUrl(edonkeylink);
		_ASSERT(pLink != NULL);
		CED2KFileLink* pFileLink = pLink->GetFileLink();
		if (pFileLink == NULL)
			throw CString(_T("not an ed2k server or file link"));
		InitializeFromLink(pFileLink, eCatID);
	}
	catch (CString error)
	{
		OUTPUT_DEBUG_TRACE();
		CString buffer;
		buffer.Format(GetResString(IDS_ERR_INVALIDLINK), error.GetBuffer());
		AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_LINKERROR), buffer);
		SetStatus(PS_ERROR);
	}
	delete pLink;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPartFile::CPartFile(CED2KFileLink* fileLink, EnumCategories eCatID)
{
	InitializeFromLink(fileLink, eCatID);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::InitializeFromLink(CED2KFileLink* fileLink, EnumCategories eCatID)
{
	EMULE_TRY

	Init();

	try
	{
		SetFileName(fileLink->GetName());
		SetFileSize(fileLink->GetSize());
	//	Prevent to donload an empty file
		if (!GetFileSize())
		{
			AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_SKIPZEROLENGTHFILE), m_strFileName);
			SetStatus(PS_ERROR);
			return ;
		}
	//	Don't ask for a hashset, if there is none
		if (GetFileSize() < PARTSIZE)
			m_bHashSetNeeded = false;
		md4cpy(m_fileHash, fileLink->GetHashKey());
		if (!g_eMuleApp.m_pDownloadQueue->FileExists(m_fileHash))
		{
		//	Set Category, don't save data yet
			if (eCatID == CAT_NONE)
				g_eMuleApp.m_pDownloadQueue->SetAutoCat(this);
			else
				SetCatID(eCatID);

			CreatePartFile();
		}
		else
			SetStatus(PS_ERROR);
	}
	catch (CString error)
	{
		OUTPUT_DEBUG_TRACE();
		CString buffer;
		buffer.Format(GetResString(IDS_ERR_INVALIDLINK), error.GetBuffer());
		AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_LINKERROR), buffer);
		SetStatus(PS_ERROR);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::Init()
{
	EMULE_TRY

	m_strTempDir = _T("");
	m_dwLastFileSourcesRequestTime = 0;
	m_dwLastPurgeTime = ::GetTickCount();
	m_bPaused = false;
	m_bStopped = false;
	SetStatus(PS_EMPTY);
	m_dwBytesTransferred = 0;
	priority = PR_NORMAL;
	m_iNumTransferringSrcs = 0;
	m_iSrcNNP = 0;
	m_iSrcOnQueue = 0;
	m_iSrcHighQR = 0;
	m_iSrcConnecting = 0;
	m_iSrcTooManyConns = 0;
	m_iSrcConnected = 0;
	m_iSrcTimeout = 0;
	m_iSrcConnViaServer = 0;
	m_iSrcUnknown = 0;
	m_iSrcRecHash = 0;
	m_iSrcLowToLow = 0;
	m_iSrcQueueFull = 0;
	m_iSrcNoUpload = 0;
	m_iSrcBanned = 0;
	m_iSrcError = 0;
	m_iSrcA4AF = 0;
	m_dwDataRate = 0;
	m_bHashSetNeeded = true;
	m_uProcessCounter = 0;
	m_dblPercentCompleted = 0.0;
	m_strPartMetFileName = _T("");
	m_dwCompletedSize = 0;
	m_bPreviewing = false;
	lastseencomplete = NULL;
	m_dwAvailablePartsCount = 0;
	m_ClientSrcAnswered = 0;
	m_LastNoNeededCheck = 0;
	m_eRating = PF_RATING_NONE;
	m_nTotalBufferData = 0;
	m_nLastBufferFlushTime = 0;
	m_bRecoveringArchive = false;
	m_iGainDueToCompression = 0;
	m_qwLostDueToCorruption = 0;
	m_iSessionGainDueToCompression = 0;
	m_qwSessionLostDueToCorruption = 0;
	m_iTotalPacketsSavedDueToICH = 0;
	hasRating = false;
	hasComment = false;
	m_nLastCompleteSrcCount = 0;
	m_timeLastDownTransfer = CTime::GetCurrentTime();
	m_SessionStartTime = m_timeLastDownTransfer;
	m_SessionStartSize = 0;
	m_AvgDataRate = 0;
	m_StartTimeReset = false;
	m_eCategoryID = CAT_NONE;
	m_bIsBeingDeleted = false;
	m_bIsPreallocated = false;
//	We need to initialize this variable to prevent extra screen refreshing
	m_dwLastRefreshedDLDisplayTime = ::GetTickCount();
//	Async data flush (i.e. only in process)
	m_bDataFlushReq = false;
//	Set to true for fakes.rar download
	m_bIsFakesDotRar = false;
//	Set intial gaps sum to 0
	m_dwGapsSum = 0;
//	Set total size of completed parts to 0
	m_dwCompletedPartsSize = 0;

	if (g_eMuleApp.m_pGlobPrefs->IsDAPEnabled())
		m_bAutoPriority = true;
	else
		m_bAutoPriority = false;

	m_bIsPreviewableArchive = false;
	m_bIsMovie = false;
	m_bIsAviMovie = false;
	m_bIsMpgMovie  = false;
	m_bIsMpgAudio = false;

	InvalidateRemoteQueueRankingStats();
	::InitializeCriticalSection(&m_csDownloadingSourceList);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPartFile::~CPartFile()
{
	EMULE_TRY

	if (!m_bIsBeingDeleted && (m_hPartFileWrite.m_hFile != INVALID_HANDLE_VALUE))
	{
	//	Ensure all buffered data is written
		FlushBuffer();

		ClosePartFile();
	//	Update met file
		SavePartFile();
	}

	m_srcPartFrequencies.RemoveAll();
	m_PartsStatusVector.clear();

//	Will be unlocked on exit
	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

	while (!gaplist.IsEmpty())
	{
		delete gaplist.RemoveHead();
	}

	::DeleteCriticalSection(&m_csDownloadingSourceList);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::CreatePartFile()
{
	EMULE_TRY

	m_strTempDir = g_eMuleApp.m_pGlobPrefs->GetTempDir();

	if (GetCatID() != CAT_NONE)
	{
		CString strCatTempDir =	CCat::GetCatByID(GetCatID())->GetTempPath();
		if (!strCatTempDir.IsEmpty())
			m_strTempDir = strCatTempDir;
	}

	if (m_strTempDir.Right(1) == _T('\\'))
	{
		m_strTempDir.Truncate(m_strTempDir.GetLength() - 1);
	}

	SetPath(m_strTempDir);

	int iPartNumber = 0;
	CString strFileName;

	do
	{
		iPartNumber++;
		strFileName.Format(_T("%s\\%03i.part"), m_strTempDir, iPartNumber);
	}
	while (PathFileExists(strFileName));

	m_strPartMetFileName.Format(_T("%03i.part.met"), iPartNumber);
	m_strFullName.Format(_T("%s\\%s"), m_strTempDir, m_strPartMetFileName);

	CString buffer = m_strPartMetFileName.Left(m_strPartMetFileName.GetLength() - 4);
	CFileTag* partnametag = new CFileTag(FT_PARTFILENAME, buffer);

	m_tagArray.Add(partnametag);

	CString partfull = m_strFullName.Left(m_strFullName.GetLength() - 4);

	SetFilePath(partfull);

	if (!m_hPartFileWrite.Open(partfull, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::osSequentialScan))
	{
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_CREATEPARTFILE));
		SetStatus(PS_ERROR);
	}
	if (!m_hPartFileRead.Open(partfull, CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan))
	{
		m_hPartFileWrite.Close();
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_CREATEPARTFILE));
		SetStatus(PS_ERROR);
	}

	const uint32 dwPartCnt = GetPartCount();

	m_srcPartFrequencies.SetSize(dwPartCnt);
	m_PartsStatusVector.resize(dwPartCnt);

	for (uint32 i = 0; i < dwPartCnt; i++)
	{
		m_srcPartFrequencies[i] = 0;
		m_PartsStatusVector[i] = 0;
	}

//	note: the gap should be initialized after parts status vector was filled
	AddGap(0, GetFileSize() - 1);

	m_bPaused = false;

//	Before we save file we need to establish UAP
	if (g_eMuleApp.m_pGlobPrefs->IsUAPEnabled())
	{
		SetAutoULPriority(true);
		SetULPriority(PR_RELEASE);
		UpdateSharedFileDisplay();
	}

//	Don't request hashset for file which size < PARTSIZE
//	NB! file with size = PARTSIZE has hashset consisted of two hashes
	if (GetFileSize() < PARTSIZE)
	{
		m_bHashSetNeeded = false;
	}

//	Filename cleanup
	if (g_eMuleApp.m_pGlobPrefs->GetAutoFilenameCleanup())
	{
		SetFileName(CleanupFilename(GetFileName()).GetBuffer());
	}

	SavePartFile();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::LoadPartFile(LPCTSTR in_directory, LPCTSTR in_filename)
{
	EMULE_TRY

	CMap<uint16, uint16, Gap_Struct*, Gap_Struct*> gap_map;
	CSafeFile file;

	m_dwBytesTransferred = 0;
	m_strPartMetFileName = in_filename;
	m_strTempDir = in_directory;
	if (m_strTempDir.Right(1) == _T('\\'))
		m_strTempDir.Truncate(m_strTempDir.GetLength() - 1);
	SetPath(m_strTempDir);

	m_strFullName.Format(_T("%s\\%s"), GetPath(), m_strPartMetFileName);

	try
	{
	//	Read file data form part.met file
		if (!file.Open(m_strFullName, CFile::modeRead | CFile::shareDenyWrite))
		{
			AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_OPENMET), m_strPartMetFileName, m_strFileName);
			g_eMuleApp.m_pdlgEmule->DisableAutoBackup();
			return false;
		}
		byte version;
		file.Read(&version, 1);
		if (version != PARTFILE_VERSION)
		{
			if (file.m_hFile != INVALID_HANDLE_VALUE)
				file.Close();
			AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_BADMETVERSION), m_strPartMetFileName, m_strFileName);
			g_eMuleApp.m_pdlgEmule->DisableAutoBackup();
			return false;
		}
		LoadDateFromFile(file);
		if (!LoadHashsetFromFile(file, false))
		{
			AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_HASHSETCORR), in_filename);
		}
		LoadSettingsFile();

		uint32 tagcount;
		file.Read(&tagcount, 4);
		for (uint32 j = 0; j != tagcount;j++)
		{
			CFileTag* newtag = new CFileTag(file);
			switch (newtag->GetSpecialTag())
			{
				case FT_FILENAME:
				{
					if (newtag->GetStringValue().IsEmpty())
					{
						AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_METCORRUPT), m_strPartMetFileName, m_strFileName);
						delete newtag;
						g_eMuleApp.m_pdlgEmule->DisableAutoBackup();

						return false;
					}
					SetFileName(newtag->GetStringValue());
					delete newtag;
					break;
				}
				case FT_LASTSEENCOMPLETE:
				{
					lastseencomplete = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_FILESIZE:
				{
					SetFileSize(newtag->GetIntValue());
					delete newtag;
					break;
				}
				case FT_TRANSFERRED:
				{
					m_dwBytesTransferred = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_PRIORITY:
				{
					priority = newtag->GetIntValue();
					delete newtag;

					if (priority == PR_AUTO)
					{
						SetAutoPriority(true);
						priority = PR_HIGH;
					}
					else
						SetAutoPriority(false);
					break;
				}
				case FT_STATUS:
				{
					m_bPaused = newtag->GetIntValue();
					LoadSettingsFile();
					delete newtag;
					break;
				}
				case FT_ULPRIORITY:
				{
					if ((byte)newtag->GetIntValue() == PR_AUTO)
					{
						SetAutoULPriority(true);
						SetULPriority(PR_RELEASE);
					}
					else
					{
						SetAutoULPriority(false);
						SetULPriority((byte)newtag->GetIntValue());
					}
					delete newtag;
					break;
				}
				case FT_CATEGORY:
				{
					m_eCategoryID = static_cast<_EnumCategories>(newtag->GetIntValue());
					delete newtag;
					break;
				}
				default:
				{
				//	Start Changes by Slugfiller for better exception handling
					if ( (!newtag->GetSpecialTag()) && ( newtag->GetTagName()[0] == FT_GAPSTART ||
					                                     newtag->GetTagName()[0] == FT_GAPEND ) )
					{
						Gap_Struct * gap;
						CString s = newtag->GetTagName();
						s = s.Right(s.GetLength() - 1);
						uint16 gapkey = _tstoi(s);
						if (!gap_map.Lookup(gapkey, gap))
						{
							gap = new Gap_Struct;
							gap_map.SetAt(gapkey, gap);
							gap->m_dwStartOffset = -1;
							gap->m_dwEndOffset = -1;
						}
						if (newtag->GetTagName()[0] == FT_GAPSTART)
							gap->m_dwStartOffset = newtag->GetIntValue();
						if (newtag->GetTagName()[0] == FT_GAPEND)
							gap->m_dwEndOffset = newtag->GetIntValue() - 1;
						delete newtag;
					//	End Changes by Slugfiller for better exception handling
					}
					else
						m_tagArray.Add(newtag);
				}

				CStdioFile	f;
				CString		strPath = RemoveFileExtension(m_strFullName);

				strPath += _T(".dir");
				if (f.Open(strPath, CFile::modeRead | CFile::shareDenyWrite | CFile::typeText))
				{
					f.ReadString(m_strAlternativePath);
					f.Close();
				}
			}

		}
		if (file.m_hFile != INVALID_HANDLE_VALUE)
			file.Close();
	}
	catch (CFileException * error)
	{
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_METCORRUPT), m_strPartMetFileName, m_strFileName);
		else
		{
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_FILEERROR), m_strPartMetFileName, m_strFileName, GetErrorMessage(error));
		}
		error->Delete();
		for (POSITION pos = gap_map.GetStartPosition(); pos != NULL;)
		{
			Gap_Struct* gap;
			uint16 gapkey;
			gap_map.GetNextAssoc(pos, gapkey, gap);
			delete gap;
		}
		g_eMuleApp.m_pdlgEmule->DisableAutoBackup();

		return false;
	}

// at this point, the file size was defined. initialize related variables
	const uint32 dwPartCnt = GetPartCount();
	
	m_srcPartFrequencies.SetSize(dwPartCnt);
	m_PartsStatusVector.resize(dwPartCnt);

	for (uint32 i = 0; i < dwPartCnt; i++)
	{
		m_srcPartFrequencies[i] = 0;
		m_PartsStatusVector[i] = PART_VERIFIED;	//AddGap will reset this where required
	}

//	Now to flush the map into the list
	for (POSITION pos = gap_map.GetStartPosition(); pos != NULL;)
	{
		Gap_Struct* gap;
		uint16 gapkey;
		gap_map.GetNextAssoc(pos, gapkey, gap);

		if (gap->m_dwStartOffset != -1 && gap->m_dwEndOffset != -1 && gap->m_dwStartOffset <= gap->m_dwEndOffset && gap->m_dwStartOffset < GetFileSize())
		{
			if (gap->m_dwEndOffset >= GetFileSize())
				gap->m_dwEndOffset = GetFileSize() - 1; 		// Clipping
			AddGap(gap->m_dwStartOffset, gap->m_dwEndOffset); 	// All tags accounted for, use safe adding
		}
		delete gap;
	}

//	calculate the size of completed parts
	for (uint32 i = 0; i < GetPartCount(); i++)
	{
		uint32 dwStart, dwEnd;

		if (IsPartComplete(i, &dwStart, &dwEnd))
		{
			m_dwCompletedPartsSize += dwEnd - dwStart + 1;
		}
	}

	EMULE_TRY

	LoadPartFileStats();

	EMULE_CATCH

//	Open permanent handles
	CString strSearchPath = m_strFullName.Left(m_strFullName.GetLength() - 4);

	if (!m_hPartFileWrite.Open(strSearchPath, CFile::modeWrite | CFile::shareDenyWrite | CFile::osSequentialScan))
	{
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_FILEOPEN), strSearchPath, m_strFileName);
		return false;
	}
	if (!m_hPartFileRead.Open(strSearchPath, CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan))
	{
		m_hPartFileWrite.Close();
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_FILEOPEN), strSearchPath, m_strFileName);
		return false;
	}
	SetFilePath(strSearchPath);

	if (m_hPartFileWrite.GetLength() < GetFileSize())
		AddGap(m_hPartFileWrite.GetLength(), GetFileSize() - 1);

//	Goes both ways - Partfile should never be too large
	if (m_hPartFileWrite.GetLength() > GetFileSize())
	{
		TRACE("Partfile \"%s\" is too large! Truncating %I64u bytes.\n", GetFileName(), m_hPartFileWrite.GetLength() - GetFileSize());
		m_hPartFileWrite.SetLength(GetFileSize());
	}

	SetStatus(PS_EMPTY);

//	Check hashcount, filestatus etc.
	CSingleLock Lock(&m_mutexHashList, TRUE);
	INT_PTR iSize = m_partHashArray.GetCount();
	Lock.Unlock();
	if (iSize != GetED2KPartHashCount())
	{
		m_bHashSetNeeded = true;
		return true;
	}
	else
	{
		m_bHashSetNeeded = false;
		for (int i = 0; i < GetPartCount(); i++)
		{
			if (IsPartComplete(i))
			{
				SetStatus(PS_READY);
				break;
			}
		}	
	}

	m_csGapListAndPartStatus.Lock();

	bool	bIsGapListEmpty = gaplist.IsEmpty();

	m_csGapListAndPartStatus.Unlock();

// Is this file complete already?
	if (bIsGapListEmpty)
	{
		CompleteFile(false);
		return true;
	}

//	Check date of .part file - if it's wrong, rehash file
	CFileStatus filestatus;

//	"...returns m_attribute without high-order flags" indicates a known MFC bug, wonder how many unknown there are...
	m_hPartFileWrite.GetStatus(filestatus);
	m_timeLastDownTransfer = filestatus.m_mtime;

	uint32	dwFileDate = filestatus.m_mtime.GetTime();

	AdjustNTFSDaylightFileTime(&dwFileDate, strSearchPath);
	if (m_timetLastWriteDate != dwFileDate)
	{
		AddLogLine(false, RGB_LOG_WARNING + GetResString(IDS_ERR_REHASH), m_strFileName);
	//	Rehash
		SetStatus(PS_WAITINGFORHASH);
		CHashFileThread* addfilethread = (CHashFileThread*) AfxBeginThread(RUNTIME_CLASS(CHashFileThread), THREAD_PRIORITY_NORMAL + g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority(), 0, CREATE_SUSPENDED);
		addfilethread->SetValues(0, GetPath(), m_hPartFileWrite.GetFileName().GetBuffer(), this);
		addfilethread->ResumeThread();
	}

	UpdateCompletedInfos();
	if (m_dwCompletedSize > m_dwBytesTransferred)
		m_iGainDueToCompression = m_dwCompletedSize - m_dwBytesTransferred;
	else if (m_dwCompletedSize != m_dwBytesTransferred)
		m_qwLostDueToCorruption = m_dwBytesTransferred - m_dwCompletedSize;

//	Smoothed "remaining time" calculation
	m_SessionStartSize = m_dwCompletedSize;
	if (m_bPaused)
		SetStartTimeReset(true);

//	load own file comments and ratings
	UpdateFileRatingCommentAvail();

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::TryToRecoverPartFile(LPCTSTR in_directory, LPCTSTR in_filename)
{
	EMULE_TRY

	CString strTempName;

	m_strPartMetFileName = in_filename;
	m_strTempDir = in_directory;
	if (m_strTempDir.Right(1) == _T('\\'))
		m_strTempDir.Truncate(m_strTempDir.GetLength() - 1);
	SetPath(m_strTempDir);

	m_strFullName.Format(_T("%s\\%s"), GetPath(), m_strPartMetFileName);

//	Rename ".met" to ".met.bad"
	strTempName.Format(_T("%s.bad"), m_strFullName);
	if (_trename(m_strFullName, strTempName) != 0)
		AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Failed to rename part.met file '%s' to '%s' - %s"), m_strFullName, strTempName, strerror(errno));

//	Copy ".met.bak" to ".met"
	strTempName.Format(_T("%s.bak"), m_strFullName);
	if (CopyFile(strTempName, m_strFullName, false) == 0)
	{
		AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Failed to copy .bak file '%s' to '%s'"), strTempName, m_strFullName);
		AddLogLine(false, RGB_LOG_ERROR_TXT + GetResString(IDS_TRYTORECOVER_FAIL), m_strFullName);
		return false;
	}

	AddLogLine(false, GetResString(IDS_TRYTORECOVER), m_strFullName);

	bool bLoadPartFileStatus = LoadPartFile(in_directory, in_filename);

	if (bLoadPartFileStatus)
		AddLogLine(false, RGB_LOG_SUCCESS + GetResString(IDS_RECOVERED_PARTMET), GetFileName());
	else
		AddLogLine(false, RGB_LOG_ERROR_TXT + GetResString(IDS_TRYTORECOVER_FAIL), m_strFullName);

	return bLoadPartFileStatus;

	EMULE_CATCH

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::SavePartFile()
{
	EMULE_TRY

	switch (m_eStatus)
	{
		case PS_WAITINGFORHASH:
		case PS_HASHING:
		case PS_COMPLETING:
		case PS_COMPLETE:
		return false;
	}

//	Protect file corruption during calling from different threads
//	will be unlocked on exit
	CSingleLock sLock(&m_csSavePartFile, TRUE);

//	Get filedate
	CFileFind ff;
	CString strSearchPath = m_strFullName.Left(m_strFullName.GetLength() - 4);
	bool bEnd = !ff.FindFile(strSearchPath, 0);
	if (!bEnd)
		ff.FindNextFile();
	if (bEnd || ff.IsDirectory())
	{
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_SAVEMET), GetResString(IDS_ERR_PART_FNF), m_strPartMetFileName, m_strFileName);
		return false;
	}

//	Create a backup of ".met"
	::CopyFile(m_strFullName, m_strFullName + _T(".bak"), FALSE);

	CTime lwtime;
	ff.GetLastWriteTime(lwtime);
	m_timetLastWriteDate = lwtime.GetTime();
	AdjustNTFSDaylightFileTime(&m_timetLastWriteDate, ff.GetFilePath());
	ff.Close();
	uint32 lsc = mktime(lastseencomplete.GetLocalTm());

//	Create a temporary file
	CString strTempName(m_strFullName);

	strTempName.Append(_T(".tmp"));
	if (_tremove(strTempName) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strTempName);

	CStdioFile file;

	try
	{
	//	Open the file
		file.Open(strTempName, CFile::modeWrite|CFile::modeCreate|CFile::typeBinary);
	//	Version
		byte uVersion = PARTFILE_VERSION;
		file.Write(&uVersion, 1);
	//	Date
		file.Write(&m_timetLastWriteDate, 4);
	//	Hash
		file.Write(&m_fileHash, 16);
		CSingleLock Lock(&m_mutexHashList, TRUE);
		uint16 uParts = m_partHashArray.GetCount();
		file.Write(&uParts, 2);
		for (int x = 0; x != uParts; x++)
			file.Write(m_partHashArray[x], 16);
		Lock.Unlock();

	//	Tags
		CArray<Gap_Struct, Gap_Struct> aGaps;

	//	Make a gap list copy to avoid synchronization object allocation for a long time
		GetGapListCopy(&aGaps);

		uint32 dwTagCount = m_tagArray.GetCount() + 8 + (aGaps.GetCount() * 2);
		file.Write(&dwTagCount, 4);
		CFileTag nametag(FT_FILENAME, GetFileName());
		nametag.WriteToFile(file);
		CFileTag sizetag(FT_FILESIZE, GetFileSize());
		sizetag.WriteToFile(file);
		CFileTag transtag(FT_TRANSFERRED, m_dwBytesTransferred);
		transtag.WriteToFile(file);
		CFileTag statustag(FT_STATUS, (m_bPaused) ? 1 : 0);
		statustag.WriteToFile(file);

		CFileTag prioritytag(FT_PRIORITY, (IsAutoPrioritized()) ? PR_AUTO : priority);
		prioritytag.WriteToFile(file);

		CFileTag lsctag(FT_LASTSEENCOMPLETE, lsc);
		lsctag.WriteToFile(file);

		CFileTag ulprioritytag(FT_ULPRIORITY, (IsULAutoPrioritized()) ? PR_AUTO : GetULPriority());
		ulprioritytag.WriteToFile(file);

		CFileTag categorytag(FT_CATEGORY, static_cast<int>(m_eCategoryID));
		categorytag.WriteToFile(file);

		for (uint32 j = 0; j != m_tagArray.GetCount();j++)
			m_tagArray[j]->WriteToFile(file);

	//	Gaps
		TCHAR	acNameBuffer[16];
		uint32	dwPos = 0;

		Gap_Struct	*pGap = aGaps.GetData();
		Gap_Struct	*pGapMax = pGap + aGaps.GetCount();

		for (; pGap < pGapMax; pGap++)
		{
			_itot(dwPos++, &acNameBuffer[1], 10);
			acNameBuffer[0] = FT_GAPSTART;
			CFileTag gapstarttag(acNameBuffer, pGap->m_dwStartOffset);
			gapstarttag.WriteToFile(file);
		//	gap start = first missing byte but gap ends = first non-missing byte in edonkey
		//	but I think its easier to user the real limits
			acNameBuffer[0] = FT_GAPEND;
			CFileTag gapendtag(acNameBuffer, pGap->m_dwEndOffset + 1);
			gapendtag.WriteToFile(file);
		}
	}
	catch(CFileException* error)
	{
		OUTPUT_DEBUG_TRACE	();
		AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_SAVEMET), GetErrorMessage(error), m_strPartMetFileName, m_strFileName);
		error->Delete();
		file.Close();
 	//	Remove the partially written or otherwise damaged temporary file
 		_tremove(strTempName);
		return false;
	}

	_commit(_fileno(file.m_pStream));
	file.Close();

//	Delete original ".met", after successfully writing the temporary part.met file...
	if (_tremove(m_strFullName) != 0 && errno != ENOENT)
		AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Failed to remove '%s' - %s"), m_strFullName, _tcserror(errno));

//	Rename ".met.tmp" to ".met"
	if (_trename(strTempName, m_strFullName) != 0)
		AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Failed to move temporary part.met file '%s' to '%s' - %s"), strTempName, m_strFullName, strerror(errno));

	CStdioFile	fDir;
	CString		strPath = RemoveFileExtension(m_strFullName);

	strPath += _T(".dir");
	try
	{
	//	ChangeDir
		if (!m_strAlternativePath.IsEmpty())
		{
			if (fDir.Open(strPath, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText))
			{
				fDir.WriteString((LPCTSTR)m_strAlternativePath);
				fDir.Close();
			}
		}
	}
	catch(CFileException* error)
	{
		OUTPUT_DEBUG_TRACE();
		AddLogLine(false, RGB_LOG_ERROR_TXT _T("Error saving %s - %s"), strPath, GetErrorMessage(error));
		error->Delete();
		fDir.Close();
	}

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::PartFileHashFinished(CKnownFile* pKnownFile)
{
	EMULE_TRY

	bool	bErrorFound = false;

	CSingleLock Lock(&m_mutexHashList, TRUE);

	uint32 dwCount = static_cast<uint32>(m_partHashArray.GetSize());

//	Number of parts to check taking into consideration files with size % PARTSIZE = 0 and size < PARTSIZE
	dwCount = (dwCount > GetPartCount()) ? GetPartCount() : dwCount;

	for (uint32 i = 0; i < dwCount; i++)
	{
		uint32 dwPartStart, dwPartEnd;

		if (IsPartComplete(i, &dwPartStart, &dwPartEnd))
		{
			if ((pKnownFile->GetPartHash(i) == NULL) || (md4cmp(pKnownFile->GetPartHash(i), GetPartHash(i)) != 0))
			{
				AddLogLine(false, RGB_LOG_WARNING + GetResString(IDS_ERR_FOUNDCORRUPTION), i, GetFileName());
				AddGap(dwPartStart, dwPartEnd);
				bErrorFound = true;
			}
		}
	}
	Lock.Unlock();
	delete pKnownFile;

	if (!bErrorFound)
	{
		if (m_eStatus == PS_COMPLETING)
		{
			CompleteFile(true);
			return;
		}
		else
			AddLogLine(false, IDS_HASHINGDONE, GetFileName());
	}
	else
	{
		SetStatus(PS_READY);
		SavePartFile();
	//	If a file is "ready" it can't be paused
		m_bPaused = false;
		return ;
	}
	SetStatus(PS_READY);
	SavePartFile();
	g_eMuleApp.m_pSharedFilesList->SafeAddKnownFile(this);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::AddGap(uint32 dwStart, uint32 dwEnd)
{
	EMULE_TRY

	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

	POSITION pos1, pos2;

	for (pos1 = gaplist.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		Gap_Struct* cur_gap = gaplist.GetNext(pos1);
		if (cur_gap->m_dwStartOffset >= dwStart && cur_gap->m_dwEndOffset <= dwEnd)
		{ // this gap is inside the new gap - delete
			gaplist.RemoveAt(pos2);
			m_dwGapsSum -= cur_gap->m_dwEndOffset - cur_gap->m_dwStartOffset + 1;
			FillGapInParts(cur_gap->m_dwStartOffset, cur_gap->m_dwEndOffset);
			delete cur_gap;
		}
		else if (cur_gap->m_dwStartOffset >= dwStart && cur_gap->m_dwStartOffset <= dwEnd)
		{ // a part of this gap is in the new gap - extend limit and delete
			dwEnd = cur_gap->m_dwEndOffset;
			gaplist.RemoveAt(pos2);
			m_dwGapsSum -= dwEnd - cur_gap->m_dwStartOffset + 1;
			FillGapInParts(cur_gap->m_dwStartOffset, dwEnd);
			delete cur_gap;
		}
		else if (cur_gap->m_dwEndOffset <= dwEnd && cur_gap->m_dwEndOffset >= dwStart)
		{ // a part of this gap is in the new gap - extend limit and delete
			dwStart = cur_gap->m_dwStartOffset;
			gaplist.RemoveAt(pos2);
			m_dwGapsSum -= cur_gap->m_dwEndOffset - dwStart + 1;
			FillGapInParts(dwStart, cur_gap->m_dwEndOffset);
			delete cur_gap;
		}
		else if (dwStart >= cur_gap->m_dwStartOffset && dwEnd <= cur_gap->m_dwEndOffset)
		{ // new gap is already inside this gap - return
			return;
		}
	}
	Gap_Struct* new_gap = new Gap_Struct;
	new_gap->m_dwStartOffset = dwStart;
	new_gap->m_dwEndOffset = dwEnd;
	m_dwGapsSum += dwEnd - dwStart + 1;
	AddGapToParts(dwStart, dwEnd);
	gaplist.AddTail(new_gap);

	lockGap.Unlock();

	UpdateDisplayedInfo();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsComplete() returns false if there are gaps in the range 'start' to 'dwEnd' (inclusive).
bool CPartFile::IsComplete(uint32 start, uint32 dwEnd)
{
	EMULE_TRY

	if (dwEnd >= GetFileSize())
		dwEnd = GetFileSize() - 1;

	bool	bRc = true;

// 	Will be unlocked on exit
	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

	for (POSITION pos = gaplist.GetHeadPosition(); pos != NULL;)
	{
		Gap_Struct * cur_gap = gaplist.GetNext(pos);

		if ( (cur_gap->m_dwStartOffset >= start && cur_gap->m_dwEndOffset <= dwEnd)			// gap is inside range (this check is unnecessary)
		     || (cur_gap->m_dwStartOffset >= start && cur_gap->m_dwStartOffset <= dwEnd)	// gap starts inside range
		     || (cur_gap->m_dwEndOffset <= dwEnd && cur_gap->m_dwEndOffset >= start)		// gap ends inside range
		     || (start >= cur_gap->m_dwStartOffset && dwEnd <= cur_gap->m_dwEndOffset) )	// range is inside gap
		{
			bRc = false;
			break;
		}
	}
	return bRc;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsPartComplete() returns true if a part is complete and verified.
bool CPartFile::IsPartComplete(uint32 dwPart) const
{
	return (m_PartsStatusVector[dwPart] == PART_VERIFIED);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsPartComplete() returns true if a part is complete and verified.
//		Params:
//			dwPart   - part number;
//			pdwStart - pointer to get part start offset;
//			pdwEnd   - pointer to get part end offset (inclusive).
bool CPartFile::IsPartComplete(uint32 dwPart, uint32 *pdwStart, uint32 *pdwEnd) const
{
	uint32 dwStart = dwPart * PARTSIZE;
	uint32 dwEnd = ((GetFileSize() - dwStart) > PARTSIZE) ? (dwStart + PARTSIZE - 1) : (GetFileSize() - 1);

	*pdwStart = dwStart;
	*pdwEnd = dwEnd;

	return (m_PartsStatusVector[dwPart] == PART_VERIFIED);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsPartFull() returns true if all data of the was received.
bool CPartFile::IsPartFull(uint32 dwPart) const
{
	return ((m_PartsStatusVector[dwPart] & 0xFFFFFF) == 0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::IsPartDownloading(uint32 dwPart) const
{
	EMULE_TRY

	for (POSITION pos = requestedblocks_list.GetHeadPosition(); pos != NULL;)
	{
		Requested_Block_Struct *pReqBlock = requestedblocks_list.GetNext(pos);

		if (dwPart == (pReqBlock->m_dwStartOffset / PARTSIZE))
			return true;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::IsAlreadyRequested(uint32 start, uint32 dwEnd)
{
	EMULE_TRY

	for (POSITION pos = requestedblocks_list.GetHeadPosition(); pos != NULL;)
	{
		Requested_Block_Struct * cur_block = requestedblocks_list.GetNext(pos);
		if ((start <= cur_block->m_dwEndOffset) && (dwEnd >= cur_block->m_dwStartOffset))
			return true;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetGapsInPart() return request blocks for the corresponding part
//		Params:
//			numPart    - a part to examine;
//			newblocks  - output array for allocated request blocks (can be NULL to check for block allocation);
//			dwCount    - number of blocks to allocate.
//		Return:
//			number of allocated blocks or number of possible blocks to allocate if newblocks = NULL.
uint32 CPartFile::GetGapsInPart(uint16 numPart, Requested_Block_Struct** newblocks, uint32 dwCount)
{
	uint32	dwNewBlockCount = 0;

	EMULE_TRY

//	Calculate offsets of the Part
	const uint32 uPartStart = numPart * PARTSIZE;
	const uint32 uPartEnd = ((GetFileSize() - uPartStart) > PARTSIZE) ? (uPartStart + PARTSIZE - 1) : (GetFileSize() - 1);

	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);	// will be unlocked on exit

//	Retrieve Gap(s) matching the part
	for (POSITION pos = gaplist.GetHeadPosition(); pos != NULL;)
	{
		Gap_Struct *CurGap = gaplist.GetNext(pos);

	//	Check if Gap is inside the limit
		if (CurGap->m_dwStartOffset > uPartEnd || CurGap->m_dwEndOffset < uPartStart)
			continue;

	//	Calculate offsets of the Gap within the part
		uint32 uGapStart = (CurGap->m_dwStartOffset < uPartStart) ? uPartStart : CurGap->m_dwStartOffset;
		const uint32 uGapEnd = (CurGap->m_dwEndOffset > uPartEnd) ? uPartEnd : CurGap->m_dwEndOffset;

	//	Create n request block(s)
		while ((uGapEnd >= uGapStart) && (dwNewBlockCount < dwCount))
		{
		//	Calculate offsets of the block
			const uint32 startOffset = uGapStart;
			const uint32 endOffset = ((uGapStart + EMBLOCKSIZE - 1) >= uGapEnd) ? uGapEnd : ((uGapStart + EMBLOCKSIZE - 1) - (uGapStart % EMBLOCKSIZE));

		//	Prepare offset of next block
			uGapStart = endOffset + 1;

		//	Check if block has already been requested
			if (!IsAlreadyRequested(startOffset, endOffset))
			{
				if (newblocks != NULL)
				{
				//	Create 1 request block
					Requested_Block_Struct * block = new Requested_Block_Struct;
					block->m_dwStartOffset = startOffset;
					block->m_dwEndOffset = endOffset;
					block->m_dwBytesTransferred = 0;
					md4cpy(block->m_fileHash, GetFileHash());

				//	Flag the block as 'requested'
					requestedblocks_list.AddTail(block);

				//	Return the block to the source
					newblocks[dwNewBlockCount] = block;
				}
				dwNewBlockCount++;
			}
		}
	}

	EMULE_CATCH

	return dwNewBlockCount;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::FillGap(uint32 dwStart, uint32 dwEnd)
{
	EMULE_TRY

	POSITION pos1, pos2;

	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

	for (pos1 = gaplist.GetHeadPosition(); (pos2 = pos1) != NULL; )
	{
		Gap_Struct* cur_gap = gaplist.GetNext(pos1);
		if (cur_gap->m_dwStartOffset >= dwStart && cur_gap->m_dwEndOffset <= dwEnd)
		{ // our part fills this gap completely
			gaplist.RemoveAt(pos2);
			m_dwGapsSum -= cur_gap->m_dwEndOffset - cur_gap->m_dwStartOffset + 1;
			FillGapInParts(cur_gap->m_dwStartOffset, cur_gap->m_dwEndOffset);
			delete cur_gap;
		}
		else if (cur_gap->m_dwStartOffset >= dwStart && cur_gap->m_dwStartOffset <= dwEnd)
		{ // a part of this gap is in the part - set limit
			m_dwGapsSum -= dwEnd - cur_gap->m_dwStartOffset + 1;
			FillGapInParts(cur_gap->m_dwStartOffset, dwEnd);
			cur_gap->m_dwStartOffset = dwEnd + 1;
		}
		else if (cur_gap->m_dwEndOffset <= dwEnd && cur_gap->m_dwEndOffset >= dwStart)
		{ // a part of this gap is in the part - set limit
			m_dwGapsSum -= cur_gap->m_dwEndOffset - dwStart + 1;
			FillGapInParts(dwStart, cur_gap->m_dwEndOffset);
			cur_gap->m_dwEndOffset = dwStart - 1;
		}
		else if (dwStart >= cur_gap->m_dwStartOffset && dwEnd <= cur_gap->m_dwEndOffset)
		{
			uint32 dwTmp = cur_gap->m_dwEndOffset;

			cur_gap->m_dwEndOffset = dwStart - 1;
			cur_gap = new Gap_Struct;
			cur_gap->m_dwStartOffset = dwEnd + 1;
			cur_gap->m_dwEndOffset = dwTmp;
			m_dwGapsSum -= dwEnd - dwStart + 1;
			FillGapInParts(dwStart, dwEnd);
			gaplist.InsertAfter(pos1, cur_gap);
			break;
		}
	}
	lockGap.Unlock();

	UpdateCompletedInfos();
	UpdateDisplayedInfo();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function updates the part status vector based on gap information
// Must be called only under m_csGapListAndPartStatus protection
void CPartFile::FillGapInParts(uint32 dwGapStart, uint32 dwGapEnd)
{
	for (uint32 dwPart = (dwGapStart / PARTSIZE); dwPart <= (dwGapEnd / PARTSIZE); dwPart++)
	{
		uint32 dwPartStart = dwPart * PARTSIZE;
		uint32 dwPartEnd = ((GetFileSize() - dwPartStart) > PARTSIZE) ? (dwPartStart + PARTSIZE - 1) : (GetFileSize() - 1);
		uint32 dwPartFilledSize = GetPartLeftToDLSize(dwPart);

		if (dwGapStart <= dwPartStart)
		{
			if (dwGapEnd >= dwPartEnd)	//	[Gs (Ps Pe) Ge]
				dwPartFilledSize = 0;
			else						//	[Gs (Ps Ge] Pe)
				dwPartFilledSize -= dwGapEnd - dwPartStart + 1;
		}
		else
		{
			if (dwGapEnd < dwPartEnd)	//	(Ps [Gs Ge] Pe) else (Ps [Gs Pe) Ge]
				dwPartEnd = dwGapEnd;
			dwPartFilledSize -= dwPartEnd - dwGapStart + 1;
		}

		m_PartsStatusVector[dwPart] = (m_PartsStatusVector[dwPart] & 0xFF000000) | dwPartFilledSize;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function updates the part status vector based on gap information
// Must be called only under m_csGapListAndPartStatus protection
void CPartFile::AddGapToParts(uint32 dwGapStart, uint32 dwGapEnd)
{
	for (uint32 dwPart = (dwGapStart / PARTSIZE); dwPart <= (dwGapEnd / PARTSIZE); dwPart++)
	{
		uint32 dwPartStart = dwPart * PARTSIZE;
		uint32 dwPartEnd = ((GetFileSize() - dwPartStart) > PARTSIZE) ? (dwPartStart + PARTSIZE - 1) : (GetFileSize() - 1);
		uint32 dwPartFilledSize = GetPartLeftToDLSize(dwPart);

		if (dwGapStart <= dwPartStart)
		{
			if (dwGapEnd >= dwPartEnd)	//	[Gs (Ps Pe) Ge]
				dwPartFilledSize = dwPartEnd - dwPartStart + 1;
			else						//	[Gs (Ps Ge] Pe)
				dwPartFilledSize += dwGapEnd - dwPartStart + 1;
		}
		else
		{
			if (dwGapEnd < dwPartEnd)	//	(Ps [Gs Ge] Pe) else (Ps [Gs Pe) Ge]
				dwPartEnd = dwGapEnd;
			dwPartFilledSize += dwPartEnd - dwGapStart + 1;
		}
	//	Reset PART_VERIFIED status as a gap was added to this part
		m_PartsStatusVector[dwPart] =
			(m_PartsStatusVector[dwPart] & (0xFF000000 & ~PART_VERIFIED)) | dwPartFilledSize;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::UpdateCompletedInfos()
{
	EMULE_TRY

	if ((m_dwGapsSum != 0) || (requestedblocks_list.GetCount() != 0))
	{
		m_dblPercentCompleted = (1.0 - static_cast<double>(m_dwGapsSum) / GetFileSize()) * 100;
		m_dwCompletedSize = GetFileSize() - m_dwGapsSum;
	}
	else
	{
		m_dblPercentCompleted = 100.0;
		m_dwCompletedSize = GetFileSize();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::DrawStatusBar(CDC* dc, RECT* rect, bool bFlat)
{
	EMULE_TRY

	const COLORREF crProgress = (bFlat) ? RGB(0, 150, 0) : RGB(0, 192, 0);
	COLORREF crHave, crPending, crMissing;

//	grayPause - Colors by status
	EnumPartFileStatuses	eFileStatus = GetStatus();
	bool			bNotPaused = (eFileStatus == PS_EMPTY) || (eFileStatus == PS_READY);

	if (bNotPaused)
	{
		crHave = (bFlat) ? RGB(0, 0, 0) : RGB(95, 95, 95);
		crPending = RGB(255, 208, 0);
		crMissing = RGB(255, 0, 0);
	}
	else
	{
		crHave = (bFlat) ? RGB(105, 105, 105) : RGB(142, 142, 142);
		crPending = RGB(255, 240, 142);
		crMissing = RGB(255, 142, 142);
	}

	CBarShader s_ChunkBar(rect->bottom - rect->top, rect->right - rect->left, crHave, GetFileSize());

	if (m_eStatus == PS_COMPLETE || m_eStatus == PS_COMPLETING)
	{
		s_ChunkBar.Fill(crProgress);
		s_ChunkBar.Draw(dc, rect->left, rect->top, bFlat);
		m_dblPercentCompleted = 100.0;
		m_dwCompletedSize = GetFileSize();
		return;
	}

//	Red gaps
	CArray<Gap_Struct, Gap_Struct> aGaps;

//	Make a gap list copy to avoid synchronization object allocation for a long time
	GetGapListCopy(&aGaps);

	Gap_Struct	*pGap = aGaps.GetData();
	Gap_Struct	*pGapMax = pGap + aGaps.GetCount();

	for (; pGap < pGapMax; pGap++)
	{
		uint32 dwGapBeg = pGap->m_dwStartOffset;
		uint32 dwGapEnd = pGap->m_dwEndOffset + 1;

		for (uint32 i = (dwGapBeg / PARTSIZE); i < GetPartCount(); i++)
		{
			const uint32 dwPartStart = i * PARTSIZE;
			const uint32 dwPartEnd = ((GetFileSize() - dwPartStart) > PARTSIZE) ? (dwPartStart + PARTSIZE) : GetFileSize();

			if (dwGapEnd > dwPartEnd)
				dwGapEnd = dwPartEnd; //the rest is in the next part

			COLORREF	color;
			int			iFreq;

			if (m_srcPartFrequencies.GetCount() >= (INT_PTR)i && ((iFreq = m_srcPartFrequencies[i] - 1) >= 0))
			{
				if (bNotPaused)
					color = RGB(0, ((210 - 8 * iFreq) < 0) ? 0 : (210 - 8 * iFreq), 255);
				else
					color = RGB(100, ((210 - 8 * iFreq) < 0) ? 0 : (255 - 8 * iFreq), 255);
			}
			else
				color = crMissing;

			s_ChunkBar.FillRange(dwGapBeg, dwGapEnd, color);

			dwGapBeg = dwGapEnd;
			dwGapEnd = pGap->m_dwEndOffset + 1;
			if (dwGapBeg == dwGapEnd) // finished?
				break;
		}
	}

//	Yellow pending parts
	for (POSITION pos = requestedblocks_list.GetHeadPosition(); pos != NULL;)
	{
		Requested_Block_Struct * block = requestedblocks_list.GetNext(pos);
		s_ChunkBar.FillRange((block->m_dwStartOffset + block->m_dwBytesTransferred), block->m_dwEndOffset, crPending);
	}

	s_ChunkBar.Draw(dc, rect->left, rect->top, bFlat);

//	Green progress
	RECT gaprect;
	gaprect.top = rect->top;
	gaprect.bottom = gaprect.top + PROGRESS_HEIGHT;
	gaprect.left = rect->left;

	uint32	w = rect->right - rect->left + 1;
	uint32	wp = (uint32)(m_dblPercentCompleted / 100.0 * w + 0.5);

	if (!bFlat)
	{
		CBarShader s_LoadBar(PROGRESS_HEIGHT, wp, crPending, m_dwCompletedSize);
		s_LoadBar.FillRange(0, m_dwCompletedPartsSize, crProgress);
		s_LoadBar.Draw(dc, gaprect.left, gaprect.top, false);
	}
	else
	{
		uint32	wc = (uint32)(static_cast<double>(m_dwCompletedPartsSize) / GetFileSize() * w + 0.5);
		int	left = rect->left;
		CBrush *pOldBrush;

	//	Green
		gaprect.right = left + wc;
		CBrush pr_Brush(crProgress);
		pOldBrush = dc->SelectObject(&pr_Brush);
		dc->FillRect(&gaprect, &pr_Brush);

	//	Yellow
		gaprect.left = left + wc;
		gaprect.right = left + wp;
		CBrush pe_Brush(crPending);
		dc->SelectObject(&pe_Brush);
		dc->FillRect(&gaprect, &pe_Brush);

	//	Draw gray progress only if flat
		gaprect.left = left + wp;
		gaprect.right = left + w;
		CBrush rgb_Brush(RGB(224, 224, 224));
		dc->SelectObject(&rgb_Brush);
		dc->FillRect(&gaprect, &rgb_Brush);

		dc->SelectObject(pOldBrush);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::WritePartStatus(CFile* file)
{
	EMULE_TRY

	uint16 parts = GetED2KPartCount();
	file->Write(&parts, 2);
	uint16 done = 0;
	while (done != parts)
	{
		byte towrite = 0;
		for (uint32 i = 0; i < 8; i++)
		{
		//	Report that we have "void" part for files with size % PARTSIZE = 0
			if ( ((done == (parts - 1)) && (parts != GetPartCount())) ||
				(IsPartShared(done) && IsPartComplete(done)) )
				towrite |= (1 << i);

			if (++done == parts)
				break;
		}
		file->Write(&towrite, 1);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::WriteCompleteSourcesCount(CFile* file)
{
	uint16 completecount = GetCompleteSourcesCount();
	file->Write(&completecount, 2);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CPartFile::GetValidSourcesCount()
{
	EMULE_TRY

	int counter = 0;
	POSITION pos;
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
		if (!m_srcLists[sl].IsEmpty())
			for (pos = m_srcLists[sl].GetHeadPosition();pos != NULL;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos);
				if ( pSource->GetDownloadState() != DS_ONQUEUE && pSource->GetDownloadState() != DS_DOWNLOADING &&
				     pSource->GetDownloadState() != DS_NONEEDEDPARTS )
					counter++;
			}
	return counter;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CPartFile::GetNotCurrentSourcesCount()
{
	EMULE_TRY

	uint16 counter = 0;

	POSITION pos;
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
		if (!m_srcLists[sl].IsEmpty())
			for (pos = m_srcLists[sl].GetHeadPosition();pos != NULL;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos);
				if (pSource->GetDownloadState() != DS_ONQUEUE && pSource->GetDownloadState() != DS_DOWNLOADING)
					counter++;
			}
	return counter;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
EnumPartFileStatuses CPartFile::GetStatus(bool bIgnorePause)
{
	EMULE_TRY

	if (!m_bPaused || m_eStatus == PS_ERROR || m_eStatus == PS_COMPLETING || m_eStatus == PS_COMPLETE || bIgnorePause)
	{
		return m_eStatus;
	}
	else if (m_bStopped)
	{
		return PS_STOPPED;
	}
	else
	{
		return PS_PAUSED;
	}

	EMULE_CATCH

	return PS_READY;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Inserted by Vorlost for load rarest parts first, code by Denominator
uint16 CPartFile::GetRemainingBlocksInPart(uint16 partnumber)
{
	EMULE_TRY

	uint16		uNumRemainingBlocks = 0;
	for (uint32 i = 0; i < ((PARTSIZE + EMBLOCKSIZE - 1) / EMBLOCKSIZE); i++)
	{
		uint32 start = (PARTSIZE * partnumber) + i * EMBLOCKSIZE;
		uint32 dwEnd = (PARTSIZE * partnumber) + ((i + 1) * EMBLOCKSIZE) - 1;
		if (dwEnd >= PARTSIZE * ((uint32)partnumber + 1))
			dwEnd = (PARTSIZE * (partnumber + 1)) - 1;
		if (start >= GetFileSize())
			break;
		if (dwEnd >= GetFileSize())
			dwEnd = GetFileSize() - 1;
		if (!IsComplete(start, dwEnd) && !IsAlreadyRequested(start, dwEnd))
			uNumRemainingBlocks++;
	}
	return uNumRemainingBlocks;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Remove no needed sources from download queue
//	[In] number_sources_to_remove: Number of sources to be removed from list or 0xffffffff if we want all removed
void CPartFile::RemoveNoNeededSources(uint32 number_sources_to_remove)
{
	EMULE_TRY

	POSITION pos;
	uint32 removed = 0;
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
		if (!m_srcLists[sl].IsEmpty())
			for (pos = m_srcLists[sl].GetHeadPosition(); pos != NULL && removed < number_sources_to_remove;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos);
				if ((pSource->GetDownloadState() == DS_NONEEDEDPARTS) || (pSource->IsRemoteQueueFull()))  //netwolf: katsyonak change
				{
					g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
					removed ++;
				}
			}

	EMULE_CATCH

	m_dwLastPurgeTime = ::GetTickCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CPartFile::Process(uint32 dwReduceDownload /*in percent*/, uint32 dwIteration)
{
	EMULE_TRY

	uint16			nOldTransSourceCount = GetTransferringSrcCount();
	uint16			iOldSrcOnQueueCount = GetOnQueueSrcCount();
	DWORD			dwCurTick = ::GetTickCount();
	CUpDownClient  *pSource = NULL;
	bool			bSocketsUsageIsHigh = false;
	bool			bIsClientConnected = false;
	bool			bReaskDownload = false;
	POSITION		pos1, pos2;
	uint32			dwCurrentDataRate = 0;
	uint32			dwLastNoNeededCheckTime = 0;
	uint32			dwDataRate = 0;

//	flush the data to disk if:
//	1. it was requested
//	2. buffer size exceeds limit
//	3. data was not written within time limit
	if (m_bDataFlushReq
		|| (m_nTotalBufferData > (static_cast<uint32>(g_eMuleApp.m_pGlobPrefs->GetFileBufferSize()) * 30000))
		|| (dwCurTick > (m_nLastBufferFlushTime + BUFFER_TIME_LIMIT)) )
	{
	//	Avoid flushing while copying preview file
		if (!m_bPreviewing)
		{
			FlushBuffer();
		}
	}

#ifdef OLD_SOCKETS_ENABLED
	bSocketsUsageIsHigh = g_eMuleApp.m_pListenSocket->GetNumOpenSockets() > (g_eMuleApp.m_pGlobPrefs->GetMaxConnections() * 0.75);
#endif //OLD_SOCKETS_ENABLED

	uint32	dwNumTransferringSrcs = 0;

//	Calculate data rate, set limit, etc.
//	9 iterations for download & 1 iteration for other files
	if (dwIteration < 10 && !m_downloadingSourceList.IsEmpty())
	{
		EnterCriticalSection(&m_csDownloadingSourceList);
		POSITION posHeadList = m_downloadingSourceList.GetHeadPosition();
		LeaveCriticalSection(&m_csDownloadingSourceList);

		for (pos1 = posHeadList; (pos2 = pos1) != NULL;)
		{
			EnterCriticalSection(&m_csDownloadingSourceList);
			pSource = m_downloadingSourceList.GetNext(pos1);
			LeaveCriticalSection(&m_csDownloadingSourceList);
			if (pSource)
			{
			//	If new client has non DL state, then remove him
				if (pSource->GetDownloadState() != DS_DOWNLOADING)
				{
					AddDebugLogLine(_T("%s: corruption of m_downloadingSourceList"), __FUNCTION__);
					EnterCriticalSection(&m_csDownloadingSourceList);
					m_downloadingSourceList.RemoveAt(pos2);
					LeaveCriticalSection(&m_csDownloadingSourceList);
					continue;
				}
				dwCurrentDataRate = pSource->DownloadProcess(dwReduceDownload);

			//	Since client can be dropped due timeout we need to check the state again
				if (pSource->GetDownloadState() != DS_DOWNLOADING)
				{
					continue;
				}
				dwNumTransferringSrcs++;
				dwDataRate += dwCurrentDataRate;
			}
		}
		m_iNumTransferringSrcs = dwNumTransferringSrcs;
		m_dwDataRate = dwDataRate;
	}
	else
	{
	//	Initialize temp variables only if we need it
		bool	bAlreadyCounted = false;
		uint16	uNNPSrcCount = 0;
		uint16	uOnQueueSrcCount = 0;
		uint16	uHighQRSrcCount = 0;
		uint16	uConnectingSrcCount = 0;
		uint16	uTooManyConnsSrcCount = 0;
		uint16	uConnectedSrcCount = 0;
		uint16	uTimeoutSrcCount = 0;
		uint16	uConnViaServerSrcCount = 0;
		uint16	uUnknownSrcCount = 0;
		uint16	uReqHashSrcCount = 0;
		uint16	uLowToLowIDSrcCount = 0;
		uint16	uQueueFullSrcCount = 0;
		uint16	uNoUploadSrcCount = 0;
		uint16	uBannedSrcCount = 0;
		uint16	uErrorSrcCount = 0;
		uint16	uLowestQR = 0xFFFF;
		uint16	uHighestQR = 0;

	//	Cache number of sources
		uint16	uSourceNumber = GetSourceCount();
		uint16	uMaxSources = g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile();

	//	In case of timeout all sources are dropped every round without restrictions.
	//	Preferred droppingorder by state is: Low2Low, NNP, QueueFull, HighQR.
	//	Set the maximal number of sources to drop this round dynamicly by demand.
		uint16	uDropLimit = 0;
		if ((m_dwLastPurgeTime + PURGE_TIME) < dwCurTick)
		{
		//	Drop more sources if slots are >105% overcrowded
			if (uSourceNumber > uMaxSources * 1.05)
				uDropLimit = 10;
		//	Drop few sources if >95% slots full
			else if (uSourceNumber > uMaxSources * 0.95)
				uDropLimit = 3;
		//	Drop only one source if >85% slots full
			else if (uSourceNumber > uMaxSources * 0.85)
				uDropLimit = 1;
		}

	//	Count how many sources are dropped
		uint16	uDroppedTotal = 0;
		uint16	uDroppedLowToLowID = 0;
		uint16	uDroppedNNP = 0;
		uint16	uDroppedQueueFull = 0;
		uint16	uDroppedHighQR = 0;
        
		for (uint32 dwSlotIndex = 0; dwSlotIndex < SOURCESSLOTS; dwSlotIndex++)
		{
			if (!m_srcLists[dwSlotIndex].IsEmpty())
			{
				for (pos1 = m_srcLists[dwSlotIndex].GetHeadPosition();pos1 != NULL;)
				{
					pSource = m_srcLists[dwSlotIndex].GetNext(pos1);

					if (pSource == NULL)
						continue;

					bAlreadyCounted = false;
					bIsClientConnected = true;
					bReaskDownload = false;
					uint32	dwFileReaskTime = FILEREASKTIME;

					switch (pSource->GetDownloadState())
					{
						case DS_DOWNLOADING:
						{
							dwCurrentDataRate = pSource->DownloadProcess(dwReduceDownload);
						//	Since client can be dropped due timeout we need to check the state
							if (pSource->GetDownloadState() != DS_DOWNLOADING)
								break;
						//	Needed to update the source count
						//	GetTransferringSrcCount() isn't updated inside this function.
							dwNumTransferringSrcs++;
							dwDataRate += dwCurrentDataRate;
							break;
						}
						case DS_BANNED:
						{
							if (!bAlreadyCounted)
							{
								uBannedSrcCount++;
								bAlreadyCounted = true;
							}
							break;
						}
						case DS_ERROR:
						{
							if (!bAlreadyCounted)
							{
								uErrorSrcCount++;
								bAlreadyCounted = true;
							}
						//	Formerly this was executed only if the source was not already counted (inside if-clause above).
							pSource->Disconnected();
							break;
						}
						case DS_CONNECTED:
						{
#ifdef OLD_SOCKETS_ENABLED
						//	If the request socket doesn't exist or isn't connected
							if (!(pSource->m_pRequestSocket && pSource->m_pRequestSocket->IsConnected()))
							{
							//	It was DS_NONE before but that keeps for some time problematic clients.
							//	There is no need to do that as there will be no change (from our experience).
								pSource->SetDownloadState(DS_ERROR);

								if (!bAlreadyCounted)
								{
									uUnknownSrcCount++;
									bAlreadyCounted = true;
								}
								break;
							}
#endif //OLD_SOCKETS_ENABLED

							if (dwCurTick - pSource->GetEnteredConnectedState() > CONNECTION_TIMEOUT + 20000)
							{
								if (!bAlreadyCounted)
								{
								//	Actually this is a bit wrong because ALL timed out sources are dropped so there are none left afterwards.
								//	Logical this counter represents the number of sources which had timed out since the last processing.
									uTimeoutSrcCount++;
									bAlreadyCounted = true;
								}
								g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
								uSourceNumber--;
								break;
							}
							if (!bAlreadyCounted)
							{
								uConnectedSrcCount++;
								bAlreadyCounted = true;
							}
							bReaskDownload = true;
							break;
						}
						case DS_LOWTOLOWIP:
						{
							if (uDroppedTotal < uDropLimit)
							//	Prefere to drop these Low2Low before all other
							{
								g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
								uSourceNumber--;
								uDroppedTotal++;
								uDroppedLowToLowID++;
								break;
							}
							if (!bAlreadyCounted)
							{
								uLowToLowIDSrcCount++;
								bAlreadyCounted = true;
							}
#ifdef OLD_SOCKETS_ENABLED
					 	//	If we now have a high ip we can ask
							if (g_eMuleApp.m_pServerConnect->IsLowID())
							{
								break;
							}
#endif //OLD_SOCKETS_ENABLED
						}
						case DS_NONEEDEDPARTS:
						{
							if (uDroppedTotal < uDropLimit
							//	Prefere to drop Low2Low before NNP
								&& m_iSrcLowToLow == uDroppedLowToLowID )
							{
								if (!pSource->SwapToAnotherFile(NULL))
								{
									g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
									uSourceNumber--;
									uDroppedTotal++;
									uDroppedNNP++;
									break;
								}
							}
							if (!bAlreadyCounted)
							{
								uNNPSrcCount++;
								bAlreadyCounted = true;
							}

						//	Doubled reasktime for no needed parts - save connections and traffic
						//	NNP sources will be checked filereasktime*4 when 3/4 of max. sockets are open
							if (bSocketsUsageIsHigh)
								dwFileReaskTime = FILEREASKTIME * 2;

							if (!((!pSource->GetLastAskedTime()) || (dwCurTick - pSource->GetLastAskedTime()) > dwFileReaskTime * 2))
								break;
						}
						case DS_ONQUEUE:
						{
							uint16	uQueueRank = pSource->GetRemoteQueueRank();

							if (pSource->IsRemoteQueueFull())
							{
								if (uDroppedTotal < uDropLimit
								//	Prefere to drop Low2Low and NNP before QueueFull
									&& m_iSrcLowToLow == uDroppedLowToLowID
									&& m_iSrcNNP == uDroppedNNP
								//	QueueFull sources get a second chance
									&& pSource->GetAskedCountDown() > 1 )
								{
									g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
									uSourceNumber--;
									uDroppedTotal++;
									uDroppedQueueFull++;
									break;
								}
								if (!bAlreadyCounted)
								{
									uQueueFullSrcCount++;
									bAlreadyCounted = true;
								}
							}
							else
							{
							//	Check remote queue position (HighQR)
								if (uQueueRank > QUEUERANK_HIGH)
								{
									if (uDroppedTotal < uDropLimit
										&& g_eMuleApp.m_pGlobPrefs->GetDropHighestQREnabled()
									//	Prefere to drop Low2Low and NNP before HighQR
										&& m_iSrcLowToLow == uDroppedLowToLowID
										&& m_iSrcNNP == uDroppedNNP
									//	Drop if the source has worst QR
										&& uQueueRank == m_uHighestRemoteQueueRank )
									{
										g_eMuleApp.m_pDownloadQueue->RemoveSource(pSource);
										uSourceNumber--;
										uDroppedTotal++;
										uDroppedHighQR++;
										break;
									}
									if (!bAlreadyCounted)
									{
										uHighQRSrcCount++;
										bAlreadyCounted = true;
									}
								}
								else if (!bAlreadyCounted)
								{
									uOnQueueSrcCount++;
									bAlreadyCounted = true;
								}
							}

							if (uQueueRank != 0)
							{
							//	Adjust reasktime when many connections used: QR<250 = 43:20 min, QR>250 = 54:10 min
								if (bSocketsUsageIsHigh)
								{
									if (uQueueRank > QUEUERANK_LOW)
										dwFileReaskTime = FILEREASKTIME * 2.5;
									else
										dwFileReaskTime = FILEREASKTIME * 2;
								}
							//	Findout best and worst QR
								if (uQueueRank > uHighestQR)
									uHighestQR = uQueueRank;
								if (uQueueRank < uLowestQR)
									uLowestQR = uQueueRank;
							}

						//	LANCAST (moosetea) - We reask much faster if lancast user, UDP happens in 10 seconds
							if (pSource->IsOnLAN())
							{
								if ( (!pSource->GetLastAskedTime())
								     || (dwCurTick - pSource->GetLastAskedTime()) > LANCASTFILEREASKTIMEUDP )
								{
									pSource->UDPReaskForDownload();
								}
							}
							else
							{
#ifdef OLD_SOCKETS_ENABLED
								if ( g_eMuleApp.m_pServerConnect->IsConnected()
								     && ( (!pSource->GetLastAskedTime())
								          || (dwCurTick - pSource->GetLastAskedTime()) > dwFileReaskTime - 20000 ) )
								{
									pSource->UDPReaskForDownload();
								}
#endif
							}

#ifdef OLD_SOCKETS_ENABLED
						//	emulates a "Connecting via server..." without request socket timeout
							if ( g_eMuleApp.m_pServerConnect->IsConnected()
							     && (pSource->GetLastAskedTime())
							     && ((dwCurTick - pSource->GetLastAskedTime()) > (FILEREASKTIME - L2HAC_PREPARE_PRECEDE))
							     && pSource->HasLowID()
							     && pSource->GetL2HACTime()
							     && !g_eMuleApp.m_pServerConnect->IsLowID() )
							{
								pSource->SetDownloadState(DS_WAITCALLBACK);
							}
#endif
						}
						case DS_CONNECTING:
						{
							if (!bAlreadyCounted)
							{
								uConnectingSrcCount++;
								bAlreadyCounted = true;
							}
						}
						case DS_TOOMANYCONNS:
						{
							if (!bAlreadyCounted)
							{
								uTooManyConnsSrcCount++;
								bAlreadyCounted = true;
							}
						}
						case DS_NONE:
						{
							if (!bAlreadyCounted)
							{
								uNoUploadSrcCount++;
								bAlreadyCounted = true;
							}
							bReaskDownload = true;
						}
						case DS_WAITCALLBACK:
						{
							if (!bAlreadyCounted)
							{
								uConnViaServerSrcCount++;
								bAlreadyCounted = true;
							}
							bReaskDownload = true;
							break;
						}
						case DS_REQHASHSET:
						{
							uReqHashSrcCount++;
							break;
						}
						default:
						{
							uUnknownSrcCount++;
						}
					}
#ifdef OLD_SOCKETS_ENABLED
					if (bReaskDownload && g_eMuleApp.m_pServerConnect->IsConnected())
					{
					//	LANCAST (moosetea) - Because reask much faster if lancast user ie we may reask as soon as we get disconnected,
					//	we only use TCP on a LAN, because emule may try to send a TCP request and UDP request at the same time!
						if (pSource->IsOnLAN())
							dwFileReaskTime = LANCASTFILEREASKTIMETCP;

						if (!pSource->GetLastAskedTime() || (dwCurTick - pSource->GetLastAskedTime()) > dwFileReaskTime)
							bIsClientConnected = pSource->AskForDownload();
					}
#endif
					pSource = NULL;
				}
			}
		}
	//	Check if at least 1 source has been dropped this time
		if (uDroppedTotal != 0)
		{
		//	Need to ensure that next dropping is properly delayed in every case.
			m_dwLastPurgeTime = dwCurTick;
		/*	Producing many lines - so uncomment it only if u want to look into details
			AddDebugLogLine( RGB_LOG_DIMMED + _T("Dropped %u/%u sources from file '%s': %u/%u L2L, %u/%u NNP, %u/%u QF, %u/%u HQR(%u) [%u < QR < %u]")
				, uDroppedTotal, (uSourceNumber + uDroppedTotal), GetFileName()
				, uDroppedLowToLowID, (uLowToLowIDSrcCount + uDroppedLowToLowID)
				, uDroppedNNP, (uNNPSrcCount + uDroppedNNP)
				, uDroppedQueueFull, (uQueueFullSrcCount + uDroppedQueueFull)
				, uDroppedHighQR, (uHighQRSrcCount + uDroppedHighQR)
				, m_uHighestRemoteQueueRank, uLowestQR, uHighestQR );									
		*/
		}

	//	Set the members using temp vars.  We have to use temporary vars so that the disp counts don't get
	//	screwed up during processing.  We don't need to count this, cause we can get a transferring
	//	sources over m_downloadingSourceList.GetCount()
		m_iSrcNNP = uNNPSrcCount;
		m_iSrcOnQueue = uOnQueueSrcCount;
		m_iSrcHighQR = uHighQRSrcCount;
		m_iSrcConnecting = uConnectingSrcCount;
		m_iSrcTooManyConns = uTooManyConnsSrcCount;
		m_iSrcConnected = uConnectedSrcCount;
		m_iSrcTimeout = uTimeoutSrcCount;
		m_iSrcConnViaServer = uConnViaServerSrcCount;
		m_iSrcUnknown = uUnknownSrcCount;
		m_iSrcRecHash = uReqHashSrcCount;
		m_iSrcLowToLow = uLowToLowIDSrcCount;
		m_iSrcQueueFull = uQueueFullSrcCount;
		m_iSrcNoUpload = uNoUploadSrcCount;
		m_iSrcBanned = uBannedSrcCount;
		m_iSrcError = uErrorSrcCount;
		m_iNumTransferringSrcs = dwNumTransferringSrcs;
		m_uHighestRemoteQueueRank = uHighestQR;
		m_uLowestRemoteQueueRank = uLowestQR;
		m_dwDataRate = dwDataRate;

	//	A4AF management
	//	Swap sources with NNS status if possible
		bool bIsNNS;

		if (((!m_LastNoNeededCheck) || (dwCurTick - m_LastNoNeededCheck) > 30000))
		{
			m_LastNoNeededCheck = dwCurTick;
			for (int sl = 0; sl < SOURCESSLOTS; sl++)
			{
				if (!m_srcLists[sl].IsEmpty())
				{
					for (pos1 = m_srcLists[sl].GetHeadPosition(); pos1 != NULL;)
					{
						pSource = m_srcLists[sl].GetNext(pos1);
						bIsNNS = false;
					//	Check if current source has a part for this file
						if (pSource->GetDownloadState() == DS_NONEEDEDPARTS)
						{
						//	Check if A4AF Auto file is enable
							if (g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile() == NULL)
							//	A4AF auto is disable => switch to any file
								pSource->SwapToAnotherFile(NULL);
							else if (g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile() != this)
							{
							//	A4AF auto is enable & isn't current file => switch to A4AF auto file
								if (pSource->m_otherNoNeededMap.IsEmpty())
								{
									pSource->SwapToAnotherFile(g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile());
								}
								else
								{
									bIsNNS = pSource->m_otherNoNeededMap.Lookup(g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile(), dwLastNoNeededCheckTime);
									if (!bIsNNS || (dwCurTick - dwLastNoNeededCheckTime) > FILEREASKTIME)
									{
										pSource->SwapToAnotherFile(g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile());
									}
								}
							}
						}
					}
				}
			}
		}

	//	If this "File" is A4AF auto & A4AF list isn't empty,
	//	then check A4AF list and swap source to "this", if souce isn't NNS
		if (g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile() == this && !m_A4AFsrcList.IsEmpty())
		{
		//	Use a two-pointer loop to prevent exception due to swaping (delete)
			for (pos1 = m_A4AFsrcList.GetHeadPosition(); pos1 != NULL;)
			{
				pSource = m_A4AFsrcList.GetNext(pos1);
				if (pSource != NULL)
				{
					if (pSource->m_otherNoNeededMap.IsEmpty())
					{
						pSource->SwapToAnotherFile(this);
					}
					else
					{
						bIsNNS = pSource->m_otherNoNeededMap.Lookup(this,dwLastNoNeededCheckTime);
						if (!bIsNNS || (dwCurTick - dwLastNoNeededCheckTime) > FILEREASKTIME)
						{
							pSource->SwapToAnotherFile(this);
						}
					}
				}
			}
		}

		m_iSrcA4AF = m_A4AFsrcList.GetCount();

	//	Check if we want new sources from server
	//	1. connection state will be checked in DL-queue manager, so we don't need to do it here
	//	2. we can call SetRequiredSourcesRefresh() so many time as we want, cause this function
	//	   checks if the file exists.
		if ( ( (!m_dwLastFileSourcesRequestTime) || (dwCurTick - m_dwLastFileSourcesRequestTime) > SERVERREASKTIME)
			&& g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFileSoft() > uSourceNumber )
		{
		//	Local server
			m_dwLastFileSourcesRequestTime = dwCurTick;
			g_eMuleApp.m_pDownloadQueue->SetRequiredSourcesRefresh(this);
		}
	//	Call SaveSources() only one time (in case there isn't DL)
#ifdef OLD_SOCKETS_ENABLED
		if (dwIteration && g_eMuleApp.m_pGlobPrefs->SLSEnable() && m_downloadingSourceList.IsEmpty() && g_eMuleApp.m_pServerConnect->IsConnected())
		{
			m_sourcesaver.Process(this, g_eMuleApp.m_pGlobPrefs->SLSMaxSourcesPerFile());
		}
#endif //OLD_SOCKETS_ENABLED

	//	Calculate data rate, set limit etc. (every 5 sec)
		m_uProcessCounter++;
		if (m_uProcessCounter == 5)
		{
			m_uProcessCounter = 0;
			UpdateCompletedInfos();
			UpdateDisplayedInfo();
		}
	}

//	For status category filters check the status change
//	1. is transfering <-> is not transfering
//	2. is on the queue <-> is not on the queue
	if ( (nOldTransSourceCount == 0 && GetTransferringSrcCount() != 0)
		|| (nOldTransSourceCount != 0 && GetTransferringSrcCount() == 0)
	     || (iOldSrcOnQueueCount == 0 && GetOnQueueSrcCount() != 0)
	     || (iOldSrcOnQueueCount != 0 && GetOnQueueSrcCount() == 0) )
	{
		UpdateGUIAfterStateChange();
	}

	return m_dwDataRate;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::AddSources(CMemFile& pSources, uint32 iServerIP, uint16 iServerPort)
{
	EMULE_TRY

//	If this part file is paused or complete/completing or erroneous, no sources needed any more
	EnumPartFileStatuses	eFileStatus = GetStatus();

	if (eFileStatus == PS_STOPPED || eFileStatus == PS_COMPLETE
		|| eFileStatus == PS_COMPLETING || eFileStatus == PS_ERROR)
	{
		return;
	}

	byte byteCount;
	byte byteLowIDDropped = 0;
	byte bytePossibleSources = 0;

	pSources.Read(&byteCount, 1);

//	If there is a need for more sources just remove NNS
	int iSourceCountPlusNew = GetSourceCount() + byteCount;

	if ( iSourceCountPlusNew > g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile()
		&& (m_dwLastPurgeTime + PURGE_TIME) < ::GetTickCount() )
	{
		RemoveNoNeededSources(iSourceCountPlusNew - g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile());
	}

	uint32 dwUserID;
	uint16 iUserPort;

	for (int i = 0; i < static_cast<int>(byteCount); i++)
	{
		pSources.Read(&dwUserID,4);
		pSources.Read(&iUserPort,2);

#ifdef OLD_SOCKETS_ENABLED

	//	Check first if we are this source
		if (g_eMuleApp.m_pServerConnect->GetClientID() < 0x1000000 && g_eMuleApp.m_pServerConnect->IsConnected())
		{
			if ( g_eMuleApp.m_pServerConnect->GetClientID() == dwUserID &&
				 g_eMuleApp.m_pGlobPrefs->GetPort() == iUserPort &&
				 inet_addr(g_eMuleApp.m_pServerConnect->GetCurrentServer()->GetFullIP()) == iServerIP )
			{
				continue;
			}
		}
		else if ( g_eMuleApp.m_pServerConnect->GetClientID() == dwUserID &&
				  g_eMuleApp.m_pGlobPrefs->GetPort() == iUserPort )
		{
			continue;
		}
		else if ( dwUserID < 0x1000000 &&
				  !g_eMuleApp.m_pServerConnect->IsLocalServer(iServerIP, iServerPort) )
		{
			byteLowIDDropped++;
			continue;
		}
#endif //OLD_SOCKETS_ENABLED

		if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > GetSourceCount())
		{
			bytePossibleSources++;
			CUpDownClient* pNewSource = g_eMuleApp.m_pDownloadQueue->CheckAndAddSource(this, dwUserID, iUserPort, iServerIP, iServerPort);
			if (pNewSource)
				pNewSource->SetUserName(GetResString(IDS_SERVER_SOURCE));
		}
		else
			break;
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	NewSrcPartsInfo & UpdateAvailablePartsCount & CompleteSourcesCount merged (CPU load)
void CPartFile::NewSrcPartsInfo()
{
	EMULE_TRY

//	Cache part count
	uint32		dwPartCount = GetPartCount();

	CUpDownClient*			pSource;
	POSITION				pos;
	uint16					uNewCompleteSrcCount = 0;
	CArray<uint16,uint16>	PartFrequencyArray;

	PartFrequencyArray.SetSize(dwPartCount);

	for (int sl = 0; sl < SOURCESSLOTS; ++sl)
	{
		if (!m_srcLists[sl].IsEmpty())
		{
			for (pos = m_srcLists[sl].GetHeadPosition(); pos != NULL;)
			{
				pSource = m_srcLists[sl].GetNext(pos);
				if (pSource)
				{
					uint32		dwSourcePartsCounter = 0;
				//	Count available parts from source
					for (uint32 i = 0; i < dwPartCount; i++)
					{
						if (pSource->IsPartAvailable(i))
						{
							PartFrequencyArray[i] += 1;
							dwSourcePartsCounter++;
						}
					}
				//	Count full sources
					if (dwSourcePartsCounter == dwPartCount)
					{
						//ASSERT(pSource->IsCompleteSource());
						uNewCompleteSrcCount++;
					}
				}
			}
		}
	}

	m_nLastCompleteSrcCount = uNewCompleteSrcCount;

//	Increase size if necessary
	if (static_cast<uint32>(m_srcPartFrequencies.GetSize()) < dwPartCount)
	{
		m_srcPartFrequencies.SetSize(dwPartCount);
	}

	uint16	nNewMaxSrcPartFrequency = 0;
	uint32	dwNewAvailablePartsCount = 0;
//	Count available parts over all sources
	for (uint32 i = 0; i < dwPartCount; i++)
	{
		m_srcPartFrequencies[i] = PartFrequencyArray[i];

		if (m_srcPartFrequencies[i] > 0)
		{
			dwNewAvailablePartsCount++;

			if (m_srcPartFrequencies[i] > nNewMaxSrcPartFrequency)
			{
				nNewMaxSrcPartFrequency = m_srcPartFrequencies[i];
			}
		}
	}

	m_MaxSrcPartFrequency = nNewMaxSrcPartFrequency;
	m_dwAvailablePartsCount = dwNewAvailablePartsCount;

	if (dwNewAvailablePartsCount == dwPartCount)
	{
		lastseencomplete = CTime::GetCurrentTime();
	}

	PartFrequencyArray.RemoveAll();
	UpdateDisplayedInfo();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Intelligent Chunk Request(ICR)
uint16 CPartFile::GetNextRequiredPart(CUpDownClient *pSource)
{
#ifdef OLD_SOCKETS_ENABLED
	EMULE_TRY

	if (pSource == NULL)
		return 0xFFFF;

	uint32	dwGoodPart = pSource->GetLastDownPartAsked();

	if ((dwGoodPart != 0xFFFF) && (GetGapsInPart(dwGoodPart, NULL, 1) != 0))
	{
	//	AddDebugLogLine(_T("Priority 0 (always choose currently downloaded part until it will be finished). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
		return dwGoodPart;
	}

	const uint32	dwPartCnt = GetPartCount();
	std::vector<uint32> PartsFilledSizeVector(dwPartCnt, NOT_AVAILABLE); // mark parts as not available to save CPU load
	uint32			dwNumAvailableNeededParts = 0;
	uint32			dwNumPartiallyBlockedParts = 0;

//	Scan for requested chunks in order to find out how much they're filled
	for (uint32 i = 0; i < dwPartCnt; i++)
	{
		if (pSource->IsPartAvailable(i))
		{
			if ((PartsFilledSizeVector[i] = GetPartLeftToDLSize(i)) != 0)
				dwNumAvailableNeededParts++;
		}
	}
	if (dwNumAvailableNeededParts == 0)
		return 0xFFFF;

//	Scan DownloadList for Blocked parts
	std::vector<byte>	abyteBlockedParts(dwPartCnt, FREE_TO_DL);
	uint32				dwTmp;

	for (POSITION pos = requestedblocks_list.GetHeadPosition(); pos != NULL;)
	{
		Requested_Block_Struct *pReqBlock = requestedblocks_list.GetNext(pos);

		dwTmp = pReqBlock->m_dwStartOffset / PARTSIZE;
		if ((abyteBlockedParts[dwTmp] == PARTIALLY_BLOCKED) || (abyteBlockedParts[dwTmp] == FULLY_BLOCKED))
			continue;
	//	Let's check if all gaps in part were already requested.
		if (GetGapsInPart(dwTmp, NULL, 1) == 0)
		{
		//	If we can't get this part from this client, reduce the count
			if ((PartsFilledSizeVector[dwTmp] != 0) && (PartsFilledSizeVector[dwTmp] != NOT_AVAILABLE))
				dwNumAvailableNeededParts--;
		//	No gaps found => fully blocked
			abyteBlockedParts[dwTmp] = FULLY_BLOCKED;
		}
		else
		{
		//	Count the blocked parts only in case if they are available & was not counted before
			if (PartsFilledSizeVector[dwTmp] != NOT_AVAILABLE)
				dwNumPartiallyBlockedParts++;
			abyteBlockedParts[dwTmp] = PARTIALLY_BLOCKED;
		}
	}

//	All arrays/lists self-destruct in their destructors on return
	std::vector<uint32>	adwRareParts(dwPartCnt);	// priority 2
	CList<int>		moviePartsList(3);				// priority 1 => only first+last parts
	std::vector<uint32>	adwRandomParts(dwPartCnt);	// priority 4
	const uint32	dwPartCnt1 = dwPartCnt - 1, dwLastPartSize = GetLastPartSize();
	const uint32	dwPreviewMode = GetMovieMode();
//	Mode 0=NoPreview, 1=AVI(1F,1L), 4=MPG(1F), 5=MPG(2F), 7=AVI(1F,2L) F=First,L=Last

//	Define condition of rareness
//	Reduce this threshold for low number of sources to avoid downloading of the same part from several sources
	uint32		dwRarestPart = (m_MaxSrcPartFrequency > 11) ? 4 : (m_MaxSrcPartFrequency / 4 + 1);

//	Define a shortest-to-complete condition, cause it will be inverted if all needed parts are downloaded
	const bool bShortestToComplete = (dwNumAvailableNeededParts > dwNumPartiallyBlockedParts);

	uint32		dwMostCompletedRarePartIdx, dwMostCompletedPartIdx;
	uint32		dwMostCompletedRarePartSz = PARTSIZE;
	uint32		dwMostCompletedPartSz = (bShortestToComplete) ? PARTSIZE : 0;
	uint32		dwRareRank = ~0, dwRandomRank = ~0;
	uint32		dwRareFilledRand = 0, dwRarePartCnt = 0, dwRandomPartCnt = 0;

//	For each part in the part file...
	for (uint32 i = 0; i < dwPartCnt; i++)
	{
	//	If the part is available and it hasn't been completed...
		if (PartsFilledSizeVector[i] != NOT_AVAILABLE && abyteBlockedParts[i] != FULLY_BLOCKED && PartsFilledSizeVector[i] != 0)
		{
		//	------------------------ Rare Parts -------------------------
			if (dwRarestPart >= (dwTmp = m_srcPartFrequencies[i]))
			{
				if (PartsFilledSizeVector[i] == PARTSIZE || ((i == dwPartCnt1) && PartsFilledSizeVector[i] == dwLastPartSize))
				{
					if (dwTmp <= dwRareRank)	//select the rarest parts
					{
						if (dwTmp < dwRareRank)
						{
							dwRareRank = dwTmp;
							dwRarePartCnt = 0;
						}
						adwRareParts[dwRarePartCnt++] = i;
					}
				}
				else if (PartsFilledSizeVector[i] < dwMostCompletedRarePartSz)
				{
					dwMostCompletedRarePartSz = PartsFilledSizeVector[i];
					dwMostCompletedRarePartIdx = i;
					dwRareFilledRand = dwTmp;
				}
			}
		//	--------------------------- Movie or Archive ---------------------------
			switch (dwPreviewMode)
			{
				case 7:	//	AVI(1F,2L) downloading sequence: chunk 0, last, next to last
				{
					if (dwPartCnt > 2 && (i == dwPartCnt - 2))
						moviePartsList.AddHead(i);
					if (dwPartCnt > 2 && (i == dwPartCnt - 1))
						moviePartsList.InsertAfter(moviePartsList.GetHeadPosition(), i);
					if (i == 0)
						moviePartsList.AddHead(0);
					break;
				}
				case 5:	//	MPG(2F) downloading sequence: chunk 0, chunk 1
				{
					if (i == 1 && dwPartCnt > 1)
						moviePartsList.AddHead(1);
					if (i == 0)
						moviePartsList.AddHead(0);
					break;
				}
				case 1:	//	AVI(1F,1L) downloading sequence: chunk 0, last chunk
				{
					if (dwPartCnt > 1 && (i == dwPartCnt1))
						moviePartsList.AddHead(i);
				}
				case 4:	//	MPG(1F)
				{
					if (i == 0)
						moviePartsList.AddHead(0);
				}
			}
		//	--------------------- widespreaded parts ---------------------
		//	Check blocked parts - don't download already downloading parts unless there's something else
			if ((abyteBlockedParts[i] == PARTIALLY_BLOCKED) && bShortestToComplete)
				continue;

			if (PartsFilledSizeVector[i] == PARTSIZE || ((i == dwPartCnt1) && PartsFilledSizeVector[i] == dwLastPartSize))
			{
			//	Fresh part
				if ((dwTmp = m_srcPartFrequencies[i]) <= dwRandomRank)
				{
					if (dwTmp < dwRandomRank)	//select the rarest parts among them
					{
						dwRandomRank = dwTmp;
						dwRandomPartCnt = 0;
					}
					adwRandomParts[dwRandomPartCnt++] = i;
				}
			}
			else
			{
			//	Invert condition shortest-to-complete condition if all parts are partially blocked:
			//	1) select the most completed part if there're chunks not in downloading state
			//	2) select the most incomplete part if all available parts from this source're in downloading state
				if (!((PartsFilledSizeVector[i] < dwMostCompletedPartSz) ^ bShortestToComplete))
				{
					dwMostCompletedPartIdx = i;
					dwMostCompletedPartSz = PartsFilledSizeVector[i];
				}
			}
		}
	}

//	Select final result
//	---------------------------------- Movies Part  -----------------------------------------------------
//	The preview parts have highest priority in order to provide the user information about the file as soon as possible
//	This will lead to the faster decision about futher download
	if (!moviePartsList.IsEmpty())
	{
	//	Select the most appropriate part (for the case when all preview parts are already in downloading)
		dwGoodPart = moviePartsList.GetTail();
	//	Avoid several downloads for the same part if there's a preview part which isn't in downloading state yet
		do
		{
			dwTmp = moviePartsList.RemoveTail();
			if (abyteBlockedParts[dwTmp] != PARTIALLY_BLOCKED)
			{
				dwGoodPart = dwTmp;
				break;
			}
		} while(!moviePartsList.IsEmpty());
	//	AddDebugLogLine(_T("Priority 1 (Preview part). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
	}
//	---------------------------------- Rare Parts ------------------------------------------------------
	else if ((dwRarePartCnt != 0) || (dwRareFilledRand != 0))
	{
	//	Download an incomplete part first, but if there're more than one source for an incomplete part
	//	and there's a fresh part with only one source, start downloading of the unique fresh part
		if ((dwRareFilledRand != 0) && ((dwRareFilledRand == 1) || (dwRareRank != 1)))
		{
			dwGoodPart = dwMostCompletedRarePartIdx;
//			AddDebugLogLine(_T("Priority 2 (Rarest unfinished part). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
		}
		else
		{
		//	If there is only one rare part, then take it, otherwise take random one
			if (dwRarePartCnt == 1)
				dwTmp = 0;
			else
				dwTmp = static_cast<uint32>(rand() * (dwRarePartCnt - 1) + RAND_MAX / 2) / RAND_MAX;
			dwGoodPart = adwRareParts[dwTmp];
//			AddDebugLogLine(_T("Priority 3 (Rarest unrequested part). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
		}
	}
//	--------------------------------------- Requested ----------------------------------------------------
//	Try to complete an UnfinishedPart and MostDownloaded as soon as possible to become a new source
//	it doesn't matter which one to download, if there are same filled parts.
	else if ((dwMostCompletedPartSz != PARTSIZE) && (dwMostCompletedPartSz != 0))
	{
		dwGoodPart = dwMostCompletedPartIdx;
	//	AddDebugLogLine(_T("Priority 4 (Widespread unfinished part). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
	}
//	-------------------------------- Unrequested & Random -------------------------------------------------
	else if (dwRandomPartCnt != 0)
	{
		if (dwRandomPartCnt == 1)
			dwTmp = 0;
		else
			dwTmp = static_cast<uint32>(rand() * (dwRandomPartCnt - 1) + RAND_MAX / 2) / RAND_MAX;
		dwGoodPart = adwRandomParts[dwTmp];
	//	AddDebugLogLine(_T("Priority 5 (Widespread unrequested part). File: %s, Part: %u"), this->GetFileName(), dwGoodPart);
	}
	else
	{
		dwGoodPart = 0xFFFF;
	}

/*
	CString strDebug;

	strDebug.Format(_T("ICR-Test: File: %s, A-parts : %u, BP-Parts %u. Choosen Part: %u"),
						GetFileName(), dwNumAvailableNeededParts, dwNumPartiallyBlockedParts, dwGoodPart);
	if (dwGoodPart < dwPartCnt)
	{
		strDebug.AppendFormat(_T(", P-Freq: %u, blocked %s"), m_srcPartFrequencies[dwGoodPart], ((abyteBlockedParts[dwGoodPart] == PARTIALLY_BLOCKED)? _T("yes"):_T("no")));
	}
	AddDebugLogLine(strDebug);
*/

	return dwGoodPart;

	EMULE_CATCH
#endif //OLD_SOCKETS_ENABLED
	return 0xFFFF;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CPartFile::GetPartFilledSize(uint32 dwPartNum) const
{
	uint32 dwStart = dwPartNum * PARTSIZE;
	uint32 dwEnd = ((GetFileSize() - dwStart) > PARTSIZE) ? (dwStart + PARTSIZE - 1) : (GetFileSize() - 1);

	uint32 dwLeftSize = (dwEnd - dwStart + 1) - GetPartLeftToDLSize(dwPartNum);

	ASSERT(dwLeftSize <= PARTSIZE);
	return dwLeftSize;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::RemoveBlockFromList(uint32 start, uint32 dwEnd)
{
	EMULE_TRY

	POSITION pos1, pos2;
	for (pos1 = requestedblocks_list.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		if (requestedblocks_list.GetNext(pos1)->m_dwStartOffset <= start && requestedblocks_list.GetAt(pos2)->m_dwEndOffset >= dwEnd)
			requestedblocks_list.RemoveAt(pos2);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::RemoveAllRequestedBlocks(void)
{
	EMULE_TRY

	requestedblocks_list.RemoveAll();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::CompleteFile(bool bIsHashingDone)
{
	EMULE_TRY

	StopFile(false);

//	Show completing instead of paused
	SetStatus(PS_COMPLETING);

	if (g_eMuleApp.m_pGlobPrefs->ShowCatTabInfo())
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();

	UpdateDisplayedInfo(true);

//	If the part file isn't already hashed...
	if (!bIsHashingDone)
	{
	//	Create a thread to hash the part file in the background
		m_dwDataRate = 0;

		CString	partFileNameBase = m_strPartMetFileName.Left(m_strPartMetFileName.GetLength() - 4);
		CHashFileThread	*addFileThread;

		addFileThread = (CHashFileThread*)AfxBeginThread( RUNTIME_CLASS(CHashFileThread),
		                THREAD_PRIORITY_BELOW_NORMAL + g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority(),
		                0,
		                CREATE_SUSPENDED );
		addFileThread->SetValues(0, m_strTempDir, partFileNameBase, this);
		addFileThread->ResumeThread();
		return;
	}
	else
	{
		CWinThread *pThread = AfxBeginThread((AFX_THREADPROC)CompleteThreadProc, this, THREAD_PRIORITY_BELOW_NORMAL + g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority(), 0, 0);

		if (pThread == NULL)
		{
			throw CString(_T("error creating file complete thread"));
		}
	}
	GetAvgDataRate(true);

	if (g_eMuleApp.m_pGlobPrefs->ShowCatTabInfo())
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();

	UpdateDisplayedInfo(true);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
static void GenerateNewFileName(const CString& outputDir, CString& newname)
{
	ASSERT(!newname.IsEmpty());

	int namecount = 0;

	TCHAR p_fname[_MAX_FNAME];
	TCHAR p_fext[_MAX_EXT];
	_tsplitpath(newname, NULL, NULL, p_fname, p_fext);

	CString fname(p_fname);
	CString fext(p_fext);

//	Search for matching ()s and check if it contains a number inside
	if (fname.Right(1) == ')')
	{
		int ll = fname.GetLength();
		int ob = fname.ReverseFind( '(' );
	//	Check for ) existence or () situation
		if (ob != -1 && ob + 2 != ll)
		{
			bool found = true;
			for (int i = ob + 1; i < ll - 1; i++)
			{
			//	Check for digits inside brackets
				if (!_istdigit(fname[i]))
				{
					found = false;
					break;
				}
			}
			if (found)
			{
				namecount = _ttoi(fname.Mid(ob + 1, ll - ob - 2));
				fname.Truncate(ob);
			}
		}
	}

	do
	{
		namecount++;
		newname.Format(_T("%s\\%s(%d)%s"), outputDir, fname, namecount, fext);
	}
	while (PathFileExists(newname));
}

//	Use threads for file completion
BOOL CPartFile::PerformFileComplete(void)
{
	EMULE_TRY

//	Removes any old fakes.rar that may exist in shared directories before completing the newest fakes.rar
	if (IsFakesDotRar())
	{
		CStringList	strSharedDirsList;
		CString		strFakesFile;

		strFakesFile.Format(_T("%s\\fakes.rar"), g_eMuleApp.m_pGlobPrefs->GetIncomingDir());
		if (PathFileExists(strFakesFile))
			if(!::DeleteFile(strFakesFile))
				AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Couldn't delete fakes.rar (%s)"), strFakesFile);


		g_eMuleApp.m_pGlobPrefs->SharedDirListCopy(&strSharedDirsList);

		for (POSITION pos = strSharedDirsList.GetHeadPosition(); pos != NULL; )
		{
			strFakesFile.Format(_T("%s\\fakes.rar"), strSharedDirsList.GetNext(pos));
			if (PathFileExists(strFakesFile))
				if(!::DeleteFile(strFakesFile))
					AddDebugLogLine(RGB_LOG_ERROR_TXT _T("Couldn't delete fakes.rar (%s)"), strFakesFile);
		}
	}

//	Get current category
	EnumCategories eCatID = GetCatID();

	CString partfilename = m_strFullName.Left(m_strFullName.GetLength() - 4);

	CString newname, strOutputDir = GetOutputDir();
	newname.Format(_T("%s\\%s"), strOutputDir, g_eMuleApp.StripInvalidFilenameChars(GetFileName()));

//	Create all required directories
	CreateAllDirectories(&strOutputDir);

	bool bRenamed = false;
	if (PathFileExists(newname))
	{
		bRenamed = true;
		GenerateNewFileName(strOutputDir, newname);
	}

//	Save full .met file name for temp. file deletions, because a name is altered before it
	CString	strTempFullName = m_strFullName;

//	Make copy if files are on different partitions and a file isn't small (> 1 Mb)
//	Small files will be copied by system routine which blocks uploading start of the same file
	if ( (_toupper(partfilename[0]) != _toupper(newname[0])) &&
		(partfilename[1] == _T(':') && newname[1] == _T(':')) && (GetFileSize() > 1024*1024) )
	{
		bool	bException = false;
		CFile	fInputFile, fOutputFile;
		CString	strTempNew = newname;

		strTempNew += _T(".$$$");
		try
		{
		//	We going to write everything into the temporary file at the same place the newname located
		//	and after we finished - rename, this will take care of synchronization problem with SharedFiles
			if ( !fInputFile.Open( partfilename, CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary
			                       | CFile::osSequentialScan ) )
			{
			//	Let's catch to care about closing everything
				throw CString(_T("can't open file for reading (") + partfilename + _T(')'));
			}
			if ( !fOutputFile.Open( strTempNew, CFile::modeWrite | CFile::shareExclusive | CFile::osSequentialScan
								   | CFile::modeCreate ) )
			{
			//	Let's catch to care about closing everything
				throw CString(_T("can't open file for writing (") + strTempNew + _T(')'));
			}

			int		iBlockSize = g_eMuleApp.m_pGlobPrefs->SlowCompleteBlockSize() * 1024;
			char	*pcBuffer = new char[iBlockSize];
			DWORD	dwRead;

		//	Only one extensive hard drive access at a time to reduce I/O load
			CSingleLock Lock(&g_eMuleApp.m_csPreventExtensiveHDAccess, TRUE);

		//	Most of the work done here
		//	just copy the file data from one to another
			do
			{
				dwRead = fInputFile.Read(pcBuffer, iBlockSize);
				fOutputFile.Write(pcBuffer, dwRead);
			//	Release the CPU for other processes
				Sleep(0);
			}
			while (dwRead > 0);

			Lock.Unlock();
			delete []pcBuffer;

			fInputFile.Close();
			fOutputFile.Close();

		//	Rename temporary file back to normal name
			if (_trename(strTempNew, newname))
			{
			//	Lets catch to care about closing everything
				throw CString(_T("renaming of completed file failed"));
			}

		//	Synchronization with reading upload thread
			CSingleLock lockComplete(&m_csFileCompletion, TRUE);

			m_strFullName = newname;
			SetPath(strOutputDir);
			SetFilePath(m_strFullName);
			m_bPaused = false;
			m_eStatus = PS_COMPLETE;
			m_iSrcA4AF = 0;

			lockComplete.Unlock();

		//	Update GUI after synchronization is released
			UpdateGUIAfterStateChange();

		//	Close permanent handles
			try
			{
				ClosePartFile();
			}
			catch (CFileException *error)
			{
				OUTPUT_DEBUG_TRACE();
				AddDebugLogLine(_T("Failed to close permanent handle %s - %s"), GetFileName(), GetErrorMessage(error));
				error->Delete();
			}
			::DeleteFile(partfilename);
		}
	//	Check for exception that will be thrown by "new"
		catch (CMemoryException * error)
		{
			error->Delete();
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_OUTMEM));
			bException = true;
		}
		catch (CString str)
		{
			AddLogLine(true, RGB_LOG_ERROR + str);
			bException = true;
		}
		catch (CException *error)
		{
		//	Sending message for erroneous file
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_COMPLETIONFAILED), GetFileName(), GetErrorMessage(error));
			g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError(), g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled());
			g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError());
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_COMPLETIONFAILED), GetFileName(), GetErrorMessage(error));
			error->Delete();
			bException = true;
		}
		if (bException)
		{
		//	File should be closed to make Initialization possible
			try
			{
				ClosePartFile();
			}
			catch (CFileException *error)
			{
				OUTPUT_DEBUG_TRACE();
				AddDebugLogLine(_T("Failed to close permanent handle %s - %s"), GetFileName(), GetErrorMessage(error));
				error->Delete();
			}

			if (fOutputFile.m_hFile != INVALID_HANDLE_VALUE)
				fOutputFile.Close();
		//	Delete temporary file if it still exists
			::DeleteFile(strTempNew);

			m_bPaused = true;
			SetStatus(PS_ERROR);
		//	If fInputFile is still opened, it will be closed here in its destructor
			return FALSE;
		}
	}
	else
	{
	//	Synchronization with reading upload thread
		CSingleLock lockComplete(&m_csFileCompletion, TRUE);

	//	Close permanent handles
		try
		{
			ClosePartFile();
		}
		catch (CFileException *error)
		{
			OUTPUT_DEBUG_TRACE();
			AddDebugLogLine(_T("Failed to close permanent handle %s - %s"), GetFileName(), GetErrorMessage(error));
			error->Delete();
		}

		if (_trename(partfilename, newname))
		{
			lockComplete.Unlock();

		//	Sending message for erroneous file
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_COMPLETIONFAILED), GetFileName(), _tcserror(errno));
			g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError(), g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled());
			g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError());
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_COMPLETIONFAILED), GetFileName(), _tcserror(errno));
			AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_COMPLETIONFAILED2));
			m_bPaused = true;
			SetStatus(PS_ERROR);
			return FALSE;
		}

		m_strFullName = newname;
		SetPath(strOutputDir);
		SetFilePath(m_strFullName);
		m_bPaused = false;
		m_eStatus = PS_COMPLETE;
		m_iSrcA4AF = 0;

		lockComplete.Unlock();

	//	Update GUI after synchronization is released
		UpdateGUIAfterStateChange();
	}

//	To have the accurate date stored in known.met we have to update the 'date' of a just completed file.
//	if we don't update the file date here (after commiting the file and before adding the record to known.met),
//	that file will be rehashed at next startup and there would also be a duplicate entry (hash+size) in known.met because of different file date!
	struct _stat st;
	if (_tstat(newname, &st) == 0)
	{
		m_timetLastWriteDate = st.st_mtime;
		AdjustNTFSDaylightFileTime(&m_timetLastWriteDate, newname);
	}

	if (_tremove(strTempFullName))
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strTempFullName);

	CString	strFileToRemove;
	CString	strFilePath = RemoveFileExtension(strTempFullName);

//	Remove .dir file
	strFileToRemove = strFilePath + _T(".dir");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .bak file
	strFileToRemove = strTempFullName + _T(".bak");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .settings file
	strFileToRemove = strFilePath + _T(".settings");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .txtscr file
	strFileToRemove = strFilePath + _T(".txtsrc");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove the stats file
	strFileToRemove = strFilePath + _T(".stats");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Change AutoPriority to NORMAL when UAP is disabled
	if (IsULAutoPrioritized() && !g_eMuleApp.m_pGlobPrefs->IsUAPEnabled())
	{
		SetAutoULPriority(false);
		SetULPriority(PR_NORMAL);
	}

	UpdateDisplayedInfo(true);

	CCat		*pCat = CCat::GetCatByID(GetCatID());
	CString		strCategory = (pCat != NULL) ? pCat->GetTitle() : GetResString(IDS_CAT_UNCATEGORIZED);

	AddLogLine(true, RGB_LOG_SUCCESS + GetResString(IDS_DOWNLOADDONE), GetFileName(), strCategory);

//	Sending message for completed file
	CString MessageText;
	MessageText.Format(GetResString(IDS_TBN_DOWNLOADDONE), GetFileName(), strCategory);
	g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetUseDownloadNotifier(), g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
	g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_DLOAD, false, g_eMuleApp.m_pGlobPrefs->GetUseDownloadNotifier());
	if (bRenamed)
		AddLogLine(true, IDS_DOWNLOADRENAMED, _tcsrchr(newname, '\\') + 1);
	g_eMuleApp.m_pKnownFilesList->SafeAddKnownFile(this);
	g_eMuleApp.m_pDownloadQueue->RemoveFile(this);
	UpdateDisplayedInfo();

#ifdef OLD_SOCKETS_ENABLED
	g_eMuleApp.m_pMMServer->AddFinishedFile(this);
#endif //OLD_SOCKETS_ENABLED

//	Starts next paused file
	if (g_eMuleApp.m_pGlobPrefs->DownloadPausedOnComplete())
	{
		g_eMuleApp.m_pDownloadQueue->StartNextFile(eCatID);
	}

	g_eMuleApp.m_pGlobPrefs->Add2DownCompletedFiles();			// Increments cumDownCompletedFiles in prefs struct
	g_eMuleApp.m_pGlobPrefs->Add2DownSessionCompletedFiles(); 	// Increments sesDownCompletedFiles in prefs struct
	g_eMuleApp.m_pGlobPrefs->SaveCompletedDownloadsStat();		// Saves cumDownCompletedFiles to INI

	if (GetPath() != g_eMuleApp.m_pGlobPrefs->GetIncomingDir())
	{
		if (g_eMuleApp.m_pGlobPrefs->SharedDirListCheckAndAdd(GetPath(), true))
		{
		//	New path was added to the list, scan for files
			g_eMuleApp.m_pSharedFilesList->Reload();
		}
	}

	if (g_eMuleApp.m_pGlobPrefs->IsAutoClearCompleted())
	{
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.SetRedraw(false);
		g_eMuleApp.m_pDownloadList->ClearCompleted(CAT_ALL);
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.SetRedraw(true);
	}

	g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();

//	Scan completed files for viruses
	if (g_eMuleApp.m_pGlobPrefs->IsAVEnabled() &&
		g_eMuleApp.m_pGlobPrefs->IsAVScanCompleted() && !g_eMuleApp.m_pGlobPrefs->GetAVPath().IsEmpty())
	{
		strTempFullName.Format(_T("%s \"%s\""), g_eMuleApp.m_pGlobPrefs->GetAVParams(), m_strFullName);
		ShellExecute(NULL, "open", g_eMuleApp.m_pGlobPrefs->GetAVPath(), strTempFullName, NULL, SW_SHOW);
	}

//	FakeCheck
	if (IsFakesDotRar())
	{
		CFakecheck* pFakeCheck = g_eMuleApp.m_pFakeCheck;
		CString strFakesDotDatPath = g_eMuleApp.m_pGlobPrefs->GetConfigDir() + _T("fakes.dat");
		CString strBackupFilePath = strFakesDotDatPath + _T(".bak");

	//	Backups fakes.dat
		::MoveFile(strFakesDotDatPath, strBackupFilePath);

		if (pFakeCheck->ExtractRARArchive(GetFilePath(), g_eMuleApp.m_pGlobPrefs->GetConfigDir()))
		{
			g_eMuleApp.AddLogLine(true, RGB_LOG_SUCCESS + GetResString(IDS_FAKE_SUCCESS_UPDATE), g_eMuleApp.m_pGlobPrefs->GetFakesDatVersion(), g_eMuleApp.m_pGlobPrefs->GetDLingFakeListVersion());
			g_eMuleApp.m_pGlobPrefs->SetFakesDatVersion(g_eMuleApp.m_pGlobPrefs->GetDLingFakeListVersion());
			g_eMuleApp.m_pGlobPrefs->SetDLingFakeListVersion(0);
			g_eMuleApp.m_pGlobPrefs->SetDLingFakeListLink(_T(""));
			pFakeCheck->LoadFromDatFile();
			::DeleteFile(strBackupFilePath);
		}
		else
		{
		//	The extraction was unsuccessful, so we restore fakes.dat
			::DeleteFile(strFakesDotDatPath);
			::MoveFile(strBackupFilePath, strFakesDotDatPath);
		}

		CKnownFile* pThisFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(GetFileHash());

		if (pThisFile != NULL)
		{
			pThisFile->SetAutoULPriority(false);
			pThisFile->SetULPriority(PR_RELEASE);
			g_eMuleApp.m_pSharedFilesList->Reload();
		}
	}

	return TRUE;

	EMULE_CATCH

	return FALSE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::RemoveAllSources(bool bTryToSwap)
{
	EMULE_TRY

//	TODO transfer sources to other downloading files if possible
	CUpDownClient* pRemoveSource;

	for (int sl = 0;sl < SOURCESSLOTS;sl++)
	{
		while (!m_srcLists[sl].IsEmpty())
		{
			pRemoveSource = m_srcLists[sl].GetHead();
			if (bTryToSwap)
			{
				if (!pRemoveSource->SwapToAnotherFile(NULL, true, true))
				{
				//	If the switching wasn't succesfull we remove the source from the downloadqueue
					g_eMuleApp.m_pDownloadQueue->RemoveSource(pRemoveSource);
				}
				else
				{
				//	If the switching was succesfull we need to remove the entry from graphical list
					g_eMuleApp.m_pDownloadList->RemoveSource(pRemoveSource, this);
				}
			}
			else
			{
			//	We weren't asked to swap so we remove it
				g_eMuleApp.m_pDownloadQueue->RemoveSource(pRemoveSource);
			}
		}
	}

//	Remove A4AF entries from graphical list & anotherRequestList
	if (!m_A4AFsrcList.IsEmpty())
	{
		for (POSITION pos = m_A4AFsrcList.GetHeadPosition(); pos != NULL;)
		{
			pRemoveSource = m_A4AFsrcList.GetNext(pos);
			g_eMuleApp.m_pDownloadList->RemoveSource(pRemoveSource, this);
			pRemoveSource->RemoveRequestForAnotherFile(this);
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::DeleteFile()
{
	EMULE_TRY

	ASSERT (!m_bPreviewing);

	m_bIsBeingDeleted = true;

	StopFile();

	ClosePartFile();

	if (_tremove(m_strFullName))
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), m_strFullName);

	CString partfilename = m_strFullName.Left(m_strFullName.GetLength() - 4);
	if (_tremove(partfilename))
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), partfilename);

	CString	strFileToRemove;
	CString	strFilePath = RemoveFileExtension(m_strFullName);

//	Remove .dir file
	strFileToRemove = strFilePath + _T(".dir");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .bak file
	strFileToRemove = m_strFullName + _T(".bak");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .settings file
	strFileToRemove = strFilePath + _T(".settings");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove .txtscr file
	strFileToRemove = strFilePath + _T(".txtsrc");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	Remove the stats file
	strFileToRemove = strFilePath + _T(".stats");
	if (_tremove(strFileToRemove) && errno != ENOENT)
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_DELETEFAILED), strFileToRemove);

//	File properties get deleted inside one of these calls now...
	g_eMuleApp.m_pSharedFilesList->RemoveFile(this);
	g_eMuleApp.m_pDownloadQueue->RemoveFile(this);
	g_eMuleApp.m_pDownloadList->RemoveFile(this);

	delete this;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::HashSinglePart(uint32 dwPartNum)
{
	EMULE_TRY

	UINT dwResErrStrID;
	uint32 dwPartCnt = static_cast<uint32>(GetPartCount());

	if ((GetHashCount() <= dwPartNum) && (dwPartCnt > 1))
	{
		dwResErrStrID = IDS_ERR_HASHERRORWARNING;
	}
	else if (!GetPartHash(dwPartNum) && (dwPartCnt != 1))
	{
		dwResErrStrID = IDS_ERR_INCOMPLETEHASH;
	}
	else
	{
		uchar	hashresult[16];
	//	Duplicate part file read handle to eliminate read collisions
		CFile	*pPartFile = m_hPartFileRead.Duplicate();

		pPartFile->Seek((LONGLONG)PARTSIZE * dwPartNum, CFile::begin);

		uint32 length = PARTSIZE;

		if ((ULONGLONG)PARTSIZE * (dwPartNum + 1) > pPartFile->GetLength())
		{
			length = (pPartFile->GetLength() - ((ULONGLONG)PARTSIZE * dwPartNum));
		}
		ASSERT(length <= PARTSIZE);
		CreateHashFromFile(pPartFile, length, hashresult);

	//	File handle is closed in destructor
		delete pPartFile;

		return (md4cmp(hashresult, (GetFileSize() >= PARTSIZE) ? GetPartHash(dwPartNum) : m_fileHash) == 0) ? true : false;
	}

	AddLogLine(true, RGB_LOG_WARNING + GetResString(dwResErrStrID), GetFileName());
	this->m_bHashSetNeeded = true;
	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::IsMovieChunk(byte mode, uint16 chunk)
{
	EMULE_TRY

	uint32 lastblock = (PARTSIZE + EMBLOCKSIZE - 1) / EMBLOCKSIZE, dwEnd;
	if (chunk == GetPartCount() - 2)
		dwEnd = (PARTSIZE * chunk) + (lastblock * EMBLOCKSIZE) - 1; // End address from 1 chunk before the last chunk

	switch (mode)
	{
		case 0:  //Mode 0: No Preview
		return false;
		break;
		case 1:  //Mode 1: 1F,1L(2L) AVI
		if (chunk == 0)
			return true; //1F
		if (chunk == GetPartCount() - 1)
			return true; //1L is a moviechunk ?
		if (chunk == (GetPartCount() - 2) && (((GetFileSize() - 1) - MINLASTMOVIESIZE) <= dwEnd))
			return true; //2L is a Part of 1/8L at the end of the file
		break;
		case 2:  //Mode 2: 1F,2F,1L(2L) Long AVI
		if (chunk == 0)
			return true; //1F
		if (chunk == 1)
			return true; //2F
		if (chunk == GetPartCount() - 1)
			return true; //1L is a moviechunk ?
		if (chunk == (GetPartCount() - 2) && (((GetFileSize() - 1) - MINLASTMOVIESIZE) <= dwEnd))
			return true; //2L is a Part of 1/8L at the end of the file
		break;
		case 3:  //Mode 3: 1F,2F,3F,1L(2L) Longest AVI
		if (chunk == 0)
			return true; //1F
		if (chunk == 1)
			return true; //2F
		if (chunk == 2)
			return true; //3F
		if (chunk == GetPartCount() - 1)
			return true; //1L is a moviechunk
		if (chunk == (GetPartCount() - 2) && (((GetFileSize() - 1) - MINLASTMOVIESIZE) <= dwEnd))
			return true; //2L is a Part of 1/8L at the end of the file
		break;
		case 4:  //Mode 4: 1F MPG
		if (chunk == 0)
			return true; //1F
		break;
		case 5:  //Mode 5: 1F,2F Long MPG
		if (chunk == 0)
			return true; //1F
		if (chunk == 1)
			return true; //2F
		break;
		case 6:  //Mode 6: 1F,2F,3F Longest MPG
		if (chunk == 0)
			return true; //1F
		if (chunk == 1)
			return true; //2F
		if (chunk == 2)
			return true; //3F
		break;
		case 7:  //Mode 7: 1F,2F,1L,2L AVI/MPG normal
		if (chunk == 0)
			return true; //1F
		if (chunk == 1)
			return true; //2F
		if (chunk == GetPartCount() - 1)
			return true; //1L is a moviechunk ?
		if (chunk == GetPartCount() - 2)
			return true; //2L is a moviechunk ?
		break;
		default:
		return false;
		break;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::IsBetterMovieChunk(uint16 oldchunk, uint16 newchunk)
{
	EMULE_TRY

//	The routine is based on that both old & new-chunk are moviechunks
	if (oldchunk == 0) //Chunk = 1F
	{
	//	1/8F & !1/8L & (newchunk=1L | 2L)
		if ( IsComplete(0, (MINFIRSTMOVIESIZE) - 1) 										// first 1/8 chunk present
		     && !IsComplete((GetFileSize() - 1) - (MINLASTMOVIESIZE) , GetFileSize() - 1) 	// last 1/8 chunk not present
		     && ( newchunk == GetPartCount() - 1  											// better chunk could be 1L
		          || (newchunk == GetPartCount() - 2) ) ) 									// better chunk could be 2L
			return true;
	}
	else if (oldchunk == GetPartCount() - 1 || oldchunk == GetPartCount() - 2) //Chunk = 1L | 2L
	{
	//	(1/8L & !1/8F) | (1/8L & 1/8F)
		if ( ( ( !IsComplete(0, (MINFIRSTMOVIESIZE) - 1) 											// first 1/8 chunk not present
		         && IsComplete((GetFileSize() - 1) - (MINLASTMOVIESIZE) , GetFileSize() - 1) ) 		// last 1/8 chunk present
		       || ( IsComplete(0, (MINFIRSTMOVIESIZE) - 1) 											// first 1/8 chunk present
		            && IsComplete((GetFileSize() - 1) - (MINLASTMOVIESIZE) , GetFileSize() - 1) ) 	// last 1/8 chunk present
		       && newchunk == 0 ) )  																// better chunk could be 1F
			return true;
	}
	else if (oldchunk == 1) //Chunk = 2F
	{
	//	1F | 1L
		if (newchunk == 0 || newchunk == GetPartCount() - 1)
			return true;
	}
	else if (oldchunk == 2) //Chunk = 3F
	{
	//	1F | 1L otherwise 2F
		if (newchunk == 0 || newchunk == GetPartCount() - 1)
			return true;
		else if (newchunk == 1)
			return true;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Allow to preview zip/rar files
bool CPartFile::IsArchive(bool bOnlyPreviewable)
{
	if (bOnlyPreviewable)
		return m_bIsPreviewableArchive;

	return (ED2KFT_ARCHIVE == GetFileType());
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::SetPriority(byte byteNewPriority, bool bSaveSettings /*= true*/)
{
	EMULE_TRY

	priority = byteNewPriority;
	g_eMuleApp.m_pDownloadQueue->SortByPriority();
	UpdateDisplayedInfo(true);
	if (bSaveSettings)
		SavePartFile();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::StopFile(bool bUpdateDisplay /* TRUE */)
{
	EMULE_TRY

	if (m_eStatus == PS_COMPLETE || m_eStatus == PS_COMPLETING)
	{
		return;
	}

//	Change status as soon as possible in order to prevent
//	unpredictable behaviour	during the source remove
	m_bPaused = true;
	m_bStopped = true;

//	Need to tell any connected clients to stop sending the file
//	and send downloading clients a cancel (NOT if completing)
	CUpDownClient* pSource;

	EnterCriticalSection(&m_csDownloadingSourceList);
	POSITION posHeadList = m_downloadingSourceList.GetHeadPosition();
	LeaveCriticalSection(&m_csDownloadingSourceList);

	for (POSITION pos = posHeadList; pos != NULL;)
	{
		EnterCriticalSection(&m_csDownloadingSourceList);
		pSource = m_downloadingSourceList.GetNext(pos);
		LeaveCriticalSection(&m_csDownloadingSourceList);
#ifdef OLD_SOCKETS_ENABLED
		if (pSource)
		{
			pSource->SendCancelTransfer();
			pSource->SetDownloadState(DS_ONQUEUE);
		}
#endif //OLD_SOCKETS_ENABLED
	}

//	Reset A4AF auto file
	if (g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile() == this)
	{
		g_eMuleApp.m_pDownloadQueue->SetA4AFAutoFile(NULL);
	}

	RemoveAllSources(true);

	m_dwDataRate = 0;
	m_iNumTransferringSrcs = 0;
	m_iSrcNNP = 0;
	m_iSrcOnQueue = 0;
	m_iSrcHighQR = 0;
	m_iSrcConnecting = 0;
	m_iSrcTooManyConns = 0;
	m_iSrcConnected = 0;
	m_iSrcTimeout = 0;
	m_iSrcConnViaServer = 0;
	m_iSrcUnknown = 0;
	m_iSrcRecHash = 0;
	m_iSrcLowToLow = 0;
	m_iSrcQueueFull = 0;
	m_iSrcNoUpload = 0;
	m_iSrcBanned = 0;
	m_iSrcError = 0;
	m_iSrcA4AF = 0;
	m_A4AFsrcList.RemoveAll();
	InvalidateRemoteQueueRankingStats();

//	Save info to disk
	if (!m_bIsBeingDeleted)
	{
		FlushBuffer();
		SavePartFile();
		SaveSettingsFile();
	}

//	Update GUI information (if the file was stopped for completing GUI will be updated in CompleteFile)
	if (bUpdateDisplay)
	{
		UpdateDisplayedInfo(true);
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::PauseFile()
{
	EMULE_TRY

	if (m_eStatus == PS_COMPLETE || m_eStatus == PS_COMPLETING)
		return ;

	CUpDownClient* pSource;
	POSITION pos;

//	Reset A4AF auto file
	if (g_eMuleApp.m_pDownloadQueue->GetA4AFAutoFile() == this)
	{
		g_eMuleApp.m_pDownloadQueue->SetA4AFAutoFile(NULL);
	}

//	Send CANCEL to downloading clients
	EnterCriticalSection(&m_csDownloadingSourceList);
	POSITION posHeadList = m_downloadingSourceList.GetHeadPosition();
	LeaveCriticalSection(&m_csDownloadingSourceList);

	for (pos = posHeadList; pos != NULL;)
	{
		EnterCriticalSection(&m_csDownloadingSourceList);
		pSource = m_downloadingSourceList.GetNext(pos);
		LeaveCriticalSection(&m_csDownloadingSourceList);

		if (pSource)
		{
			pSource->SendCancelTransfer();
			pSource->SetDownloadState(DS_ONQUEUE);
		}
	}
//	At this point nobody is downloading so we can try to swap sources
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
	{
		if (!m_srcLists[sl].IsEmpty())
		{
			for (pos = m_srcLists[sl].GetHeadPosition(); pos != NULL;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos);
			//	Swap source to any other file
				if (pSource->GetDownloadState() == DS_ONQUEUE)
				{
					pSource->SwapToAnotherFile(NULL);
				}
			}
		}
	}

	m_bPaused = true;
	m_bStopped = false;
	m_dwDataRate = 0;
	m_iNumTransferringSrcs = 0;
	UpdateDisplayedInfo(true);

	if (!m_bIsBeingDeleted)
	{
		SavePartFile();
		SaveSettingsFile();
	}

	g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::ResumeFile()
{
	EMULE_TRY

	if (m_eStatus == PS_COMPLETE || m_eStatus == PS_COMPLETING)
	{
		return;
	}

//	Check if chunks are availible & set status
	EnumPartFileStatuses	eNewStatus = PS_EMPTY;

	for (uint32 ui = 0; ui < GetPartCount(); ui++)
	{
		if (IsPartComplete(ui))
		{
			eNewStatus = PS_READY;
			break;
		}
	}
	SetStatus(eNewStatus);

	if (GetStartTimeReset())
	{
	//	Needed for files which are paused after adding or loading
		m_SessionStartTime = CTime::GetCurrentTime();
		SetStartTimeReset(false);
	}

	m_bPaused = false;
	m_bStopped = false;
	m_dwLastFileSourcesRequestTime = 0;
	SavePartFile();
	SaveSettingsFile();
	UpdateDisplayedInfo(true);

	g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Used to load initial sources from Server right after connect
void CPartFile::GetSourcesAfterServerConnect()
{
	EMULE_TRY

	if (m_dwLastFileSourcesRequestTime > 0)
	{
		m_dwLastFileSourcesRequestTime = 0;
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CPartFile::GetPartFileStatus()
{
	EMULE_TRY

	UINT		dwResStrId;

	if (GetTransferringSrcCount() > 0)
	{
		dwResStrId = IDS_DOWNLOADING;
	}
	else
	{
		dwResStrId = (!IsStalled()) ? IDS_WAITING : IDS_STALLED;
	}

	switch (GetStatus())
	{
		case PS_HASHING:
		case PS_WAITINGFORHASH:
			dwResStrId = IDS_HASHING;
			break;
		case PS_COMPLETING:
			dwResStrId = IDS_COMPLETING;
			break;
		case PS_COMPLETE:
			dwResStrId = IDS_COMPLETE;
			break;
		case PS_PAUSED:
			dwResStrId = IDS_PAUSED;
			break;
		case PS_ERROR:
			dwResStrId = IDS_ERRORLIKE;
			break;
		case PS_STOPPED:
			dwResStrId = IDS_STOPPED;
			break;
	}
	return GetResString(dwResStrId);

	EMULE_CATCH

	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CPartFile::GetPartFileStatusID()
{
	EMULE_TRY

	int	ipfStatus = 0;

	if (GetTransferringSrcCount() > 0)
	{
		ipfStatus = PS_DOWNLOADING;
	}
	else
	{
		if (!IsStalled())
			ipfStatus = PS_WAITINGFORSOURCE;
		else
			ipfStatus = PS_STALLED;
	}

	switch (GetStatus())
	{
		case PS_HASHING:
		case PS_WAITINGFORHASH:
			ipfStatus = PS_HASHING;
			break;
		case PS_COMPLETING:
		case PS_COMPLETE:
		case PS_PAUSED:
		case PS_ERROR:
		case PS_STOPPED:
			ipfStatus = GetStatus();
			break;
	}
	return ipfStatus;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CPartFile::GetPartFileStatusRang()
{
	EMULE_TRY

//	Sort order is Complete - Completing - Downloading - Waiting - Stalled - Paused - Errors
	int status = 2;
	if (GetTransferringSrcCount() == 0)
	{
		if (!IsStalled())
			status = 3;
		else
			status = 4;
	}
	switch (GetStatus())
	{
		case PS_HASHING:
		case PS_WAITINGFORHASH:
			status = 6;
			break;
		case PS_COMPLETING:
			status = 1;
			break;
		case PS_COMPLETE:
			status = 0;
			break;
		case PS_PAUSED:
			status = 5;
			break;
		case PS_STOPPED:
			status = 7;
			break;
		case PS_ERROR:
			status = 8;
			break;
	}
	return status;

	EMULE_CATCH

	return 1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
sint32 CPartFile::GetTimeRemaining(bool bAverage /*=false*/)
{
	EMULE_TRY

	uint32	iDataRate = 0;
	sint32	iTimeRemaining = -1;

	if (bAverage)
		iDataRate = GetAvgDataRate();
	else
		iDataRate = GetDataRate();
	iTimeRemaining = (iDataRate > 0 ? (GetFileSize() - GetCompletedSize()) / iDataRate : -1);
	return ((iTimeRemaining > 0 && iTimeRemaining <= 8640000 /*100 days*/) ? iTimeRemaining : -1);

	EMULE_CATCH

	return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetAvgDataRate() recalculates the average bytes/second for the session. It stores the
//	average in 'm_AvgDataRate' as well as returning it.
uint32 CPartFile::GetAvgDataRate(bool bUpdated /*=false*/)
{
	EMULE_TRY

	if (bUpdated)
	{
		CTimeSpan	timeSpan;

	//	Calc on last FlushBuffer() to keep sessionstats for completed files
		if (m_eStatus == PS_COMPLETING || m_eStatus == PS_COMPLETE)
		{
			timeSpan = GetFlushTimeSpan();
		}
		else
		{
		//	TODO: Modify to use time since first data
			timeSpan = GetSessionTimeSpan();
		}

	//	Delay 1st calculation a bit & dont use datarate for balancing (peak-stopper?)
		uint32	iSeconds = timeSpan.GetTotalSeconds();

		if (iSeconds > 10)
			m_AvgDataRate = ((GetSessionTransferred() /*+ GetDataRate()*/) / (iSeconds /*+ 1*/));
		else
			m_AvgDataRate = 0;
	}

	EMULE_CATCH
	return m_AvgDataRate;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::PreviewFile()
{
	EMULE_TRY

	if (IsArchive(true))
	{
		if ((!m_bRecoveringArchive) && (!m_bPreviewing))
			CArchiveRecovery::recover(this, true);
		return;
	}

	if (!PreviewAvailable())
		return;

	if (g_eMuleApp.m_pGlobPrefs->BackupPreview())
	{
		m_bPreviewing = true;
		CPreviewThread* pThread = (CPreviewThread*) AfxBeginThread(RUNTIME_CLASS(CPreviewThread), THREAD_PRIORITY_NORMAL + g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority(), 0, CREATE_SUSPENDED);
		pThread->SetValues(this, g_eMuleApp.m_pGlobPrefs->GetVideoPlayer());
		pThread->ResumeThread();
	}
	else
	{
		CString strLine = this->GetFullName();
		CString player = g_eMuleApp.m_pGlobPrefs->GetVideoPlayer();
		strLine.Replace(_T(".met"), _T(""));

	//	If path has spaces use quotes
		if (strLine.Find(_T(' ')) != -1)
		{
			strLine = _T('\"') + strLine + _T('\"');
		}

		if (player.IsEmpty())
		{
			ShellExecute(NULL, _T("open"), strLine, NULL, NULL, SW_SHOWNORMAL);
		}
		else
		{
			CString strPlayerPath = player;
			int i = strPlayerPath.ReverseFind('\\');
			strPlayerPath = (i >= 0) ? strPlayerPath.Left(i + 1) : _T("");
			ShellExecute(NULL, _T("open"), player, strLine, strPlayerPath, SW_SHOWNORMAL);
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::PreviewAvailable()
{
	EMULE_TRY

	EnumPartFileStatuses	eFileStatus = GetStatus();

// 	No preview while completing or hashing or when file is already complete
	if (eFileStatus == PS_COMPLETING || eFileStatus == PS_COMPLETE || eFileStatus == PS_HASHING)
		return false;

//	Allow preview of archives of any length > 1k
	if (IsArchive(true))
	{
		if ( (GetFileSize() > 1024) && (GetCompletedSize() > 1024) && !m_bRecoveringArchive &&
			((GetFreeDiskSpaceX(m_strTempDir) + 100000000) > (2 * GetFileSize())) )
			return true;
		else
			return false;
	}

	byte	bytedwMovieMode = GetMovieMode();

	if (bytedwMovieMode == 0)
	{
	//	Check only for AVI style movie, as below first chunk check is enough for all MPG-style ones
		if (IsAviMovie())
			bytedwMovieMode = (GetLastPartSize() < (GetFileSize() / 100)) ? 7 : 1;
	}

	if (g_eMuleApp.m_pGlobPrefs->BackupPreview())
	{
		return ( ((eFileStatus == PS_READY) || (eFileStatus == PS_PAUSED) || (eFileStatus == PS_STOPPED))
			&& !m_bPreviewing && (GetPartCount() > 2) && IsMovie() && IsPartComplete(0) &&
			(((bytedwMovieMode != 1) && (bytedwMovieMode != 7)) || IsPartComplete(GetPartCount() - 1)) &&
			((bytedwMovieMode != 7) || IsPartComplete(GetPartCount() - 2)) &&
			((GetFreeDiskSpaceX(m_strTempDir) + 100000000) >= GetFileSize()) );
	}
	else
	{
		TCHAR szVideoPlayerFileName[_MAX_FNAME];
		_tsplitpath(g_eMuleApp.m_pGlobPrefs->GetVideoPlayer(), NULL, NULL, szVideoPlayerFileName, NULL);

	//	Enable the preview command if the according option is specified 'PreviewSmallBlocks'
	//	or if VideoLAN client is specified
		if (g_eMuleApp.m_pGlobPrefs->GetPreviewSmallBlocks() || !_tcsicmp(szVideoPlayerFileName, _T("vlc")))
		{
			if (m_bPreviewing)
				return false;

			if (!(eFileStatus == PS_READY || eFileStatus == PS_EMPTY || eFileStatus == PS_PAUSED || eFileStatus == PS_STOPPED))
				return false;

			if (!(GetFileType() == ED2KFT_VIDEO || GetFileType() == ED2KFT_AUDIO || GetFileType() == ED2KFT_CDIMAGE))
			{
				return false;
			}

		//	If it's an MPEG file, VLC is even capable of showing parts of the file if the beginning of the file is missing!
			if (IsMpgMovie() || IsMpgAudio())
			{
			//	TODO: search a block which is at least 16K (Audio) or 256K (Video)
				if (GetCompletedSize() < 16 * 1024)
					return false;
			}
			else
			{
			//	For AVI files it depends on the used codec..
				if (!IsComplete(0, 256 * 1024))
					return false;
			}

			return true;
		}
		else
		{
			return ( ((eFileStatus == PS_READY) || (eFileStatus == PS_PAUSED) || (eFileStatus == PS_STOPPED))
				&& !m_bPreviewing && (GetPartCount() > 2) && IsMovie() && IsPartComplete(0) &&
				(((bytedwMovieMode != 1) && (bytedwMovieMode != 7)) || IsPartComplete(GetPartCount() - 1)) &&
				((bytedwMovieMode != 7) || IsPartComplete(GetPartCount() - 2)) );
		}
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::AllowGet1stLast()
{
	if (!IsMovie() || (GetPartCount() < 3))
		return false;

	byte	bytedwMovieMode = GetMovieMode();

	if (bytedwMovieMode == 0)
	{
	//	Check only for AVI style movie, as below first chunk check is enough for all MPG-style ones
		if (IsAviMovie())
			bytedwMovieMode = (GetLastPartSize() < (GetFileSize() / 100)) ? 7 : 1;
	}
	return !( IsPartComplete(0) &&
		(((bytedwMovieMode != 1) && (bytedwMovieMode != 7)) || IsPartComplete(GetPartCount() - 1)) &&
		((bytedwMovieMode != 7) || IsPartComplete(GetPartCount() - 2)) );
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Packet*	CPartFile::CreateSrcInfoPacket(CUpDownClient *pForClient)
{
	EMULE_TRY

	if (pForClient->m_pReqPartFile != this)
		return NULL;

	if (GetSourceCount() == 0)
		return NULL;

	CSafeMemFile	packetStream(1024);
	uint16			uCount = 0;

	packetStream.Write(m_fileHash, 16);
	packetStream.Write(&uCount, 2);

	CUpDownClient  *pPotentialSource;
	bool 			bNeededPart;
	byte		   *pbyteForClientPartStatus;
	byte		   *pbytePotentialClientPartStatus;

	uint32			dwID;
	uint16			uPort;
	uint32			dwServerIP;
	uint16			uServerPort;

	POSITION		pos;
	const uint16	uPartCount = GetPartCount();

	for (uint32 sl = 0; sl < SOURCESSLOTS; sl++)
	{
		if (!m_srcLists[sl].IsEmpty())
		{
			for (pos = m_srcLists[sl].GetHeadPosition(); pos != NULL; )
			{
				pPotentialSource = m_srcLists[sl].GetNext(pos);

			//	Don't send a client to himself as a source
			//	Only send sources with LowID that are in same sever that client who is requesting sources
			//	Don't send clients that have no parts available
			//	Don't share Lan clients sources as they are private ips/userids
				if ( md4cmp(pPotentialSource->m_reqFileHash, pForClient->m_reqFileHash) != 0
						|| pForClient == pPotentialSource || pForClient->Compare(pPotentialSource)
						|| ( pPotentialSource->HasLowID()
						     && ( pPotentialSource->GetServerIP() != pForClient->GetServerIP()
						          || pPotentialSource->GetServerPort() != pForClient->GetServerPort() ) )
						|| pPotentialSource->GetAvailUpPartCount() == 0
						|| pPotentialSource->IsOnLAN()
						|| uPartCount != pPotentialSource->GetUpPartCount() )
					continue;

			//	Don't send incomplete established connections. to avoid sending no valid sources
				switch (pPotentialSource->GetDownloadState())
				{
					case DS_NONE:
					case DS_CONNECTING:
					case DS_ERROR:
					case DS_TOOMANYCONNS:
					continue;
					break;
				}

				bNeededPart = false;

			//	Only send source which have needed parts for this client if possible
				pbyteForClientPartStatus = pForClient->GetPartStatus();
				pbytePotentialClientPartStatus = pPotentialSource->GetPartStatus();

			//	Both clients support part statuses so we try to find needed clients
				if (pbyteForClientPartStatus != NULL && pbytePotentialClientPartStatus != NULL)
				{
					for (uint32 i = 0; i < uPartCount; i++)
					{
						if(pbytePotentialClientPartStatus[i] && !pbyteForClientPartStatus[i])
						{
							bNeededPart = true;
							break;
						}
					}
				}
			//	If not we give a sources with at least one part (for files that have more than one part)
				else
					bNeededPart = true;

				if (bNeededPart)
				{
					uCount++;
					dwID = pPotentialSource->GetUserID();
					uPort = pPotentialSource->GetUserPort();
					dwServerIP = pPotentialSource->GetServerIP();
					uServerPort = pPotentialSource->GetServerPort();

					packetStream.Write(&dwID, 4);
					packetStream.Write(&uPort, 2);
					packetStream.Write(&dwServerIP, 4);
					packetStream.Write(&uServerPort, 2);
					if (pForClient->GetSourceExchangeVersion() > 1)
			    		packetStream.Write(pPotentialSource->GetUserHash(),16);
					if (uCount > 500)
						break;
				}
			}
		}
	}

	if (uCount == 0)
		return NULL;

	packetStream.Seek(16, 0);
	packetStream.Write(&uCount, 2);

	Packet	*pPacket = new Packet(&packetStream, OP_EMULEPROT);

	pPacket->m_eOpcode = OP_ANSWERSOURCES;
	if (pPacket->m_dwSize > 354)
		pPacket->PackPacket();

	return pPacket;

	EMULE_CATCH

	return NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::AddClientSources(CMemFile *packetStream, byte byteSourceExchangeVersion)
{
	EMULE_TRY

//	If this part file is paused or complete/completing or erroneous no sources needed any more.
	EnumPartFileStatuses	eFileStatus = GetStatus();

	if ( (eFileStatus == PS_PAUSED) || (eFileStatus == PS_STOPPED)
		|| (eFileStatus == PS_COMPLETE) || (eFileStatus == PS_COMPLETING)
		|| (eFileStatus == PS_ERROR) )
		return;

	uint16		uNumSources;
	packetStream->Read(&uNumSources, 2);

//	Check if the data size matches the 'uNumSources' for v1 or v2 and eventually correct the source
//	exchange version while reading the packet data. Otherwise we could experience a higher
//	chance in dealing with wrong source data, userhashs and finally duplicate sources.
	UINT		uDataSize = packetStream->GetLength() - packetStream->GetPosition();

//	Checks if v1 packet has the correct size
	if (uNumSources * (4u + 2u + 4u + 2u) == uDataSize)
	{
		byteSourceExchangeVersion = 1;
	}
//	Checks if v2 packet has the correct size
	else if (uNumSources * (4u + 2u + 4u + 2u + 16u) == uDataSize)
	{
		byteSourceExchangeVersion = 2;
	}
	else
	{
	//	If v3 inserts additional data (like v2), the above code will correctly filter those packets.
	//	If v3 appends additional data after <count>(<Sources>)[count], we are in trouble with the
	//	above code. Though a client which does not understand v3+ should never receive such a packet.
		g_eMuleApp.m_pdlgEmule->AddDebugLogLine(_T("Received invalid source exchange packet (v%u) of data size %u for %s"), byteSourceExchangeVersion, uDataSize, GetFileName());
		return;
	}

	for (int i = 0; i != uNumSources; i++)
	{
		CClientSource		source;

		packetStream->Read(&source.sourceID, 4);
		packetStream->Read(&source.sourcePort, 2);
		packetStream->Read(&source.serverIP, 4);
		packetStream->Read(&source.serverPort, 2);
		if (byteSourceExchangeVersion > 1)
			packetStream->Read(&source.achUserHash, 16);
		AddClientSource(&source, i, true, byteSourceExchangeVersion);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::GetSizeToTransfer(uint32& pui32SizeToTransfer)
{
	pui32SizeToTransfer = m_dwGapsSum;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Updates priority of file if autopriority is activated
void CPartFile::UpdateDownloadAutoPriority(void)
{
	EMULE_TRY

	if (IsAutoPrioritized())
	{
		int iValidSources = GetSourceCount() - GetNotCurrentSourcesCount();

		if (iValidSources < g_eMuleApp.m_pGlobPrefs->PriorityHigh() && priority != PR_HIGH)
			SetPriority(PR_HIGH);
		else if (iValidSources >= g_eMuleApp.m_pGlobPrefs->PriorityHigh() && iValidSources < g_eMuleApp.m_pGlobPrefs->PriorityLow() && priority != PR_NORMAL)
			SetPriority(PR_NORMAL);
		else if (iValidSources >= g_eMuleApp.m_pGlobPrefs->PriorityLow() && priority != PR_LOW)
			SetPriority(PR_LOW);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Making this function return a higher when more sources have the extended
//	protocol will force you to ask a larger variety of people for sources
uint32 CPartFile::GetCommonFilePenalty()
{
//	TODO: implement, but never return less than MINCOMMONPENALTY!
	return MINCOMMONPENALTY;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CPartFile::GetStatsFullPath() const
{
	return RemoveFileExtension(m_strFullName) + _T(".stats");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//--- xrmb:keepPartFileStats ---
bool CPartFile::LoadPartFileStats()
{
	EMULE_TRY

	FILE * fh = _tfopen(GetStatsFullPath(), _T("rb"));

//	If we cant open this file you lose the stats
	if (fh == NULL)
		return false;

//	Get version
	byte	version = 0;
	if (fread(&version, sizeof(version), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}

	uint32 alltimerequested = 0;
	uint32 alltimeaccepted = 0;
	uint64 alltimetransferred = 0;

	if (fread(&alltimerequested, sizeof(alltimerequested), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}
	if (fread(&alltimeaccepted, sizeof(alltimeaccepted), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}
	if (fread(&alltimetransferred, sizeof(alltimetransferred), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}

	statistic.AddAll(alltimerequested, alltimeaccepted, alltimetransferred);

//	This is for the parttraffic
//	I thought about explicit adding the traffic like for the other stats above
//	but it doesnt make sense, because you'll no transfer if you reach this
	if (LoadFromFileTraffic(fh, version) == false)
	{
		fclose(fh);
		return false;
	}

	fclose(fh);

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPartFile::SavePartFileStats()
{
	EMULE_TRY

	FILE *fh = _tfopen(GetStatsFullPath(), _T("wb"));

//	If we cant open this file you lose the stats
	if (fh == NULL)
		return false;

//	Set version
	byte	version = 2;
	if (fwrite(&version, sizeof(version), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}

	uint32 alltimerequested = statistic.GetAllTimeRequests();
	uint32 alltimeaccepted = statistic.GetAllTimeAccepts();
	uint64 alltimetransferred = statistic.GetAllTimeTransferred();

	if (fwrite(&alltimerequested, sizeof(alltimerequested), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}
	if (fwrite(&alltimeaccepted, sizeof(alltimeaccepted), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}
	if (fwrite(&alltimetransferred, sizeof(alltimetransferred), 1, fh) != 1)
	{
		fclose(fh);
		return false;
	}

//	This is for the parttraffic
	if (SaveToFileTraffic(fh) == false)
	{
		fclose(fh);
		return false;
	}

	fclose(fh);

	return true;

	EMULE_CATCH

	return false;
}
//--- :xrmb ---
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Used as separate thread to complete file
UINT CPartFile::CompleteThreadProc(CPartFile* pFile)
{
	EMULE_TRY

	if (g_eMuleApp.m_pGlobPrefs->IsNTBased())
	{
		SetThreadLocale(g_eMuleApp.m_pGlobPrefs->GetLanguageID());
	}

	if (!pFile)
	{
		return -1;
	}

	pFile->PerformFileComplete();

	return 0;

	EMULE_CATCH

	return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CPartFile::DtLocale(CTime dt)
{
	static const UINT		s_adwDayResId[] = {
		IDS_SCH_EXCEPT_SUN, IDS_SCH_EXCEPT_MON, IDS_SCH_EXCEPT_TUE, IDS_SCH_EXCEPT_WED,
		IDS_SCH_EXCEPT_THU, IDS_SCH_EXCEPT_FRI, IDS_SCH_EXCEPT_SAT
	};
	SYSTEMTIME st;
	dt.GetAsSystemTime(st);

	int		nDateSize = GetDateFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, NULL, 0);
	int		nTimeSize = GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, NULL, 0);
	CString	strDate, strTime, strBuffer;

	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, strDate.GetBuffer(nDateSize), nDateSize);
	strDate.ReleaseBuffer();

	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &st, NULL, strTime.GetBuffer(nTimeSize), nTimeSize);
	strTime.ReleaseBuffer();

	strBuffer.Format(_T("%s, %s, %s"),
		GetResString(s_adwDayResId[dt.GetDayOfWeek() - 1]), strDate, strTime);
	return strBuffer;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadSettingsFile() loads settings for partfiles (Movie Preview Mode, Stopped status, Preallocation)
void CPartFile::LoadSettingsFile()
{
	EMULE_TRY

	CString strIniFile = RemoveFileExtension(m_strFullName);

	strIniFile += _T(".settings");
	if (PathFileExists(strIniFile))
	{
		CIni filesettings(strIniFile);

		filesettings.SetDefaultCategory(_T("Settings"));
		byte Mode = filesettings.GetInt(_T("MoviePreviewMode"), 0);

		if ((Mode > 7) || ((Mode & 6) == 2) || (Mode == 6))	//check for valid values 0, 1, 4, 5, 7
			Mode = 0;
		CKnownFile::SetMovieMode(Mode);
		m_bStopped = filesettings.GetBool(_T("StoppedStatus"), m_bStopped);
		m_bIsPreallocated = filesettings.GetBool(_T("Preallocated"), m_bIsPreallocated);

		filesettings.CloseWithoutSave();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SaveSettingsFile() save settings for partfiles (Movie Preview Mode, Stopped status, Preallocation)
void CPartFile::SaveSettingsFile()
{
	EMULE_TRY

	CString strIniFile = RemoveFileExtension(m_strFullName);

	strIniFile += _T(".settings");

	CIni filesettings(strIniFile);

	filesettings.SetDefaultCategory(_T("Settings"));
	filesettings.SetInt(_T("MoviePreviewMode"), ((CKnownFile*)this)->GetMovieMode());
	filesettings.SetBool(_T("StoppedStatus"), m_bStopped);
	filesettings.SetBool(_T("Preallocated"), m_bIsPreallocated);

	filesettings.SaveAndClose();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	WriteToBuffer() buffer rx data which will be flushed later by timer or size threshold.
//		Params:
//			iCompressionGain - number of bytes saved by compression (it can be negative 
//			                   after connection loss in the middle of compressed block);
//			data             - pointer to data buffer;
//			dwStart          - data start offset;
//			dwEnd            - data end offset (inclusive);
//			pBlock           - Requested Block corresponding to rx data;
//			iFlags           - additional flags.
//				PF_WR2BUF_FL_FREEBUFFER - data buffer can be used for further operations (caller doesn't free it)
//				PF_WR2BUF_FL_ENDOFBLOCK - current data is the last portion of the requested block
//		Return: number of buffered bytes
uint32 CPartFile::WriteToBuffer(sint32 iCompressionGain, BYTE *data, uint32 dwStart, uint32 dwEnd, Requested_Block_Struct *pBlock, int iFlags)
{
	EMULE_TRY

//	This is needed a few times
	uint32 lenData = dwEnd - dwStart + 1;

	if (iCompressionGain != 0)
	{
	//	Compression gain can be negative in case of disconnection in the middle of compressed stream
		m_iGainDueToCompression += static_cast<sint64>(iCompressionGain);
		m_iSessionGainDueToCompression += static_cast<sint64>(iCompressionGain);
	}

//	Occasionally packets are duplicated, no point writing it twice
	if (IsComplete(dwStart, dwEnd))
	{
		AddDebugLogLine(_T("File '%s' has already been written from %lu to %lu size %lu"), GetFileName(), dwStart, dwEnd, lenData);
	}
	else
	{
	//	Create a new buffered queue entry
		PartFileBufferedData *item = new PartFileBufferedData;

		item->m_dwStartOffset = dwStart;
		item->m_dwEndOffset = dwEnd;

		if ((iFlags & PF_WR2BUF_FL_FREEBUFFER) == 0)
		{
		//	Create copy of data as new buffer
			BYTE *buffer = new BYTE[lenData];

			memcpy2(buffer, data, lenData);
			data = buffer;
		}
		item->m_pbyteBuffer = data;

	//	Add to the queue in the correct position (most likely the end)
		PartFileBufferedData *queueItem;
		bool added = false;
		POSITION pos = m_BufferedData_list.GetTailPosition();
		while (pos != NULL)
		{
			POSITION posNext = pos;

			queueItem = m_BufferedData_list.GetPrev(pos);
			if (item->m_dwEndOffset > queueItem->m_dwEndOffset)
			{
				added = true;
				m_BufferedData_list.InsertAfter(posNext, item);
				break;
			}
		}
		if (!added)
			m_BufferedData_list.AddHead(item);

	//	Increment buffer size marker
		m_nTotalBufferData += lenData;

	//	Mark this small section of the file as filled
		FillGap(item->m_dwStartOffset, item->m_dwEndOffset);

	//	Update the flushed mark on the requested block
	//	The loop here is unfortunate but necessary to detect deleted blocks.
		pos = requestedblocks_list.GetHeadPosition();
		while (pos != NULL)
		{
			if (requestedblocks_list.GetNext(pos) == pBlock)
				pBlock->m_dwBytesTransferred += lenData;
		}

		uint32 dwPartNum = dwStart / PARTSIZE;

	// Check that all data of the part is received
		if (IsPartFull(dwPartNum))
		{
		//	Prevent data flush during processesing of the data from the sockets
			m_bDataFlushReq = true;
		}
		else if (((iFlags & PF_WR2BUF_FL_ENDOFBLOCK) != 0) && IsCorruptedPart(dwPartNum))
		{
		//	Flush and check data to minimize amount of additional unrequired redownloads
		//	Do it only by end of the block basis to minimize system load as some clients send by small portions
			m_bDataFlushReq = true;
		}

		m_timeLastDownTransfer = CTime::GetCurrentTime();

	//	Return the length of data written to the buffer
		return lenData;
	}

	EMULE_CATCH

	if ((iFlags & PF_WR2BUF_FL_FREEBUFFER) != 0)
		delete [] data;

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	FlushBuffer() flushes buffered data from the source to the part file
void CPartFile::FlushBuffer(void)
{
	EMULE_TRY

	m_bDataFlushReq = false;

	m_nLastBufferFlushTime = ::GetTickCount();

//	If there's no buffered data to flush, just return
	if (m_BufferedData_list.IsEmpty())
		return;

//	Reset the Last Down Transfer timer
	m_timeLastDownTransfer = CTime::GetCurrentTime();
	GetAvgDataRate(true);

//	Calculate the number of parts in the part file
	uint32	dwPartCount = GetPartCount();
	bool *changedPart = new bool[dwPartCount];

	try
	{
	//	Remember which parts need to be checked at the end of the flush
		for (uint32 dwPartNum = 0; dwPartNum < dwPartCount; dwPartNum++)
		{
			changedPart[dwPartNum] = false;
		}

	//	Ensure file is big enough to write data to (the last item will be the furthest from the start)
		PartFileBufferedData *item = m_BufferedData_list.GetTail();

	//	If the end of the buffered data is past the current end of the part file, extend it
		if (m_hPartFileWrite.GetLength() <= item->m_dwEndOffset)
			m_hPartFileWrite.SetLength(item->m_dwEndOffset + 1);

	//	For each block of buffered data...
		for (int i = m_BufferedData_list.GetCount(); i > 0; i--)
		{
		//	Remove item from queue
			item = m_BufferedData_list.RemoveHead();

		//	For each part spanned by the buffered data, set the part's changed flag
			for (uint32 curPart = item->m_dwStartOffset / PARTSIZE; curPart <= item->m_dwEndOffset / PARTSIZE; curPart++)
				changedPart[curPart] = true;

		//	Go to the correct position in file and write block of data
			m_hPartFileWrite.Seek(item->m_dwStartOffset, CFile::begin);

			uint32 lenData = item->m_dwEndOffset - item->m_dwStartOffset + 1;

		//	Decrease buffer size
			m_nTotalBufferData -= lenData;
			m_hPartFileWrite.Write(item->m_pbyteBuffer, lenData);

		//	Release memory used by this item
			delete [] item->m_pbyteBuffer;
			delete item;
		}

	//	If the size of the part file is greater than the size of the target file,
	//	set it to the size of the target file.
		if (m_hPartFileWrite.GetLength() > GetFileSize())
			m_hPartFileWrite.SetLength(GetFileSize());

	//	Flush to disk
		m_hPartFileWrite.Flush();

	//	Check each part of the file
		uint32	partRange = GetLastPartSize() - 1;

	//	For each part in the part file (in reverse)...
		for (int iPartNum = (int)dwPartCount - 1; iPartNum >= 0; iPartNum--)
		{
		//	If we didn't just write it, skip it
			if (changedPart[iPartNum] == false)
			{
			//	Any parts other than last must be full size
				partRange = PARTSIZE - 1;
				continue;
			}

		//	Is this 9MB part complete
			if (IsComplete(PARTSIZE * iPartNum, (PARTSIZE * (iPartNum + 1)) - 1))
			{
			//	Is part corrupt
				if (!HashSinglePart(iPartNum))
				{
					AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_PARTCORRUPT), iPartNum, GetFileName());
					AddGap(PARTSIZE * iPartNum, (PARTSIZE * iPartNum + partRange));
					m_csGapListAndPartStatus.Lock();
					m_PartsStatusVector[iPartNum] |= PART_CORRUPTED;
					m_csGapListAndPartStatus.Unlock();
				//	Increment corruption statistics
					m_qwLostDueToCorruption += (partRange + 1);
					m_qwSessionLostDueToCorruption += (partRange + 1);
				}
				else
				{
					m_csGapListAndPartStatus.Lock();
					m_PartsStatusVector[iPartNum] = (m_PartsStatusVector[iPartNum] & 0xFFFFFF) | PART_VERIFIED;
					m_csGapListAndPartStatus.Unlock();

				//	Successfully completed part, make it available for sharing
					if (m_eStatus == PS_EMPTY)
					{
						SetStatus(PS_READY);
						g_eMuleApp.m_pSharedFilesList->SafeAddKnownFile(this);
					}

					m_dwCompletedPartsSize += (partRange + 1);
				}
			}
			else if (IsCorruptedPart(iPartNum))
			{
			//	Try to recover with minimal loss
				if (HashSinglePart(iPartNum))
				{
					m_csGapListAndPartStatus.Lock();
					m_PartsStatusVector[iPartNum] = (m_PartsStatusVector[iPartNum] & 0xFFFFFF) | PART_VERIFIED;
					m_csGapListAndPartStatus.Unlock();

					m_iTotalPacketsSavedDueToICH++;
					FillGap(PARTSIZE * iPartNum, (PARTSIZE * iPartNum + partRange));
					RemoveBlockFromList(PARTSIZE * iPartNum, (PARTSIZE * iPartNum + partRange));
					AddLogLine(true, RGB_LOG_SUCCESS + GetResString(IDS_ICHWORKED), iPartNum, GetFileName());
				//	Successfully completed part, make it available for sharing
					if (m_eStatus == PS_EMPTY)
					{
						SetStatus(PS_READY);
						g_eMuleApp.m_pSharedFilesList->SafeAddKnownFile(this);
					}
					g_eMuleApp.m_pGlobPrefs->Add2SessionPartsSavedByICH();

					m_dwCompletedPartsSize += (partRange + 1);
				}
			}
		//	Any parts other than last must be full size
			partRange = PARTSIZE - 1;
		}

	//	Update met file
		SavePartFile();

	//	Is this file finished?
		m_csGapListAndPartStatus.Lock();

		bool	bIsFileFinished = gaplist.IsEmpty() && !m_bHashSetNeeded;	// can't complete without a hashset

		m_csGapListAndPartStatus.Unlock();

		if (bIsFileFinished)
			CompleteFile(false);
	}
	catch (CFileException * error)
	{
		OUTPUT_DEBUG_TRACE();

		if (g_eMuleApp.m_pGlobPrefs->IsErrorBeepEnabled())
			Beep(800, 200);

		if (error->m_cause == CFileException::diskFull)
		{
			CString strSpace = CastItoXBytes(GetFreeDiskSpaceX(m_strTempDir));
			CString strTemp;

			strTemp.Format(_T("%s (%s)"), strSpace, m_strTempDir);
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_OUTOFSPACE), GetFileName());
			AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_DWTOT_FS), strTemp);
		//	Sending message for insufficient diskspace
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_OUTOFSPACE), GetFileName());
			g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError(), g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled());
			g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnImportantError());
		}
		else
		{
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_WRITEERROR), GetFileName(), GetErrorMessage(error));
			SetStatus(PS_ERROR);
		}
		PauseFile();
		error->Delete();
	}
	catch (...)
	{
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_WRITEERROR), GetFileName(), GetResString(IDS_UNKNOWN));
		SetStatus(PS_ERROR);
		PauseFile();

	//	When ERRONEOUS close PartFile to allow Initialize
		ClosePartFile();
	}
	delete[] changedPart;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This will invert the gap list, up to caller to delete gaps when done
// 'Gaps' returned are really the filled areas, and guaranteed to be in order
void CPartFile::GetFilledList(CTypedPtrList<CPtrList, Gap_Struct*> *filled)
{
	EMULE_TRY

	Gap_Struct * gap = NULL;
	Gap_Struct *best = NULL;
	POSITION pos;
	uint32 start = 0;
	uint32 bestEnd = 0;

//	Will be unlocked on exit
	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

//	Loop until done
	bool finished = false;

	while (!finished)
	{
		finished = true;
	//	Find first gap after current start pos
		bestEnd = GetFileSize();
		pos = gaplist.GetHeadPosition();
		best = gaplist.GetHead();
		while (pos != NULL)
		{
			gap = gaplist.GetNext(pos);
			if ((gap->m_dwStartOffset > start) && (gap->m_dwEndOffset < bestEnd))
			{
				best = gap;
				bestEnd = best->m_dwEndOffset;
				finished = false;
			}
		}

		if (!finished)
		{
		//	Invert this gap
			gap = new Gap_Struct;
			gap->m_dwStartOffset = start;
			gap->m_dwEndOffset = best->m_dwStartOffset - 1;
			start = best->m_dwEndOffset + 1;
			filled->AddTail(gap);
		}
		else if (best->m_dwEndOffset < GetFileSize())
		{
			gap = new Gap_Struct;
			gap->m_dwStartOffset = best->m_dwEndOffset + 1;
			gap->m_dwEndOffset = GetFileSize();
			filled->AddTail(gap);
		}
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::UpdateFileRatingCommentAvail()
{
	EMULE_TRY

	if (!this)
		return ;

	bool prev = (hasComment || hasRating);

	hasComment = false;
	hasRating = false;

	POSITION pos;
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
		if (!m_srcLists[sl].IsEmpty())
			for (pos = m_srcLists[sl].GetHeadPosition();pos != NULL;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos);
				if (pSource->GetFileComment().GetLength() > 0)
					hasComment = true;
				if (pSource->GetFileRating() != PF_RATING_NONE)
					hasRating = true;
				if (hasComment && hasRating)
					break;
			}

	if ((!hasComment) || (!hasRating))
	{
		for (pos = m_pastCommentList.GetHeadPosition(); pos != NULL;)
		{
			CPastComment &pc = m_pastCommentList.GetNext(pos);
			if (pc.GetComment().GetLength() > 0)
				hasComment = true;
			if (pc.GetRating() > 0)
				hasRating = true;
			if (hasComment && hasRating)
				break;
		}
	}

	if ((!hasComment) || (!hasRating))
	{
		CString			strTmp = GetFileComment();
		EnumPartFileRating	eRated = GetFileRating();

		if (!strTmp.IsEmpty())
			hasComment = true;
		if (eRated != PF_RATING_NONE)
			hasRating = true;
	}

	if (prev != (hasComment || hasRating))
		UpdateDisplayedInfo();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CPartFile::GetSourceCount()
{
	EMULE_TRY

	uint32	dwCount = 0;

	for (int i = 0; i < SOURCESSLOTS; i++)
		dwCount += m_srcLists[i].GetCount();
	return dwCount;

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
EnumPartFileRating CPartFile::GetRating()
{
	EMULE_TRY

	if (!hasRating)
		return PF_RATING_NONE;

	int num, tot, fRate;

	num = tot = 0;

	POSITION pos;

//	For each source slot...
	for (int sl = 0; sl < SOURCESSLOTS; sl++)
	{
	//	If the slot isn't empty...
		if (!m_srcLists[sl].IsEmpty())
		{
			for (pos = m_srcLists[sl].GetHeadPosition(); pos != NULL;)
			{
				fRate = m_srcLists[sl].GetNext(pos)->GetFileRating();
				if (fRate > 0)
				{
					num++;
				//	Bugfix: for some $%#ing reason fair=4 & good=3, breaking the progression from fake(1) to excellent(5)
					if (fRate == PF_RATING_GOOD || fRate == PF_RATING_FAIR)
						fRate = (fRate == PF_RATING_GOOD) ? 4 : 3;

					tot += fRate;
				}
			}
		}
	}

	for (pos = m_pastCommentList.GetHeadPosition(); pos != NULL;)
	{
		fRate = m_pastCommentList.GetNext(pos).GetRating();

		if (fRate > 0)
		{
			num++;
		//	Bugfix: for some $%#ing reason  fair=4 & good=3, breaking the progression from fake(1) to excellent(5)
			if (fRate == PF_RATING_GOOD || fRate == PF_RATING_FAIR)
				fRate = (fRate == PF_RATING_GOOD) ? 4 : 3;

			tot += fRate;
		}
	}
	
	fRate = GetFileRating();
	if (fRate > 0)
	{
			num++;
		//	Bugfix: for some $%#ing reason  fair=4 & good=3, breaking the progression from fake(1) to excellent(5)
			if (fRate == PF_RATING_GOOD || fRate == PF_RATING_FAIR)
				fRate = (fRate == PF_RATING_GOOD) ? 4 : 3;

			tot += fRate;
	}

	if (num > 0)
	{
		num = static_cast<double>(tot) / num + .5;	//Cax2 - get the average of all the ratings
	//	Bugfix: for some $%#ing reason good=3 & fair=4, breaking the progression from fake(1) to excellent(5)
		if (num == 3 || num == 4)
			num = (num == 3) ? PF_RATING_FAIR : PF_RATING_GOOD;
	}
	return static_cast<_EnumPartFileRating>(num);	//Cax2 - if no ratings found, will return 0!

	EMULE_CATCH

	return PF_RATING_NONE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	UpdateDisplayedInfo() causes the DownloadList ctrl to redisplay the item for this file if either
//	'force' is true or if the redisplay timer has expired.
void CPartFile::UpdateDisplayedInfo(bool bForce)
{
	EMULE_TRY

	g_eMuleApp.m_pDownloadList->UpdateFile(this);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Hostname sources in ED2K superLink supporting Source Exchange v2
void CPartFile::AddClientSource(CClientSource* pSource, int iSource, bool bExchanged, byte byteSourceExchangeVersion)
{
	EMULE_TRY

//	Check first if we are this source
#ifdef OLD_SOCKETS_ENABLED
	if (g_eMuleApp.m_pServerConnect->GetClientID() < 0x1000000 && g_eMuleApp.m_pServerConnect->IsConnected())
	{
		if ( g_eMuleApp.m_pServerConnect->GetClientID() == pSource->sourceID &&
			 g_eMuleApp.m_pGlobPrefs->GetPort() == pSource->sourcePort )
		{
			return;
		}

	//	Although we are currently having a LowID, we could have had a HighID before, which
	//	that client is sending us now! seems unlikely ... it happend!
		if ( g_eMuleApp.m_pServerConnect->GetLocalIP() == pSource->sourceID &&
			 g_eMuleApp.m_pGlobPrefs->GetPort() == pSource->sourcePort )
		{
			return;
		}
	}
	else if ( g_eMuleApp.m_pServerConnect->GetClientID() == pSource->sourceID &&
			  g_eMuleApp.m_pGlobPrefs->GetPort() == pSource->sourcePort )
	{
		return;
	}
//	Use lowID source only if it's on the same server as us
	else if ( pSource->sourceID < 0x1000000 &&
			  !g_eMuleApp.m_pServerConnect->IsLocalServer(pSource->serverIP, pSource->serverPort) )
	{
		return;
	}
#endif //OLD_SOCKETS_ENABLED

	if ( g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > GetSourceCount()
	     && iSource < static_cast<int>(g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFileSoft()) )
	{
		CUpDownClient* pNewSource = g_eMuleApp.m_pDownloadQueue->CheckAndAddSource(this,pSource->sourceID,pSource->sourcePort, pSource->serverIP, pSource->serverPort);
		if (bExchanged && pNewSource)
		{
		//	Name the exchanged source
			pNewSource->SetUserName(GetResString(IDS_EXCHANGEDSOURCE));
		//	Check exchange version
			if (byteSourceExchangeVersion > 1)
				pNewSource->SetUserHash(pSource->achUserHash);
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::AddClientSources(CTypedPtrList<CPtrList, CClientSource*>* sources)
{
	EMULE_TRY

	POSITION pos;
	pos = sources->GetHeadPosition();
	while (pos != NULL)
	{
		CClientSource * source = (CClientSource*)sources->GetAt(pos);

		if (source->sourceType != ED2KLINK_SOURCE_IP)
		{
			sources->GetNext(pos);
			continue;
		}

		POSITION removePos = pos;
		sources->GetNext(pos);
		sources->RemoveAt(removePos);

		AddClientSource(source);
		delete source;
	}
	g_eMuleApp.m_pDownloadQueue->AddClientHostnameToResolve(sources);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create source ed2k from downloads
CString CPartFile::CreateED2KSourceLink(uint32 expirein, int sourcenum, bool bIsSourceSaver /*=false*/)
{
	EMULE_TRY

	CTypedPtrList<CPtrList, CUpDownClient*> srcstoadd;
	POSITION pos1, pos2;
	for (int sl = 0;sl < SOURCESSLOTS;sl++)
		if (!m_srcLists[sl].IsEmpty())
			for (pos1 = m_srcLists[sl].GetHeadPosition();pos1 != NULL;)
			{
				CUpDownClient* pSource = m_srcLists[sl].GetNext(pos1);
				if (pSource->HasLowID())
					continue;
				if (srcstoadd.IsEmpty())
				{
					srcstoadd.AddHead(pSource);
					continue;
				}
				if ((sourcenum == 0) || (srcstoadd.GetCount() < sourcenum) || (pSource->GetAvailablePartCount() > srcstoadd.GetTail()->GetAvailablePartCount()))
				{
					if ((sourcenum != 0) && (srcstoadd.GetCount() == sourcenum))
						srcstoadd.RemoveTail();
					ASSERT(srcstoadd.GetCount() < sourcenum);
					bool bInserted = false;
					for (pos2 = srcstoadd.GetTailPosition();pos2 != 0;)
					{
						CUpDownClient * cur_srctoadd = srcstoadd.GetPrev(pos2);
						if (cur_srctoadd->GetAvailablePartCount() > pSource->GetAvailablePartCount())
						{
							srcstoadd.InsertAfter(pos2, pSource);
							bInserted = true;
							break;
						}
					}
					if (!bInserted)
						srcstoadd.AddHead(pSource);
				}
			}

	uint32 dwID = 0;
#ifdef OLD_SOCKETS_ENABLED
	dwID = g_eMuleApp.m_pServerConnect->GetClientID();
#endif //OLD_SOCKETS_ENABLED

	CString strLink;
	strLink.Format( _T("ed2k://|file|%s|%u|%s|/"),
			g_eMuleApp.StripInvalidFilenameChars(GetFileName(), false),  // Spaces to dots
			GetFileSize(),
			HashToString(GetFileHash()) );

	if (srcstoadd.IsEmpty())
	{
		if (bIsSourceSaver)
			return strLink;
#ifdef OLD_SOCKETS_ENABLED
	//	!g_eMuleApp.m_pServerConnect->IsConnected() is not really needed since we give an ip but ...
		else if (!g_eMuleApp.m_pServerConnect->IsConnected() || g_eMuleApp.m_pServerConnect->IsLowID())
		{
			AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_SOURCELINKFAILED));
			return CString(_T(""));
		}
#endif //OLD_SOCKETS_ENABLED
	}

//	Is the expiration date is handle in other eMule mods ? a bIsSourceSaver is perhaps needed ...
	strLink += _T("|sources");
	if (expirein != 0)
	{
		COleDateTime expirationDate = COleDateTime::GetCurrentTime();
		COleDateTimeSpan timediff;
		timediff.SetDateTimeSpan(expirein, 0, 0, 0);
		expirationDate += timediff;
		strLink.AppendFormat(_T("@%02u%02u%02u"), (expirationDate.GetYear() % 100), expirationDate.GetMonth(), expirationDate.GetDay());
	}

	if (!bIsSourceSaver && dwID >= 0x1000000)
		strLink.AppendFormat(_T(",%u.%u.%u.%u:%u"), (byte)dwID, (byte)(dwID >> 8), (byte)(dwID >> 16), (byte)(dwID >> 24), g_eMuleApp.m_pGlobPrefs->GetPort());

	if (!bIsSourceSaver)
		AddDebugLogLine(_T("Generated ed2k link with %i sources for file '%s' (part count %u)"), (int)srcstoadd.GetCount(), GetFileName(), GetPartCount());

	while (!srcstoadd.IsEmpty())
	{
		CUpDownClient * pSource = srcstoadd.RemoveHead();
		if (!bIsSourceSaver)
			AddDebugLogLine(RGB_LOG_DIMMED_TXT _T("| User: %s - Available part count: %u"), pSource->GetUserName(), (int)pSource->GetAvailablePartCount());
		strLink.AppendFormat(_T(",%s:%u"), pSource->GetFullIP(), pSource->GetUserPort());
	}

	strLink.Append(_T("|/"));
	return strLink;

	EMULE_CATCH

	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::SetAlternativeOutputDir(CString *path)
{
	EMULE_TRY

	path->TrimRight(_T('\\'));
	m_strAlternativePath = *path;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetOutputDir() returns the appropriate destination directory path for this file.
CString CPartFile::GetOutputDir()
{
	EMULE_TRY

	CString	strRetPath = g_eMuleApp.m_pGlobPrefs->GetIncomingDir();
	CString	strCatPath;

	if (GetCatID() != 0)
		strCatPath = CCat::GetCatByID(GetCatID())->GetPath();

//	If no alternative path defined
	if (m_strAlternativePath.IsEmpty())
	{
	//	... and a category dir is defined...
		if (strCatPath != _T(""))
			return strCatPath; // Return the category dir
		else
			return strRetPath;	// Return the incoming dir
	}

//	MOREVIT - Why all the heavy duty shell call nonsense? You could replace all of this
//	with a single call to PathFileExists(). Somebody just thought they were more
//	clever than they were methinks.

//	We're going to use desktop folder "static" functions , we need an object
//	of IShellFolder type and DeskTop always exists
	IShellFolder	*pDesktop	= NULL;
	ITEMIDLIST	*pList	= NULL;
	CString	strDir;

//	Try to create DeskTop folder object
	if (SHGetDesktopFolder(&pDesktop) == NOERROR)
	{
		IShellFolder	* pFolder = NULL;
		ULONG	uAttrib	= SFGAO_FOLDER ; //we're going to look for the "folder" attribute

		strDir = m_strAlternativePath;
	//	"ParseDisplayName" uses UNICODE so BSTR will go
	//	We have to remember to release it
		BSTR	pData	= strDir.AllocSysString();

		if (pData)
		{
		//	'm_strAlternativePath' is always created without "\" but this was the way
		//	the original code was, maybe it was for a reason :) , so I am going to leave
		//	first check without adding "\" as well ...

		//	So here we convert "display name" (path string) into ID list used by Shell
			pDesktop->ParseDisplayName(NULL, NULL, pData, NULL, &pList, &uAttrib);
		//	Release the BSTR
			::SysFreeString(pData);
		//	If list we got is null then no such object exists
			if (!pList)
			{
			//	Add "\" to get folder name
				strDir += _T('\\');
			//	Allocate new search BSTR
				pData = strDir.AllocSysString();
				if (pData)
				{
				//	We're going to look for "folder" attribute
					uAttrib	= SFGAO_FOLDER ;
				//	We convert "display name" (path string) into ID list used by Shell
					pDesktop->ParseDisplayName(NULL, NULL, pData, NULL, &pList, &uAttrib);
				//	Release the BSTR
					::SysFreeString(pData);
				}
			}
		//	If we found the object and it's "folder" type
			if (pList && (uAttrib & SFGAO_FOLDER))
			{
				LPMALLOC	ppMalloc;

				if (SHGetMalloc(&ppMalloc) == NOERROR)
				{
					ppMalloc->Free(pList);
					ppMalloc->Release();
				}

			//	Share the destination directory
				if (g_eMuleApp.m_pGlobPrefs->SharedDirListCheckAndAdd(strDir, true))
				{
				//	New path was added to the list, scan for files
					g_eMuleApp.m_pSharedFilesList->Reload();
				}
				return m_strAlternativePath;
			}
		//	Just in case the object is not folder we have to release memory
			else if (pList)
			{
				LPMALLOC	ppMalloc;

				if (SHGetMalloc(&ppMalloc) == NOERROR)
				{
					ppMalloc->Free(pList);
					ppMalloc->Release();
				}
			}
		}
		pDesktop->Release();
	}

//	Default return
	strRetPath.TrimRight(_T('\\'));
	return strRetPath;

	EMULE_CATCH

	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CPartFile::GetDownloadFileInfo()
{
	CString strInfo, strLastSeenComplete, strLastProgress, strTimeLeft, strFileSz = CastItoXBytes(GetFileSize());
	double dblAvailability = 0.0;

	if (lastseencomplete == NULL)
		GetResString(&strLastSeenComplete, IDS_NEVER);
	else
		strLastSeenComplete = LocalizeLastSeenComplete();

	if (GetTransferred() == 0)
		GetResString(&strLastProgress, IDS_NEVER);
	else
		strLastProgress = LocalizeLastDownTransfer();

	if (GetPartCount() != 0)
		dblAvailability = GetAvailablePartCount() * 100 / GetPartCount();

	EnumPartFileStatuses	eFileStatus = GetStatus();

	if ((eFileStatus != PS_COMPLETING) && (eFileStatus != PS_COMPLETE))
		strTimeLeft = CastSecondsToHM(GetTimeRemaining());
	else
		strTimeLeft = _T("-");

	strInfo.Format( _T("%s (%s)\n\n%s: %s\n\n%s: %s\n") + GetResString(IDS_PARTINFOS),
							 GetFileName(), strFileSz,
	             GetResString(IDS_STATUS), GetPartFileStatus(),
	             GetResString(IDS_FILEHASH), HashToString(GetFileHash()),
							 GetResString(IDS_DL_FILENAME), GetFullName(),
	             GetPartCount(), GetResString(IDS_AVAIL), GetAvailablePartCount(), dblAvailability,
	             static_cast<int>(GetPercentCompleted()),
							 CastItoXBytes(GetCompletedSize()), strFileSz, GetTransferringSrcCount(),
	             GetResString(IDS_LASTSEENCOMPL), strLastSeenComplete,
	             GetResString(IDS_FD_LASTCHANGE), strLastProgress,
				 GetResString(IDS_DLCOL_REMAININGTIME), strTimeLeft,
	             GetOnQueueSrcCount(), GetQueueFullSrcCount(), GetNoNeededPartsSrcCount(),
	             GetSrcA4AFCount(), GetConnectingSrcCount() );

	return strInfo;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Improved Tooltips
CString CPartFile::GetDownloadFileInfo4Tooltips()
{
	CString strInfo, strLastSeenComplete, strLastProgress, strTransferSrcCount, strFileSz = CastItoXBytes(GetFileSize());
	double dblAvailability = 0.0;

	if (lastseencomplete == NULL)
		GetResString(&strLastSeenComplete, IDS_NEVER);
	else
		strLastSeenComplete = LocalizeLastSeenComplete();

	if (GetTransferred() == 0)
		GetResString(&strLastProgress, IDS_NEVER);
	else
		strLastProgress = LocalizeLastDownTransfer();

	if (GetPartCount() != 0)
		dblAvailability = GetAvailablePartCount() * 100 / GetPartCount();

	if (GetTransferringSrcCount() > 0)
		strTransferSrcCount.Format(GetResString(IDS_TT_PARTINFOS3), GetTransferringSrcCount());

	strInfo.Format( _T("<t=2><b>%s</b><br><t=2>%s (%u %s)<br><t=2>%d%s (%s / %s) %s<br><hr=100%%><br><b>%s:<t></b>%s<br><b>%s:<t></b>%s<br><b>%s:<t></b>%s<br><b>%s:<t></b>%d, %s: %d (%.1f%%)<br><b>%s:<t></b>%s<br><b>%s:<t></b>%s<br><b>%s:<t></b>")
	             + GetResString(IDS_TT_STATUS2) + _T("<br><b>%s:<t></b>%s"),
	             GetFileName(), strFileSz, GetFileSize(), GetResString(IDS_BYTES),
	             static_cast<int>(GetPercentCompleted()), GetResString(IDS_PROCDONE), CastItoXBytes(GetCompletedSize()), strFileSz, strTransferSrcCount,
	             GetResString(IDS_STATUS), GetPartFileStatus(),
	             GetResString(IDS_FILEHASH), HashToString(GetFileHash()),
				 GetResString(IDS_DL_FILENAME), GetFullName(),
	             GetResString(IDS_UP_PARTS), GetPartCount(), GetResString(IDS_AVAIL), GetAvailablePartCount(), dblAvailability,
	             GetResString(IDS_LASTSEENCOMPLETE), strLastSeenComplete,
				 GetResString(IDS_LASTCHANGE), strLastProgress,
	             GetResString(IDS_DL_SOURCES), GetOnQueueSrcCount(),
	             GetQueueFullSrcCount(),
	             GetNoNeededPartsSrcCount(),
	             GetSrcA4AFCount(),
	             GetConnectingSrcCount(),
                 GetResString(IDS_SIZE_ON_DISK), CastItoXBytes(GetRealFileSize()) );

	return strInfo;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Ornis: Creating progressive presentation of the partfilestatuses - for webdisplay
CString CPartFile::GetProgressString(uint16 size)
{
	EMULE_TRY

	const char crProgress = '0'; 	//green
	const char crHave = '1';		// black
	const char crPending = '2';	// yellow
	const char crMissing = '3';  	// red
	static const char crWaiting[6] = { '4', '5', '6', '7', '8', '9'};	// differents blue

	CString my_ChunkBar;

	for (uint16 i = 0; i <= (size + 1); i++)
		my_ChunkBar.AppendChar(crHave);	// one more for safety

	uint32 m_nFileSize = GetFileSize();
	double unit = static_cast<double>(size) / static_cast<double>(m_nFileSize);

	POSITION				pos;
	EnumPartFileStatuses	eFileStatus = GetStatus();

	if ((eFileStatus == PS_COMPLETE) || (eFileStatus == PS_COMPLETING))
	{
		CharFillRange(&my_ChunkBar, 0, static_cast<double>(m_nFileSize) * unit, crProgress);
	}
	else
	{
	//	Red gaps
	//	Will be unlocked on block exit
		CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);

		for (pos = gaplist.GetHeadPosition(); pos != NULL;)
		{
			Gap_Struct	*pGap = gaplist.GetNext(pos);
			uint32		dwGapBeg = pGap->m_dwStartOffset;
			uint32		dwGapEnd = pGap->m_dwEndOffset + 1;

			for (uint32 i = (dwGapBeg / PARTSIZE); i < GetPartCount(); i++)
			{
				const uint32 dwPartStart = i * PARTSIZE;
				const uint32 dwPartEnd = ((GetFileSize() - dwPartStart) > PARTSIZE) ? (dwPartStart + PARTSIZE) : GetFileSize();

				if (dwGapEnd > dwPartEnd)
					dwGapEnd = dwPartEnd; //the rest is in the next part

				byte	color;
				uint32	dwFreq;

				if (m_srcPartFrequencies.GetCount() >= (INT_PTR)i && ((dwFreq = m_srcPartFrequencies[i]) != 0))
					color = (dwFreq < 10) ? crWaiting[dwFreq / 2] : crWaiting[5];
				else
					color = crMissing;

				CharFillRange(&my_ChunkBar, static_cast<double>(dwGapBeg) * unit, static_cast<double>(dwGapEnd) * unit, color);

				dwGapBeg = dwGapEnd;
				dwGapEnd = pGap->m_dwEndOffset + 1;
				if (dwGapBeg == dwGapEnd) // finished?
					break;
			}
		}
	//	m_csGapListAndPartStatus is released here
	}

//	Yellow pending parts
	for (pos = requestedblocks_list.GetHeadPosition();pos != NULL;)
	{
		Requested_Block_Struct * block = requestedblocks_list.GetNext(pos);
		CharFillRange(&my_ChunkBar, static_cast<double>(block->m_dwStartOffset + block->m_dwBytesTransferred) * unit, static_cast<double>(block->m_dwEndOffset) * unit, crPending);
	}

	return my_ChunkBar;

	EMULE_CATCH

	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::CharFillRange(CString* buffer, double start, double dblEnd, char color)
{
	uint32	dwEnd = static_cast<uint32>(dblEnd);

	for (uint32 i = start; i <= dwEnd; i++)
		buffer->SetAt(i, color);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A4AF management
void CPartFile::DownloadAllA4AF(bool bSameCat /* FALSE */)
{
	EMULE_TRY

	for (POSITION pos = m_A4AFsrcList.GetHeadPosition(); pos != NULL; )
	{
		if(bSameCat)
		{
			CUpDownClient* pClient = m_A4AFsrcList.GetNext(pos);
			if (pClient->m_pReqPartFile->GetCatID() == GetCatID())
				pClient->SwapToAnotherFile(this);
		}
		else
		{
			m_A4AFsrcList.GetNext(pos)->SwapToAnotherFile(this);
		}

	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 1. CS method to prevent a problem by removing(adding) in the list
// 2. since the CS allows us to add clients safely, it's better to add the client to the head
// this will speed up first respose to DL invitation
void CPartFile::AddDownloadingSource(CUpDownClient* client)
{
	EMULE_TRY

	EnterCriticalSection(&m_csDownloadingSourceList);

	POSITION pos = m_downloadingSourceList.Find(client);

	if (pos == NULL)
	{
		m_downloadingSourceList.AddHead(client);
	}
	LeaveCriticalSection(&m_csDownloadingSourceList);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::RemoveDownloadingSource(CUpDownClient* client)
{
	EMULE_TRY

	EnterCriticalSection(&m_csDownloadingSourceList);

	POSITION pos = m_downloadingSourceList.Find(client);

	if (pos != NULL)
	{
		m_downloadingSourceList.RemoveAt(pos);
	}
	LeaveCriticalSection(&m_csDownloadingSourceList);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
byte* CPartFile::MMCreatePartStatus()
{
	EMULE_TRY

//	Create partstatus + info in mobilemule protocol specs
//	result needs to be deleted[] | slow, but not timecritical
	byte * result = new byte[GetPartCount() + 1];
	for (int i = 0; i < GetPartCount(); i++)
	{
		result[i] = 0;
		if (IsPartComplete(i))
		{
			result[i] = 1;
		}
		else
		{
			if (IsComplete(i * PARTSIZE + (0 * (PARTSIZE / 3)), ((i * PARTSIZE) + (1 * (PARTSIZE / 3))) - 1))
				result[i] += 2;
			if (IsComplete(i * PARTSIZE + (1 * (PARTSIZE / 3)), ((i * PARTSIZE) + (2 * (PARTSIZE / 3))) - 1))
				result[i] += 4;
			if (IsComplete(i * PARTSIZE + (2 * (PARTSIZE / 3)), ((i * PARTSIZE) + (3 * (PARTSIZE / 3))) - 1))
				result[i] += 8;
			byte freq = (byte)m_srcPartFrequencies[i];
			if (freq > 44)
				freq = 44;
			freq = (freq + 2) / 3;
			freq <<= 4;
			result[i] += freq;
		}
	}
	return result;

	EMULE_CATCH

	return NULL;
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	#zegzav:pastcomment
void CPartFile::AddPastComment(CUpDownClient *pClient)
{
	EMULE_TRY

	if ((!pClient) || (pClient->m_pReqPartFile != this))
		return ;

	if ((pClient->GetFileComment().IsEmpty()) && (pClient->GetFileRating() == PF_RATING_NONE))
		return ;

	m_pastCommentList.Add(pClient->GetUserHash(), pClient->GetUserName(), GetFileName(), pClient->GetFileComment(), pClient->GetFileRating());

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::RemovePastComment(CUpDownClient *pClient, bool bRestore)
{
	EMULE_TRY

	if (!pClient)
		return;

	if (bRestore)
	{
		CString	strPastComment;
		EnumPartFileRating	ePastRating;

		if ( (m_pastCommentList.GetCommentRating(pClient->GetUserHash(), &strPastComment, &ePastRating))
		     && (pClient->GetFileComment().IsEmpty())
		     && (pClient->GetFileRating() == PF_RATING_NONE) )
		{
		//	Restore past comment
			pClient->SetFileComment(strPastComment);
			pClient->SetFileRating(ePastRating);
		}
	}

	m_pastCommentList.Remove(pClient->GetUserHash());

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
EnumCategories CPartFile::GetCatID() const
{
//	If catID is valid... (valid range is 1..CAT_PREDEFINED-1 but a
//	real check would have to scan the category map. This isn't the
//	best place to do that.
#ifndef VS2002 // In VS2002 the const cast operator causes an ambiguity
	if (static_cast<const _EnumCategories>(m_eCategoryID) >= CAT_PREDEFINED)
#else
	if (const_cast<CPartFile*>(this)->m_eCategoryID >= CAT_PREDEFINED)
#endif
		return CAT_NONE;
	else
		return m_eCategoryID;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::SetCatID(EnumCategories eCatID)
{
	m_eCategoryID = eCatID;

//	Set new prio
	if (IsPartFile())
	{
		int	iPriority = 0;

		if (eCatID != CAT_NONE)
		{
			iPriority = CCat::GetCatByID(GetCatID())->GetPriority();
		}
		switch (iPriority)
		{
			case 0:
			{
				break;
			}
			case 1:
			{
				SetAutoPriority(false);
				SetPriority(PR_LOW, false);
				break;
			}
			case 2:
			{
				SetAutoPriority(false);
				SetPriority(PR_NORMAL, false);
				break;
			}
			case 3:
			{
				SetAutoPriority(false);
				SetPriority(PR_HIGH, false);
				break;
			}
			case 4:
			{
				SetAutoPriority(true);
				SetPriority(PR_HIGH, false);
				break;
			}
		}

		if (IsPartFile() && !m_strTempDir.IsEmpty())
			SavePartFile();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::SetStatus(EnumPartFileStatuses in)
{
	m_eStatus = in;

	UpdateGUIAfterStateChange();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::UpdateGUIAfterStateChange()
{
	EnumCategories	eCurCat = g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.GetCurTabCat();

//	Changes in file status can affect whether the file belongs to a predefined category or not
	if (eCurCat >= CAT_PREDEFINED)
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.ChangeCategoryByID(eCurCat);
	else
		UpdateDisplayedInfo(true);

//	If we're displaying file counts in the tabs, update them to reflect status changes
	if (g_eMuleApp.m_pGlobPrefs->ShowCatTabInfo())
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateCatTabTitles();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetRealFileSize() returns real file size on disk with respect to NTFS compression and/or NTFS sparse files
uint64 CPartFile::GetRealFileSize()
{
	return ::GetDiskFileSize(GetFilePath());
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsStalled() performs obaldin's stall check
bool CPartFile::IsStalled() const
{
	bool	bIsStalled = false;

	CTimeSpan	ts = CTime::GetCurrentTime() - GetLastDownTransfer();

//	If WARN_PERIOD_OF_NO_PROGRESS days have gone by without a single download...
//	(??? or maybe sooner if there are no needed parts available?)
	if (ts.GetTotalHours() >= 24 * WARN_PERIOD_OF_NO_PROGRESS)
		bIsStalled = true;

	return bIsStalled;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	AllocateNeededSpace() is the preallocate function used to allocate the space needed by a file.
//	It uses a thread to let eMule still work and not drop sources while generating the file.
void CPartFile::AllocateNeededSpace()
{
	HANDLE	lTh;
	DWORD	lThID;
	m_bIsPreallocated = true;

	PAThreadParam *tp = new PAThreadParam;

	tp->cPartFile = this;
	tp->iPosition = GetFileSize() - 1;

	lTh = CreateThread(NULL, 0L, AllocateNeededSpaceProc, tp, 0, &lThID);
	CloseHandle(lTh);

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	AllocateNeededSpaceProc() is the thread created by AllocateNeededSpace().
//	It receives a pointer to 'm_hPartFileWrite' and the filesize-1 (where it has to write the bogus data to allocate space.
DWORD WINAPI CPartFile::AllocateNeededSpaceProc(LPVOID lpParameter)
{
	PAThreadParam * tp = (PAThreadParam*)lpParameter;

	try
	{
		tp->cPartFile->m_hPartFileWrite.Seek(tp->iPosition, CFile::begin);
		tp->cPartFile->m_hPartFileWrite.Write("1", 1);
		tp->cPartFile->m_hPartFileWrite.Flush();

		tp->cPartFile->SaveSettingsFile();
	}
	catch (CFileException * error)
	{
		AddDebugLogLine( RGB_LOG_ERROR_TXT _T("Preallocate for file %s: error %s"),
						 tp->cPartFile->GetFileName(), GetErrorMessage(error) );

		tp->cPartFile->m_bIsPreallocated = false;
		error->Delete();
	}

	delete tp;
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsLastBlockComplete() returns status of last block of file (if downloaded the file was already allocated).
bool CPartFile::IsLastBlockComplete()
{
	return IsComplete(GetFileSize() - GetLastBlockSize(), GetFileSize() - 1);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	ClosePartFile() closes all .part file handles
void CPartFile::ClosePartFile()
{
	if (m_hPartFileWrite.m_hFile != INVALID_HANDLE_VALUE)
	{
		m_hPartFileRead.Close();
		m_hPartFileWrite.Close();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	ReadFileForUpload() opens known file and read information in circular way.
//		Params:
//			dwOffset      - file offset to start reading from;
//			dwBytesToRead - number of bytes to read;
//			pbyteBuffer   - buffer to receive data.
//		Return:
//			< 0 in case of error, > 0 - information was read from a part file, else 0.
int CPartFile::ReadFileForUpload(uint32 dwOffset, uint32 dwBytesToRead, byte *pbyteBuffer)
{
	int	iRc = -2;

	EMULE_TRY

//	Synchronization with completion thread
//	will be unlocked on exit
	CSingleLock lockComplete(&m_csFileCompletion, TRUE);

	if (m_eStatus != PS_COMPLETE)
	{
	//	This file isn't complete yet
		m_hPartFileRead.Seek(dwOffset, CFile::begin);

		if (uint32 dwDone = m_hPartFileRead.Read(pbyteBuffer, dwBytesToRead) != dwBytesToRead)
		{
			m_hPartFileRead.SeekToBegin();
			m_hPartFileRead.Read(pbyteBuffer + dwDone, dwBytesToRead - dwDone);
		}
		iRc = 1;
	}
	else
	{
	//	This file is complete and is located in destination directory
		iRc = (::CircularFileRead(m_strFullName, dwOffset, dwBytesToRead, pbyteBuffer)) ? 0 : -1;
	}

	EMULE_CATCH

	return iRc;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetFirstLastChunk4Preview() sets Movie Preview Mode by checking media type and needed index size (last 1% of file)
void CPartFile::GetFirstLastChunk4Preview()
{
	EMULE_TRY

	if (GetMovieMode())
		SetMovieMode(0);
	else
	{
		if (IsAviMovie())
		{
			SetMovieMode((GetLastPartSize() < (GetFileSize() / 100)) ? 7 : 1);
		}
		else if (IsMpgMovie())
		{
			if (GetFileSize() < 209715200)
				SetMovieMode(4);
			else
				SetMovieMode(5);
		}
		else
			SetMovieMode(0);
	}
	SaveSettingsFile();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetGapListCopy() returns a protected copy of the gap list
void CPartFile::GetGapListCopy(CArray<Gap_Struct, Gap_Struct> *pGapArray)
{
	Gap_Struct	*pGap;

	CSingleLock lockGap(&m_csGapListAndPartStatus, TRUE);	// will be unlocked on the exit

	pGapArray->SetSize(gaplist.GetCount(), 0);
	pGap = pGapArray->GetData();

	for (POSITION pos = gaplist.GetHeadPosition(); pos != NULL;)
	{
		*pGap++ = *gaplist.GetNext(pos);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPartFile::SetFileName(const CString& NewName, bool bClearName)
{
//	set the name & extension
	CKnownFile::SetFileName(NewName, bClearName);

	CString strExt = GetFileExtension();
	static const TCHAR s_apcPreviewableExt[][4] =
	{
		_T("zip"), _T("rar")
	};
	static const TCHAR s_apcAviMovieExt[][5] =
	{
		_T("avi"), _T("ogm"), _T("mkv"), _T("wmv"), _T("divx"), _T("xvid")
	};
	static const TCHAR s_apcMpgMovieExt[][5] =
	{
		_T("mpg"), _T("mpeg"), _T("vob"), _T("bin"), _T("mpe"), _T("dat")
	};
	static const TCHAR s_apcMpgAudioExt[][4] =
	{
		_T("mp3"), _T("mp2"), _T("mpa")
	};

//	Check previewable status
	m_bIsPreviewableArchive = false;
	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_apcPreviewableExt); ui++)
	{
		if (strExt.Compare(reinterpret_cast<const TCHAR*>(&s_apcPreviewableExt[ui])) == 0)
		{
			m_bIsPreviewableArchive = true;
			break;
		}
	}

//	Check movie status
	m_bIsMovie = (ED2KFT_VIDEO == GetFileType() || strExt == _T("bin"));

//	Check AVI movie status
	m_bIsAviMovie = false;
	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_apcAviMovieExt); ui++)
	{
		if (strExt.Compare(reinterpret_cast<const TCHAR*>(&s_apcAviMovieExt[ui])) == 0)
		{
			m_bIsAviMovie = true;
			break;
		}
	}

//	Check MPG movie status
	m_bIsMpgMovie= false;
	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_apcMpgMovieExt); ui++)
	{
		if (strExt.Compare(reinterpret_cast<const TCHAR*>(&s_apcMpgMovieExt[ui])) == 0)
		{
			m_bIsMpgMovie = true;
			break;
		}
	}

//	Check MPG movie status
	m_bIsMpgAudio = false;
	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_apcMpgAudioExt); ui++)
	{
		if (strExt.Compare(reinterpret_cast<const TCHAR*>(&s_apcMpgAudioExt[ui])) == 0)
		{
			m_bIsMpgAudio = true;
			break;
		}
	}
}
